<?php
    /*
     * $Id: configtest.php,v 1.5 2004/07/05 07:19:47 rjl Exp $
     *
     * MAIA MAILGUARD LICENSE v.1.0
     *
     * Copyright 2004 by Robert LeBlanc <rjl@renaissoft.com>
     * All rights reserved.
     *
     * PREAMBLE
     *
     * This License is designed for users of Maia Mailguard
     * ("the Software") who wish to support the Maia Mailguard project by
     * leaving "Maia Mailguard" branding information in the HTML output
     * of the pages generated by the Software, and providing links back
     * to the Maia Mailguard home page.  Users who wish to remove this
     * branding information should contact the copyright owner to obtain
     * a Rebranding License.
     *
     * DEFINITION OF TERMS
     *
     * The "Software" refers to Maia Mailguard, including all of the
     * associated PHP, Perl, and SQL scripts, documentation files, graphic
     * icons and logo images.
     *
     * GRANT OF LICENSE
     *
     * Redistribution and use in source and binary forms, with or without
     * modification, are permitted provided that the following conditions
     * are met:
     *
     * 1. Redistributions of source code must retain the above copyright
     *    notice, this list of conditions and the following disclaimer.
     *
     * 2. Redistributions in binary form must reproduce the above copyright
     *    notice, this list of conditions and the following disclaimer in the
     *    documentation and/or other materials provided with the distribution.
     *
     * 3. The end-user documentation included with the redistribution, if
     *    any, must include the following acknowledgment:
     *
     *    "This product includes software developed by Robert LeBlanc
     *    <rjl@renaissoft.com>."
     *
     *    Alternately, this acknowledgment may appear in the software itself,
     *    if and wherever such third-party acknowledgments normally appear.
     *
     * 4. At least one of the following branding conventions must be used:
     *
     *    a. The Maia Mailguard logo appears in the page-top banner of
     *       all HTML output pages in an unmodified form, and links
     *       directly to the Maia Mailguard home page; or
     *
     *    b. The "Powered by Maia Mailguard" graphic appears in the HTML
     *       output of all gateway pages that lead to this software,
     *       linking directly to the Maia Mailguard home page; or
     *
     *    c. A separate Rebranding License is obtained from the copyright
     *       owner, exempting the Licensee from 4(a) and 4(b), subject to
     *       the additional conditions laid out in that license document.
     *
     * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS
     * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
     * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
     * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
     * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
     * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
     * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
     * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
     * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
     * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
     * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
     *
     */

    require_once ("config.php");

    include_once ("PEAR/Remote.php");      // PEAR::Remote
    include_once ("PEAR/Registry.php");    // PEAR::Registry
    include_once ("Mail/mimeDecode.php");  // PEAR::Mail::mimeDecode
    include_once ("DB.php");               // PEAR::DB
    include_once ("DB/Pager.php");         // PEAR::DB_Pager
    include_once ("Log.php");              // PEAR::Log
    include_once ("jpgraph/jpgraph.php");  // JpGraph


    function get_database_type($dbh)
    {
    	return $dbh->phptype;
    }


    function table_exists($dbh, $table_name)
    {
        $select = "SELECT * FROM " . $table_name;
        $sth = $dbh->query($select);
        if (DB::isError($sth)) {
            return false;
        } else {
            $sth->free();
            return true;
        }
    }


    function column_exists($table_columns, $column_name)
    {
        return isset($table_columns[$column_name]);
    }


    function add_column($dbh, $table_name, $column_name, $type, $size, $default, $nulls)
    {
    	$db_type = get_database_type($dbh);
        $alter = "ALTER TABLE " . $table_name . " ADD " . $column_name;
        switch($db_type) {
            case "mysql":
                switch($type) {
                    case "int":
                        $alter .= " INT";
                       break;
                    case "uint":
                        $alter .= " INT UNSIGNED";
                        break;
                    case "char":
                        $alter .= " CHAR(" . $size . ")";
                        break;
                    case "varchar":
                        $alter .= " VARCHAR(" . $size . ")";
                        break;
                    case "float":
                        $alter .= " FLOAT";
                        break;
                    case "timestamp":
                        $alter .= " DATETIME";
                        break;
                    case "text":
                        $alter .= " TEXT";
                        break;
                    case "ltext":
                        $alter .= " LONGTEXT";
                        break;
                    case "mtext":
                        $alter .= " MEDIUMTEXT";
                        break;
                }
                break;
            case "pgsql":
                switch($type) {
                    case "int":
                    case "uint":
                        $alter .= " INTEGER";
                        break;
                    case "char":
                        $alter .= " CHAR(" . $size . ")";
                        break;
                    case "varchar":
                        $alter .= " VARCHAR(" . $size . ")";
                        break;
                    case "float":
                        $alter .= " NUMERIC(" . $size . ")";
                        break;
                    case "timestamp":
                        $alter .= " TIMESTAMP";
                        break;
                    case "text":
                    case "ltext":
                    case "mtext":
                        $alter .= " TEXT";
                        break;
                }
                break;
        }
        if (!empty($default)) {
            $alter .= " DEFAULT '" . $default . "'";
        }
        if (!$nulls) {
            $alter .= " NOT NULL";
        }
        $dbh->query($alter);
    }


    function drop_column($dbh, $table_name, $column_name)
    {
        $alter = "ALTER TABLE " . $table_name . " DROP " . $column_name;
        $dbh->query($alter);
    }


    function index_exists($dbh, $table_name, $index_name)
    {
        $db_type = get_database_type($dbh);
        if ($db_type == "mysql") {

            $select = "SHOW INDEX FROM " . $table_name;
            $sth = $dbh->query($select);
            if (DB::isError($sth)) {
                return false;
            } else {
            	$found = false;
                while ($row = $sth->fetchRow()) {
                    if ($row[2] == $index_name) {
                        $found = true;
                    }
                }
                $sth->free();
                return $found;
            }

        } else if ($db_type == "pgsql") {

            $select = "SELECT i.relname AS indexname " .
                      "FROM pg_class c, pg_class i " .
                      "WHERE ((c.relkind = 'r'::\"char\") " .
                      "AND (i.relkind = 'i'::\"char\") " .
                      "AND (i.relname = ?) " .
                      "AND (c.relname = ?))";
            $sth = $dbh->query($select, array($index_name, $table_name));
            if (DB::isError($sth)) {
                return false;
            } else {
            	$found = false;
                if ($row = $sth->fetchRow()) {
                    $found = true;
                }
                $sth->free();
                return $found;
            }

        }
    }


    function create_index($dbh, $table_name, $index_column, $unique)
    {
        $create = "CREATE ";
        if ($unique) {
            $create .= "UNIQUE ";
        }
        $create .= "INDEX " . $table_name . "_idx_" . $index_column . " ";
        $create .= "ON " . $table_name . " (" . $index_column . ")";
        $dbh->query($create);
    }


    function create_table($dbh, $table_name)
    {
    	$db_type = get_database_type($dbh);
    	switch($db_type) {

    	    case "mysql":

    	      	switch($table_name) {

                    case "users":
                        $create = "CREATE TABLE users (id int unsigned DEFAULT '0' NOT NULL auto_increment, " .
                                  "priority int DEFAULT '7' NOT NULL, " .
                                  "policy_id int unsigned DEFAULT '1' NOT NULL, " .
                                  "email varchar(255) NOT NULL, " .
                                  "maia_user_id int unsigned NOT NULL, " .
                                  "maia_domain_id int unsigned NOT NULL, " .
                                  "PRIMARY KEY (id), " .
                                  "KEY email (email))";
                        break;

                    case "mailaddr":
                        $create = "CREATE TABLE mailaddr (id int unsigned DEFAULT '0' NOT NULL auto_increment, " .
                                  "priority int DEFAULT '7' NOT NULL, " .
                                  "email varchar(255) NOT NULL, " .
                                  "PRIMARY KEY (id), " .
                                  "KEY email (email))";
                        break;

                    case "wblist":
                        $create = "CREATE TABLE wblist (rid int unsigned NOT NULL, " .
                                  "sid int unsigned NOT NULL, " .
                                  "wb char(1) NOT NULL, " .
                                  "PRIMARY KEY (rid,sid))";
                        break;

                    case "policy":
                        $create = "CREATE TABLE policy (id int unsigned DEFAULT '0' NOT NULL auto_increment, " .
                                  "policy_name varchar(255), " .
                                  "virus_lover char(1) DEFAULT 'Y', " .
                                  "spam_lover char(1) DEFAULT 'Y', " .
                                  "banned_files_lover char(1) DEFAULT 'Y', " .
                                  "bad_header_lover char(1) DEFAULT 'Y', " .
                                  "bypass_virus_checks char(1) DEFAULT 'Y', " .
                                  "bypass_spam_checks char(1) DEFAULT 'Y', " .
                                  "bypass_banned_checks char(1) DEFAULT 'Y', " .
                                  "bypass_header_checks char(1) DEFAULT 'Y', " .
                                  "discard_viruses char(1) DEFAULT 'N', " .
                                  "discard_spam char(1) DEFAULT 'N', " .
                                  "discard_banned_files char(1) DEFAULT 'N', " .
                                  "discard_bad_headers char(1) DEFAULT 'N', " .
                                  "spam_modifies_subj char(1) DEFAULT 'N', " .
                                  "spam_quarantine_to varchar(64) DEFAULT NULL, " .
                                  "spam_tag_level float DEFAULT '999', " .
                                  "spam_tag2_level float DEFAULT '999', " .
                                  "spam_kill_level float DEFAULT '999', " .
                                  "PRIMARY KEY (id))";
                        break;

    		    case "maia_config":
    		        $create = "CREATE TABLE maia_config (id	int unsigned DEFAULT '0' NOT NULL, " .
                                  "enable_user_autocreation char(1) DEFAULT 'N' NOT NULL, " .
                                  "enable_false_negative_management char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_stats_tracking char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_virus_scanning char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_spam_filtering char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_banned_files_checking	char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_bad_header_checking char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_charts char(1) DEFAULT 'N' NOT NULL, " .
                                  "enable_spamtraps char(1) DEFAULT 'N' NOT NULL, " .
                                  "enable_stats_reporting char(1) DEFAULT 'N' NOT NULL, " .
                                  "enable_address_linking char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_privacy_invasion char(1) DEFAULT 'N' NOT NULL, " .
                                  "enable_username_changes char(1) DEFAULT 'Y' NOT NULL, " .
                                  "internal_auth char(1) DEFAULT 'N' NOT NULL, " .
                                  "system_default_user_is_local char(1) DEFAULT 'Y' NOT NULL, " .
                                  "user_virus_scanning char(1) DEFAULT 'Y' NOT NULL, " .
                                  "user_spam_filtering char(1) DEFAULT 'Y' NOT NULL, " .
                                  "user_banned_files_checking char(1) DEFAULT 'Y' NOT NULL, " .
                                  "user_bad_header_checking char(1) DEFAULT 'Y' NOT NULL, " .
                                  "admin_email varchar(255), " .
                                  "expiry_period int unsigned DEFAULT '30', " .
                                  "ham_cache_expiry_period int unsigned DEFAULT '5', " .
                                  "reminder_threshold_count int unsigned DEFAULT '100', " .
                                  "reminder_threshold_size int unsigned DEFAULT '500000', " .
                                  "reminder_template_file varchar(255) DEFAULT 'reminder.tpl', " .
                                  "reminder_login_url varchar(255), " .
                                  "newuser_template_file varchar(255) DEFAULT 'newuser.tpl', " .
                                  "smtp_server varchar(255) DEFAULT 'localhost', " .
                                  "smtp_port int unsigned DEFAULT '10025', " .
                                  "currency_label varchar(15) DEFAULT '$', " .
                                  "bandwidth_cost float DEFAULT '0.0' NOT NULL, " .
                                  "chart_ham_colour varchar(32) DEFAULT '#DDDDB7', " .
                                  "chart_spam_colour varchar(32) DEFAULT '#FFAAAA', " .
                                  "chart_virus_colour varchar(32) DEFAULT '#CCFFCC', " .
                                  "chart_fp_colour varchar(32) DEFAULT '#C4CA73', " .
                                  "chart_fn_colour varchar(32) DEFAULT '#FF7575', " .
                                  "chart_suspected_ham_colour varchar(32) DEFAULT '#FFFFB7', " .
                                  "chart_suspected_spam_colour varchar(32) DEFAULT '#FFCCCC', " .
                                  "chart_wl_colour varchar(32) DEFAULT '#eeeeee', " .
                                  "chart_bl_colour varchar(32) DEFAULT '#888888', " .
                                  "chart_background_colour varchar(32) DEFAULT '#B0ECFF', " .
                                  "chart_font_colour varchar(32) DEFAULT '#3D3D50', " .
                                  "chart_autogeneration_interval int unsigned DEFAULT '60', " .
                                  "banner_title varchar(255) DEFAULT 'Maia Mailguard', " .
                                  "use_icons char(1) DEFAULT 'Y' NOT NULL, " .
                                  "use_logo char(1) DEFAULT 'Y' NOT NULL, " .
                                  "logo_url varchar(255) DEFAULT 'http://www.renaissoft.com/maia/', " .
                                  "logo_file varchar(255) DEFAULT 'images/maia-logotoolbar.gif', " .
                                  "logo_alt_text varchar(255) DEFAULT 'Maia Mailguard Home Page', " .
                                  "virus_info_url varchar(255) DEFAULT 'http://www.google.com/search?q=%%VIRUSNAME%%+virus+information', " .
                                  "virus_lookup	varchar(20) DEFAULT 'google', " .
                                  "primary_report_server varchar(255) DEFAULT 'maia.renaissoft.com', " .
                                  "primary_report_port int unsigned DEFAULT '443', " .
                                  "secondary_report_server varchar(255), " .
                                  "secondary_report_port int unsigned DEFAULT '443', " .
                                  "reporter_sitename varchar(255), " .
                                  "reporter_username varchar(50), " .
                                  "reporter_password varchar(50), " .
                                  "size_limit int unsigned DEFAULT '1000000', " .
                                  "oversize_policy char(1) DEFAULT 'B' NOT NULL, " .
                                  "sa_score_set	int unsigned DEFAULT '0' NOT NULL, " .
                                  "key_file varchar(255) DEFAULT 'blowfish.key', " .
                                  "PRIMARY KEY (id))";
    	                break;

    	            case "maia_languages":
    	                $create = "CREATE TABLE maia_languages (id int unsigned DEFAULT '0' NOT NULL auto_increment, " .
                                  "language_name varchar(100) NOT NULL, " .
                                  "abbreviation char(2) NOT NULL, " .
                                  "installed char(1) DEFAULT 'N' NOT NULL, " .
                                  "PRIMARY KEY (id), " .
                                  "KEY abbreviation (abbreviation))";
    	                break;

    	            case "maia_users":
    	                $create = "CREATE TABLE maia_users (id int unsigned DEFAULT '0' NOT NULL auto_increment, " .
                                  "user_name varchar(255) NOT NULL, " .
                                  "user_level char(1) DEFAULT 'U' NOT NULL, " .
                                  "reminders char(1) DEFAULT 'Y' NOT NULL, " .
                                  "charts char(1) DEFAULT 'N' NOT NULL, " .
                                  "primary_email_id int unsigned DEFAULT '0' NOT NULL, " .
                                  "language varchar(10) DEFAULT 'en' NOT NULL, " .
                                  "charset varchar(20) DEFAULT 'ISO-8859-1' NOT NULL, " .
                                  "spamtrap char(1) DEFAULT 'N' NOT NULL, " .
                                  "password varchar(32), " .
                                  "auto_whitelist char(1) DEFAULT 'Y' NOT NULL, " .
                                  "items_per_page int unsigned DEFAULT '50' NOT NULL, " .
                                  "spam_quarantine_sort char(2) DEFAULT 'XA' NOT NULL, " .
                                  "virus_quarantine_sort char(2) DEFAULT 'DA' NOT NULL, " .
                                  "header_quarantine_sort char(2) DEFAULT 'DA' NOT NULL, " .
                                  "attachment_quarantine_sort char(2) DEFAULT 'DA' NOT NULL, " .
                                  "ham_cache_sort char(2) DEFAULT 'XD' NOT NULL, " .
                                  "discard_ham char(1) DEFAULT 'N' NOT NULL, " .
                                  "PRIMARY KEY (id), " .
                                  "KEY user_name (user_name))";
    	                break;

    	            case "maia_domains":
    	                $create = "CREATE TABLE maia_domains (id int unsigned DEFAULT '0' NOT NULL auto_increment, " .
                                  "domain varchar(255) NOT NULL, " .
                                  "reminders char(1) DEFAULT 'N' NOT NULL, " .
                                  "charts char(1) DEFAULT 'N' NOT NULL, " .
                                  "enable_user_autocreation char(1) DEFAULT 'Y' NOT NULL, " .
                                  "language varchar(10) DEFAULT 'en' NOT NULL, " .
                                  "charset varchar(20) DEFAULT 'ISO-8859-1' NOT NULL, " .
                                  "PRIMARY KEY (id), " .
                                  "KEY domain (domain))";
    	                break;

    	            case "maia_domain_admins":
    	                $create = "CREATE TABLE maia_domain_admins (domain_id int unsigned NOT NULL, " .
                                  "admin_id int unsigned NOT NULL, " .
                                  "PRIMARY KEY (domain_id, admin_id))";
    	                break;

    	            case "maia_stats":
    	                $create = "CREATE TABLE maia_stats (user_id int unsigned DEFAULT '0' NOT NULL, " .
                                  "oldest_suspected_ham_date datetime, " .
                                  "newest_suspected_ham_date datetime, " .
                                  "smallest_suspected_ham_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "largest_suspected_ham_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_suspected_ham_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "lowest_suspected_ham_score float DEFAULT '0.0' NOT NULL, " .
                                  "highest_suspected_ham_score float DEFAULT '0.0' NOT NULL, " .
                                  "total_suspected_ham_score float DEFAULT '0.0' NOT NULL, " .
                                  "total_suspected_ham_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "oldest_ham_date datetime, " .
                                  "newest_ham_date datetime, " .
                                  "smallest_ham_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "largest_ham_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_ham_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "lowest_ham_score float DEFAULT '0.0' NOT NULL, " .
                                  "highest_ham_score float DEFAULT '0.0' NOT NULL, " .
                                  "total_ham_score float DEFAULT '0.0' NOT NULL, " .
                                  "total_ham_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "oldest_wl_date datetime, " .
                                  "newest_wl_date datetime, " .
                                  "smallest_wl_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "largest_wl_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_wl_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_wl_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "oldest_bl_date datetime, " .
                                  "newest_bl_date datetime, " .
                                  "smallest_bl_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "largest_bl_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_bl_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_bl_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "oldest_suspected_spam_date datetime, " .
                                  "newest_suspected_spam_date datetime, " .
                                  "smallest_suspected_spam_size	int unsigned DEFAULT '0' NOT NULL, " .
                                  "largest_suspected_spam_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_suspected_spam_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "lowest_suspected_spam_score float DEFAULT '0.0' NOT NULL, " .
                                  "highest_suspected_spam_score float DEFAULT '0.0' NOT NULL, " .
                                  "total_suspected_spam_score float DEFAULT '0.0' NOT NULL, " .
                                  "total_suspected_spam_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "oldest_fp_date datetime, " .
                                  "newest_fp_date datetime, " .
                                  "smallest_fp_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "largest_fp_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_fp_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "lowest_fp_score float DEFAULT '0.0' NOT NULL, " .
                                  "highest_fp_score float DEFAULT '0.0' NOT NULL, " .
                                  "total_fp_score float DEFAULT '0.0' NOT NULL, " .
                                  "total_fp_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "oldest_fn_date datetime, " .
                                  "newest_fn_date datetime, " .
                                  "smallest_fn_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "largest_fn_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_fn_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "lowest_fn_score float DEFAULT '0.0' NOT NULL, " .
                                  "highest_fn_score float DEFAULT '0.0' NOT NULL, " .
                                  "total_fn_score float DEFAULT '0.0' NOT NULL, " .
                                  "total_fn_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "oldest_spam_date datetime, " .
                                  "newest_spam_date datetime, " .
                                  "smallest_spam_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "largest_spam_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_spam_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "lowest_spam_score float DEFAULT '0.0' NOT NULL, " .
                                  "highest_spam_score float DEFAULT '0.0' NOT NULL, " .
                                  "total_spam_score float DEFAULT '0.0' NOT NULL, " .
                                  "total_spam_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "oldest_virus_date datetime, " .
                                  "newest_virus_date datetime, " .
                                  "smallest_virus_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "largest_virus_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_virus_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_virus_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "oldest_bad_header_date datetime, " .
                                  "newest_bad_header_date datetime, " .
                                  "smallest_bad_header_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "largest_bad_header_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_bad_header_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_bad_header_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "oldest_banned_file_date datetime, " .
                                  "newest_banned_file_date datetime, " .
                                  "smallest_banned_file_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "largest_banned_file_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_banned_file_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_banned_file_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "oldest_oversized_date datetime, " .
                                  "newest_oversized_date datetime, " .
                                  "smallest_oversized_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "largest_oversized_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_oversized_size	int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_oversized_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "PRIMARY KEY (user_id))";
    	                break;

    	            case "maia_mail":
    	                $create = "CREATE TABLE maia_mail (id int unsigned DEFAULT '0' NOT NULL auto_increment, " .
                                  "received_date datetime NOT NULL, " .
                                  "size int unsigned NOT NULL, " .
                                  "sender_email varchar(255) NOT NULL, " .
                                  "envelope_to text NOT NULL, " .
                                  "subject varchar(255) NOT NULL, " .
                                  "contents longtext NOT NULL, " .
                                  "score float, " .
                                  "PRIMARY KEY (id))";
    	                break;

    	            case "maia_mail_recipients":
    	                $create = "CREATE TABLE maia_mail_recipients (mail_id int unsigned NOT NULL, " .
                                  "recipient_id int unsigned NOT NULL, " .
                                  "type char(1) NOT NULL, " .
                                  "PRIMARY KEY (mail_id, recipient_id))";
    	                break;

    	            case "maia_viruses":
    	                $create = "CREATE TABLE maia_viruses (id int unsigned DEFAULT '0' NOT NULL auto_increment, " .
                                  "virus_name varchar(255) NOT NULL, " .
                                  "count int unsigned DEFAULT '0' NOT NULL, " .
                                  "PRIMARY KEY (id), " .
                                  "KEY virus_name (virus_name))";
    	                break;

    	            case "maia_virus_aliases":
    	                $create = "CREATE TABLE maia_virus_aliases (virus_id int unsigned NOT NULL, " .
                                  "virus_alias varchar(255) NOT NULL, " .
                                  "PRIMARY KEY (virus_id, virus_alias))";
    	                break;

    	            case "maia_viruses_detected":
    	                $create = "CREATE TABLE maia_viruses_detected (mail_id int unsigned NOT NULL, " .
                                  "virus_id int unsigned NOT NULL, " .
                                  "PRIMARY KEY (mail_id, virus_id))";
    	                break;

    	            case "maia_sa_rules":
    	                $create = "CREATE TABLE maia_sa_rules (id int unsigned DEFAULT '0' NOT NULL auto_increment, " .
                                  "rule_name varchar(255) NOT NULL, " .
                                  "rule_description varchar(255) DEFAULT '' NOT NULL, " .
                                  "rule_score_0 float DEFAULT '1.0' NOT NULL, " .
                                  "rule_score_1 float DEFAULT '1.0' NOT NULL, " .
                                  "rule_score_2	float DEFAULT '1.0' NOT NULL, " .
                                  "rule_score_3	float DEFAULT '1.0' NOT NULL, " .
                                  "rule_count int unsigned DEFAULT '0' NOT NULL, " .
                                  "PRIMARY KEY (id), " .
                                  "KEY rule_name (rule_name))";
    	                break;

    	            case "maia_sa_rules_triggered":
    	                $create = "CREATE TABLE maia_sa_rules_triggered (mail_id int unsigned NOT NULL, " .
                                  "rule_id int unsigned NOT NULL, " .
                                  "rule_score float default '0.0' NOT NULL, " .
                                  "PRIMARY KEY (mail_id, rule_id))";
    	                break;

    	            case "maia_banned_attachments_found":
    	                $create = "CREATE TABLE maia_banned_attachments_found (mail_id int unsigned NOT NULL, " .
                                  "file_name varchar(255) NOT NULL, " .
                                  "file_type varchar(20) DEFAULT 'Unknown' NOT NULL, " .
                                  "PRIMARY KEY (mail_id, file_name))";
    	                break;

    	            case "maia_stats_history":
    	                $create = "CREATE TABLE maia_stats_history (id int unsigned DEFAULT '0' NOT NULL auto_increment, " .
                                  "user_id int unsigned NOT NULL, " .
                                  "type char(1) DEFAULT 'H' NOT NULL, " .
                                  "taken_at datetime NOT NULL, " .
                                  "total_ham_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_ham_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_spam_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_spam_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_virus_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_virus_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_fp_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_fp_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_fn_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_fn_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_banned_file_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_banned_file_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_bad_header_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_bad_header_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_wl_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_wl_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_bl_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_bl_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_oversized_items int unsigned DEFAULT '0' NOT NULL, " .
                                  "total_oversized_size int unsigned DEFAULT '0' NOT NULL, " .
                                  "PRIMARY KEY (id))";
    	                break;

    	        }
    	        break;

    	    case "pgsql":

    	        switch($table_name) {

                    case "users":
                        $create = "CREATE TABLE users (id SERIAL PRIMARY KEY, " .
                                  "priority integer DEFAULT '7' NOT NULL, " .
                                  "policy_id integer DEFAULT '1' NOT NULL, " .
                                  "email varchar(255) NOT NULL UNIQUE, " .
                                  "maia_user_id integer NOT NULL, " .
                                  "maia_domain_id integer NOT NULL)";
                        break;

                    case "mailaddr":
                        $create = "CREATE TABLE mailaddr (id SERIAL PRIMARY KEY, " .
                                  "priority integer DEFAULT '7' NOT NULL, " .
                                  "email varchar(255) NOT NULL UNIQUE)";
                        break;

                    case "wblist":
                        $create = "CREATE TABLE wblist (rid integer NOT NULL, " .
                                  "sid integer NOT NULL, " .
                                  "wb char(1) NOT NULL, " .
                                  "PRIMARY KEY (rid,sid))";
                        break;

                    case "policy":
                        $create = "CREATE TABLE policy (id SERIAL PRIMARY KEY, " .
                                  "policy_name varchar(255), " .
                                  "virus_lover char(1) DEFAULT 'Y', " .
                                  "spam_lover char(1) DEFAULT 'Y', " .
                                  "banned_files_lover char(1) DEFAULT 'Y', " .
                                  "bad_header_lover char(1) DEFAULT 'Y', " .
                                  "bypass_virus_checks char(1) DEFAULT 'Y', " .
                                  "bypass_spam_checks char(1) DEFAULT 'Y', " .
                                  "bypass_banned_checks char(1) DEFAULT 'Y', " .
                                  "bypass_header_checks char(1) DEFAULT 'Y', " .
                                  "discard_viruses char(1) DEFAULT 'N', " .
                                  "discard_spam char(1) DEFAULT 'N', " .
                                  "discard_banned_files char(1) DEFAULT 'N', " .
                                  "discard_bad_headers char(1) DEFAULT 'N', " .
                                  "spam_modifies_subj char(1) DEFAULT 'N', " .
                                  "spam_quarantine_to varchar(64) DEFAULT NULL, " .
                                  "spam_tag_level numeric(7,3) DEFAULT '999', " .
                                  "spam_tag2_level numeric(7,3) DEFAULT '999', " .
                                  "spam_kill_level numeric(7,3) DEFAULT '999')";
                        break;

    	            case "maia_config":
    		        $create = "CREATE TABLE maia_config (id	integer DEFAULT '0' PRIMARY KEY, " .
                                  "enable_user_autocreation char(1) DEFAULT 'N' NOT NULL, " .
                                  "enable_false_negative_management char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_stats_tracking char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_virus_scanning char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_spam_filtering char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_banned_files_checking	char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_bad_header_checking char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_charts char(1) DEFAULT 'N' NOT NULL, " .
                                  "enable_spamtraps char(1) DEFAULT 'N' NOT NULL, " .
                                  "enable_stats_reporting char(1) DEFAULT 'N' NOT NULL, " .
                                  "enable_address_linking char(1) DEFAULT 'Y' NOT NULL, " .
                                  "enable_privacy_invasion char(1) DEFAULT 'N' NOT NULL, " .
                                  "enable_username_changes char(1) DEFAULT 'Y' NOT NULL, " .
                                  "internal_auth char(1) DEFAULT 'N' NOT NULL, " .
                                  "system_default_user_is_local char(1) DEFAULT 'Y' NOT NULL, " .
                                  "user_virus_scanning char(1) DEFAULT 'Y' NOT NULL, " .
                                  "user_spam_filtering char(1) DEFAULT 'Y' NOT NULL, " .
                                  "user_banned_files_checking char(1) DEFAULT 'Y' NOT NULL, " .
                                  "user_bad_header_checking char(1) DEFAULT 'Y' NOT NULL, " .
                                  "admin_email varchar(255), " .
                                  "expiry_period integer DEFAULT '30', " .
                                  "ham_cache_expiry_period integer DEFAULT '5', " .
                                  "reminder_threshold_count integer DEFAULT '100', " .
                                  "reminder_threshold_size integer DEFAULT '500000', " .
                                  "reminder_template_file varchar(255) DEFAULT 'reminder.tpl', " .
                                  "reminder_login_url varchar(255), " .
                                  "newuser_template_file varchar(255) DEFAULT 'newuser.tpl', " .
                                  "smtp_server varchar(255) DEFAULT 'localhost', " .
                                  "smtp_port integer DEFAULT '10025', " .
                                  "currency_label varchar(15) DEFAULT '$', " .
                                  "bandwidth_cost numeric(14,2) DEFAULT '0.0' NOT NULL, " .
                                  "chart_ham_colour varchar(32) DEFAULT '#DDDDB7', " .
                                  "chart_spam_colour varchar(32) DEFAULT '#FFAAAA', " .
                                  "chart_virus_colour varchar(32) DEFAULT '#CCFFCC', " .
                                  "chart_fp_colour varchar(32) DEFAULT '#C4CA73', " .
                                  "chart_fn_colour varchar(32) DEFAULT '#FF7575', " .
                                  "chart_suspected_ham_colour varchar(32) DEFAULT '#FFFFB7', " .
                                  "chart_suspected_spam_colour varchar(32) DEFAULT '#FFCCCC', " .
                                  "chart_wl_colour varchar(32) DEFAULT '#eeeeee', " .
                                  "chart_bl_colour varchar(32) DEFAULT '#888888', " .
                                  "chart_background_colour varchar(32) DEFAULT '#B0ECFF', " .
                                  "chart_font_colour varchar(32) DEFAULT '#3D3D50', " .
                                  "chart_autogeneration_interval integer DEFAULT '60', " .
                                  "banner_title varchar(255) DEFAULT 'Maia Mailguard', " .
                                  "use_icons char(1) DEFAULT 'Y' NOT NULL, " .
                                  "use_logo char(1) DEFAULT 'Y' NOT NULL, " .
                                  "logo_url varchar(255) DEFAULT 'http://www.renaissoft.com/maia/', " .
                                  "logo_file varchar(255) DEFAULT 'images/maia-logotoolbar.gif', " .
                                  "logo_alt_text varchar(255) DEFAULT 'Maia Mailguard Home Page', " .
                                  "virus_info_url varchar(255) DEFAULT 'http://www.google.com/search?q=%%VIRUSNAME%%+virus+information', " .
                                  "virus_lookup	varchar(20) DEFAULT 'google', " .
                                  "primary_report_server varchar(255) DEFAULT 'maia.renaissoft.com', " .
                                  "primary_report_port integer DEFAULT '443', " .
                                  "secondary_report_server varchar(255), " .
                                  "secondary_report_port integer DEFAULT '443', " .
                                  "reporter_sitename varchar(255), " .
                                  "reporter_username varchar(50), " .
                                  "reporter_password varchar(50), " .
                                  "size_limit integer DEFAULT '1000000', " .
                                  "oversize_policy char(1) DEFAULT 'B' NOT NULL, " .
                                  "sa_score_set	integer DEFAULT '0' NOT NULL, " .
                                  "key_file varchar(255) DEFAULT 'blowfish.key')";
    	                break;

    	            case "maia_languages":
    	                $create = "CREATE TABLE maia_languages (id SERIAL PRIMARY KEY, " .
                                  "language_name varchar(100) NOT NULL, " .
                                  "abbreviation char(2) NOT NULL UNIQUE, " .
                                  "installed char(1) DEFAULT 'N' NOT NULL)";
    	                break;

    	            case "maia_users":
    	                $create = "CREATE TABLE maia_users (id SERIAL PRIMARY KEY, " .
                                  "user_name varchar(255) NOT NULL UNIQUE, " .
                                  "user_level char(1) DEFAULT 'U' NOT NULL, " .
                                  "reminders char(1) DEFAULT 'Y' NOT NULL, " .
                                  "charts char(1) DEFAULT 'N' NOT NULL, " .
                                  "primary_email_id integer DEFAULT '0' NOT NULL, " .
                                  "language varchar(10) DEFAULT 'en' NOT NULL, " .
                                  "charset varchar(20) DEFAULT 'ISO-8859-1' NOT NULL, " .
                                  "spamtrap char(1) DEFAULT 'N' NOT NULL, " .
                                  "password varchar(32), " .
                                  "auto_whitelist char(1) DEFAULT 'Y' NOT NULL, " .
                                  "items_per_page integer DEFAULT '50' NOT NULL, " .
                                  "spam_quarantine_sort char(2) DEFAULT 'XA' NOT NULL, " .
                                  "virus_quarantine_sort char(2) DEFAULT 'DA' NOT NULL, " .
                                  "header_quarantine_sort char(2) DEFAULT 'DA' NOT NULL, " .
                                  "attachment_quarantine_sort char(2) DEFAULT 'DA' NOT NULL, " .
                                  "ham_cache_sort char(2) DEFAULT 'XD' NOT NULL, " .
                                  "discard_ham char(1) DEFAULT 'N' NOT NULL)";
    	                break;

    	            case "maia_domains":
    	                $create = "CREATE TABLE maia_domains (id SERIAL PRIMARY KEY, " .
                                  "domain varchar(255) NOT NULL UNIQUE, " .
                                  "reminders char(1) DEFAULT 'N' NOT NULL, " .
                                  "charts char(1) DEFAULT 'N' NOT NULL, " .
                                  "enable_user_autocreation char(1) DEFAULT 'Y' NOT NULL, " .
                                  "language varchar(10) DEFAULT 'en' NOT NULL, " .
                                  "charset varchar(20) DEFAULT 'ISO-8859-1' NOT NULL)";
    	                break;

    	            case "maia_domain_admins":
    	                $create = "CREATE TABLE maia_domain_admins (domain_id integer NOT NULL, " .
                                  "admin_id integer NOT NULL, " .
                                  "PRIMARY KEY (domain_id, admin_id))";
    	                break;

    	            case "maia_stats":
    	                $create = "CREATE TABLE maia_stats (user_id integer DEFAULT '0' NOT NULL, " .
                                  "oldest_suspected_ham_date timestamp, " .
                                  "newest_suspected_ham_date timestamp, " .
                                  "smallest_suspected_ham_size integer DEFAULT '0' NOT NULL, " .
                                  "largest_suspected_ham_size integer DEFAULT '0' NOT NULL, " .
                                  "total_suspected_ham_size integer DEFAULT '0' NOT NULL, " .
                                  "lowest_suspected_ham_score numeric(7,3) DEFAULT '0.0' NOT NULL, " .
                                  "highest_suspected_ham_score numeric(7,3) DEFAULT '0.0' NOT NULL, " .
                                  "total_suspected_ham_score numeric(14,3) DEFAULT '0.0' NOT NULL, " .
                                  "total_suspected_ham_items integer DEFAULT '0' NOT NULL, " .
                                  "oldest_ham_date timestamp, " .
                                  "newest_ham_date timestamp, " .
                                  "smallest_ham_size integer DEFAULT '0' NOT NULL, " .
                                  "largest_ham_size integer DEFAULT '0' NOT NULL, " .
                                  "total_ham_size integer DEFAULT '0' NOT NULL, " .
                                  "lowest_ham_score numeric(7,3) DEFAULT '0.0' NOT NULL, " .
                                  "highest_ham_score numeric(7,3) DEFAULT '0.0' NOT NULL, " .
                                  "total_ham_score numeric(14,3) DEFAULT '0.0' NOT NULL, " .
                                  "total_ham_items integer DEFAULT '0' NOT NULL, " .
                                  "oldest_wl_date timestamp, " .
                                  "newest_wl_date timestamp, " .
                                  "smallest_wl_size integer DEFAULT '0' NOT NULL, " .
                                  "largest_wl_size integer DEFAULT '0' NOT NULL, " .
                                  "total_wl_size integer DEFAULT '0' NOT NULL, " .
                                  "total_wl_items integer DEFAULT '0' NOT NULL, " .
                                  "oldest_bl_date timestamp, " .
                                  "newest_bl_date timestamp, " .
                                  "smallest_bl_size integer DEFAULT '0' NOT NULL, " .
                                  "largest_bl_size integer DEFAULT '0' NOT NULL, " .
                                  "total_bl_size integer DEFAULT '0' NOT NULL, " .
                                  "total_bl_items integer DEFAULT '0' NOT NULL, " .
                                  "oldest_suspected_spam_date timestamp, " .
                                  "newest_suspected_spam_date timestamp, " .
                                  "smallest_suspected_spam_size	integer DEFAULT '0' NOT NULL, " .
                                  "largest_suspected_spam_size integer DEFAULT '0' NOT NULL, " .
                                  "total_suspected_spam_size integer DEFAULT '0' NOT NULL, " .
                                  "lowest_suspected_spam_score numeric(7,3) DEFAULT '0.0' NOT NULL, " .
                                  "highest_suspected_spam_score numeric(7,3) DEFAULT '0.0' NOT NULL, " .
                                  "total_suspected_spam_score numeric(14,3) DEFAULT '0.0' NOT NULL, " .
                                  "total_suspected_spam_items integer DEFAULT '0' NOT NULL, " .
                                  "oldest_fp_date timestamp, " .
                                  "newest_fp_date timestamp, " .
                                  "smallest_fp_size integer DEFAULT '0' NOT NULL, " .
                                  "largest_fp_size integer DEFAULT '0' NOT NULL, " .
                                  "total_fp_size integer DEFAULT '0' NOT NULL, " .
                                  "lowest_fp_score numeric(7,3) DEFAULT '0.0' NOT NULL, " .
                                  "highest_fp_score numeric(7,3) DEFAULT '0.0' NOT NULL, " .
                                  "total_fp_score numeric(14,3) DEFAULT '0.0' NOT NULL, " .
                                  "total_fp_items integer DEFAULT '0' NOT NULL, " .
                                  "oldest_fn_date timestamp, " .
                                  "newest_fn_date timestamp, " .
                                  "smallest_fn_size integer DEFAULT '0' NOT NULL, " .
                                  "largest_fn_size integer DEFAULT '0' NOT NULL, " .
                                  "total_fn_size integer DEFAULT '0' NOT NULL, " .
                                  "lowest_fn_score numeric(7,3) DEFAULT '0.0' NOT NULL, " .
                                  "highest_fn_score numeric(7,3) DEFAULT '0.0' NOT NULL, " .
                                  "total_fn_score numeric(14,3) DEFAULT '0.0' NOT NULL, " .
                                  "total_fn_items integer DEFAULT '0' NOT NULL, " .
                                  "oldest_spam_date timestamp, " .
                                  "newest_spam_date timestamp, " .
                                  "smallest_spam_size integer DEFAULT '0' NOT NULL, " .
                                  "largest_spam_size integer DEFAULT '0' NOT NULL, " .
                                  "total_spam_size integer DEFAULT '0' NOT NULL, " .
                                  "lowest_spam_score numeric(7,3) DEFAULT '0.0' NOT NULL, " .
                                  "highest_spam_score numeric(7,3) DEFAULT '0.0' NOT NULL, " .
                                  "total_spam_score numeric(14,3) DEFAULT '0.0' NOT NULL, " .
                                  "total_spam_items integer DEFAULT '0' NOT NULL, " .
                                  "oldest_virus_date timestamp, " .
                                  "newest_virus_date timestamp, " .
                                  "smallest_virus_size integer DEFAULT '0' NOT NULL, " .
                                  "largest_virus_size integer DEFAULT '0' NOT NULL, " .
                                  "total_virus_size integer DEFAULT '0' NOT NULL, " .
                                  "total_virus_items integer DEFAULT '0' NOT NULL, " .
                                  "oldest_bad_header_date timestamp, " .
                                  "newest_bad_header_date timestamp, " .
                                  "smallest_bad_header_size integer DEFAULT '0' NOT NULL, " .
                                  "largest_bad_header_size integer DEFAULT '0' NOT NULL, " .
                                  "total_bad_header_size integer DEFAULT '0' NOT NULL, " .
                                  "total_bad_header_items integer DEFAULT '0' NOT NULL, " .
                                  "oldest_banned_file_date timestamp, " .
                                  "newest_banned_file_date timestamp, " .
                                  "smallest_banned_file_size integer DEFAULT '0' NOT NULL, " .
                                  "largest_banned_file_size integer DEFAULT '0' NOT NULL, " .
                                  "total_banned_file_size integer DEFAULT '0' NOT NULL, " .
                                  "total_banned_file_items integer DEFAULT '0' NOT NULL, " .
                                  "oldest_oversized_date timestamp, " .
                                  "newest_oversized_date timestamp, " .
                                  "smallest_oversized_size integer DEFAULT '0' NOT NULL, " .
                                  "largest_oversized_size integer DEFAULT '0' NOT NULL, " .
                                  "total_oversized_size	integer DEFAULT '0' NOT NULL, " .
                                  "total_oversized_items integer DEFAULT '0' NOT NULL, " .
                                  "PRIMARY KEY (user_id))";
    	                break;

    	            case "maia_mail":
    	                $create = "CREATE TABLE maia_mail (id SERIAL PRIMARY KEY, " .
                                  "received_date timestamp NOT NULL, " .
                                  "size integer NOT NULL, " .
                                  "sender_email varchar(255) NOT NULL, " .
                                  "envelope_to text NOT NULL, " .
                                  "subject varchar(255) NOT NULL, " .
                                  "contents text NOT NULL, " .
                                  "score numeric(7,3))";
    	                break;

    	            case "maia_mail_recipients":
    	                $create = "CREATE TABLE maia_mail_recipients (mail_id integer NOT NULL, " .
                                  "recipient_id integer NOT NULL, " .
                                  "type char(1) NOT NULL, " .
                                  "PRIMARY KEY (mail_id, recipient_id))";
    	                break;

    	            case "maia_viruses":
    	                $create = "CREATE TABLE maia_viruses (id SERIAL PRIMARY KEY, " .
                                  "virus_name varchar(255) NOT NULL UNIQUE, " .
                                  "count integer DEFAULT '0' NOT NULL)";
    	                break;

    	            case "maia_virus_aliases":
    	                $create = "CREATE TABLE maia_virus_aliases (virus_id integer NOT NULL, " .
                                  "virus_alias varchar(255) NOT NULL, " .
                                  "PRIMARY KEY (virus_id, virus_alias))";
    	                break;

    	            case "maia_viruses_detected":
    	                $create = "CREATE TABLE maia_viruses_detected (mail_id integer NOT NULL, " .
                                  "virus_id integer NOT NULL, " .
                                  "PRIMARY KEY (mail_id, virus_id))";
    	                break;

    	            case "maia_sa_rules":
    	                $create = "CREATE TABLE maia_sa_rules (id SERIAL PRIMARY KEY, " .
                                  "rule_name varchar(255) NOT NULL UNIQUE, " .
                                  "rule_description varchar(255) DEFAULT '' NOT NULL, " .
                                  "rule_score_0 numeric(7,3) DEFAULT '1.0' NOT NULL, " .
                                  "rule_score_1 numeric(7,3) DEFAULT '1.0' NOT NULL, " .
                                  "rule_score_2	numeric(7,3) DEFAULT '1.0' NOT NULL, " .
                                  "rule_score_3	numeric(7,3) DEFAULT '1.0' NOT NULL, " .
                                  "rule_count integer DEFAULT '0' NOT NULL)";
    	                break;

    	            case "maia_sa_rules_triggered":
    	                $create = "CREATE TABLE maia_sa_rules_triggered (mail_id integer NOT NULL, " .
                                  "rule_id integer NOT NULL, " .
                                  "rule_score numeric(7,3) default '0.0' NOT NULL, " .
                                  "PRIMARY KEY (mail_id, rule_id))";
    	                break;

    	            case "maia_banned_attachments_found":
    	                $create = "CREATE TABLE maia_banned_attachments_found (mail_id integer NOT NULL, " .
                                  "file_name varchar(255) NOT NULL, " .
                                  "file_type varchar(20) DEFAULT 'Unknown' NOT NULL, " .
                                  "PRIMARY KEY (mail_id, file_name))";
    	                break;

    	            case "maia_stats_history":
    	                $create = "CREATE TABLE maia_stats_history (id SERIAL PRIMARY KEY, " .
                                  "user_id integer NOT NULL, " .
                                  "type char(1) DEFAULT 'H' NOT NULL, " .
                                  "taken_at timestamp NOT NULL, " .
                                  "total_ham_items integer DEFAULT '0' NOT NULL, " .
                                  "total_ham_size integer DEFAULT '0' NOT NULL, " .
                                  "total_spam_items integer DEFAULT '0' NOT NULL, " .
                                  "total_spam_size integer DEFAULT '0' NOT NULL, " .
                                  "total_virus_items integer DEFAULT '0' NOT NULL, " .
                                  "total_virus_size integer DEFAULT '0' NOT NULL, " .
                                  "total_fp_items integer DEFAULT '0' NOT NULL, " .
                                  "total_fp_size integer DEFAULT '0' NOT NULL, " .
                                  "total_fn_items integer DEFAULT '0' NOT NULL, " .
                                  "total_fn_size integer DEFAULT '0' NOT NULL, " .
                                  "total_banned_file_items integer DEFAULT '0' NOT NULL, " .
                                  "total_banned_file_size integer DEFAULT '0' NOT NULL, " .
                                  "total_bad_header_items integer DEFAULT '0' NOT NULL, " .
                                  "total_bad_header_size integer DEFAULT '0' NOT NULL, " .
                                  "total_wl_items integer DEFAULT '0' NOT NULL, " .
                                  "total_wl_size integer DEFAULT '0' NOT NULL, " .
                                  "total_bl_items integer DEFAULT '0' NOT NULL, " .
                                  "total_bl_size integer DEFAULT '0' NOT NULL, " .
                                  "total_oversized_items integer DEFAULT '0' NOT NULL, " .
                                  "total_oversized_size integer DEFAULT '0' NOT NULL)";
    	                break;

    	        }
    	        break;

    	}
    	$dbh->query($create);

    }


    function find_path($path_list, $file)
    {
    	$match = "";

    	foreach ($path_list as $path)
    	{
    	    $test_file = $path . "/" . $file;
    	    if (file_exists($test_file)) {
    	        $match = $test_file;
    	        break;
    	    }
        }

    	return $match;
    }


    print("<div align=\"center\">");
    print("<table border=\"0\" cellspacing=\"2\" cellpadding=\"2\" width=\"100%\">\n");

    print("<tr><td bgcolor=\"#CCCCCC\" align=\"center\" colspan=\"2\">\n");
    print("<h1>Maia Mailguard Configuration Tester</h1></td></tr>\n");

    // PHP
    print("<tr><td bgcolor=\"#CCCCCC\">PHP</td>\n");
    $php_version = phpversion();
    $include_path = ini_get("include_path");
    $path_list = explode(":", $include_path);
    if ($php_version >= "4.0.2") {
        print("<td bgcolor=\"#00DD00\" width=\"70%\"><b>OK:</b> " . $php_version . "</td>\n");
    } else {
    	$errmsg = "PHP version 4.0.2 or later is required.  See " .
    	          "<a href=\"http://pear.php.net/\">this page</a> for " .
    	          "information about downloading a more current version of PHP.";
        print ("<td bgcolor=\"#FE899B\" width=\"75%\"><b>FAILED:</b> " . $errmsg . "</td>");
    }
    print("</tr>\n");

    // PHP Modules
    ob_start();
    phpinfo(INFO_MODULES);
    $php_info = ob_get_contents();
    ob_end_clean();


    // PEAR
    print("<tr><td bgcolor=\"#CCCCCC\">PEAR</td>\n");
    $have_pear = false;
    if (!($pear_dir = find_path($path_list, "PEAR"))) {
    	$errmsg = "Not installed.  The PEAR extension is required by several other " .
    	          "PHP extensions that Maia needs.  See <a href=\"http://pear.php.net/\">this page</a> " .
    	          "for more information about downloading and installing PEAR.";
        print ("<td bgcolor=\"#FE899B\"><b>FAILED:</b> " . $errmsg . "</td>");
    } else {
    	$have_pear = true;
    	$pear = new PEAR_Config();
        $pear_reg = new PEAR_Registry($pear->get('php_dir'));
        $pear_info = $pear_reg->packageInfo("PEAR");
        $pear_version = $pear_info["version"];
        print("<td bgcolor=\"#00DD00\"><b>OK:</b> " . $pear_version . "</td>\n");
    }
    print("</tr>\n");


    // PEAR::Mail_Mime
    print("<tr><td bgcolor=\"#CCCCCC\">PEAR::Mail_Mime</td>\n");
    if ($have_pear) {
       $mime_info = $pear_reg->packageInfo("Mail_Mime");
       if (strlen($mime_info["package"]) < 2) {
       	   $errmsg = "Not installed.  This PHP extension is required to decode " .
       	             "MIME-structured e-mail.  Use <b>pear install Mail_Mime</b> to " .
       	             "install this.";
           print ("<td bgcolor=\"#FE899B\"><b>FAILED:</b> " . $errmsg . "</td>");
       } else {
    	   $mime_version = $mime_info["version"];
           print("<td bgcolor=\"#00DD00\"><b>OK:</b> " . $mime_version . "</td>\n");
       }
    } else {
        print("<td bgcolor=\"#DDDD00\"><b>SKIPPED:</b> Requires PEAR</td>");
    }
    print("</tr>\n");


    // PEAR::DB
    print("<tr><td bgcolor=\"#CCCCCC\">PEAR::DB</td>\n");
    if ($have_pear) {
        $db_info = $pear_reg->packageInfo("DB");
        if (strlen($db_info["package"]) < 2) {
            $errmsg = "Not installed.  This PHP extension is required in order to provide " .
                      "database abstraction.  Use <b>pear install DB</b> to install this.";
            print ("<td bgcolor=\"#FE899B\" width=\"300\"><b>FAILED:</b> " . $errmsg . "</td>");
        } else {
            $test_dbh = DB::connect($maia_sql_dsn);
    	    if (DB::isError($test_dbh)) {
    	        print ("<td bgcolor=\"#FE899B\"><b>FAILED:</b> Could not connect to database.  Check the $maia_sql_dsn setting in config.php.</td>");
    	    } else {
    	        $db_version = $db_info["version"];
                print("<td bgcolor=\"#00DD00\"><b>OK:</b> " . $db_version . "</td>\n");
            }
        }
    } else {
        print("<td bgcolor=\"#DDDD00\"><b>SKIPPED:</b> Requires PEAR</td>");
    }
    print("</tr>\n");


    // PEAR::DB_Pager
    print("<tr><td bgcolor=\"#CCCCCC\">PEAR::DB_Pager</td>\n");
    if ($have_pear) {
        $pager_info = $pear_reg->packageInfo("DB_Pager");
        if (strlen($pager_info["package"]) < 2) {
            $errmsg = "Not installed.  This PHP extension is required in order to paginate " .
                      "the list of mail items in the quarantines and the ham cache.  Use " .
                      "<b>pear install DB_Pager</b> to install this.";
            print ("<td bgcolor=\"#FE899B\"><b>FAILED:</b> " . $errmsg . "</td>");
        } else {
    	    $pager_version = $pager_info["version"];
            print("<td bgcolor=\"#00DD00\"><b>OK:</b> " . $pager_version . "</td>\n");
        }
    } else {
        print("<td bgcolor=\"#DDDD00\"><b>SKIPPED:</b> Requires PEAR</td>");
    }
    print("</tr>\n");


    // PEAR::Log
    print("<tr><td bgcolor=\"#CCCCCC\">PEAR::Log</td>\n");
    if ($have_pear) {
        $log_info = $pear_reg->packageInfo("Log");
        if (strlen($log_info["package"]) < 2) {
            $errmsg = "Not installed, but only required if you want to be able to log user " .
                      "and administrator activity.  Use <b>pear install Log</b> to install " .
                      "this PHP extension.";
            print ("<td bgcolor=\"#FE899B\"><b>FAILED:</b> " . $errmsg . "</td>");
        } else {
    	    $log_version = $log_info["version"];
            print("<td bgcolor=\"#00DD00\"><b>OK:</b> " . $log_version . "</td>\n");
        }
    } else {
        print("<td bgcolor=\"#DDDD00\"><b>SKIPPED:</b> Requires PEAR</td>");
    }
    print("</tr>\n");


    // IMAP
    print("<tr><td bgcolor=\"#CCCCCC\">IMAP library</td>\n");
    if (!function_exists("imap_open")) {
    	$errmsg = "Not installed, but only required if you want to be able to authenticate " .
    	          "with IMAP or POP3.  See <a href=\"http://www.php.net/imap/\">this page</a> for " .
    	          "more information about downloading the IMAP extensions to PHP, and " .
    	          "instructions for recompiling PHP with the --with-imap flag.";
        print ("<td bgcolor=\"#FE899B\" width=\"300\"><b>FAILED:</b> " . $errmsg . "</td>");
    } else {
    	if (preg_match("/IMAP c-Client Version.*?>([0-9]+)/si", $php_info, $match)) {
    	    $imap_version = $match[1];
    	} else {
    	    $imap_version = "Unknown";
    	}
        print("<td bgcolor=\"#00DD00\"><b>OK:</b> " . $imap_version . "</td>\n");
    }
    print("</tr>\n");

    // LDAP
    print("<tr><td bgcolor=\"#CCCCCC\">LDAP library</td>\n");
    if (!function_exists("ldap_connect")) {
    	$errmsg = "Not installed, but only required if you want to be able to authenticate " .
    	          "with LDAP.  See <a href=\"http://www.php.net/ldap/\">this page</a> for " .
    	          "more information about downloading the LDAP extensions to PHP, and " .
    	          "instructions for recompiling PHP with the --with-ldap flag.";
        print ("<td bgcolor=\"#FE899B\" width=\"300\"><b>FAILED:</b> " . $errmsg . "</td>");
    } else {
        print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
    }
    print("</tr>\n");


    // mcrypt
    print("<tr><td bgcolor=\"#CCCCCC\">MCrypt library</td>\n");
    if (!function_exists("mcrypt_get_iv_size")) {
    	$errmsg = "Not installed, but only required if you want to be able to encrypt the " .
    	          "mail stored in Maia's database (quarantine and ham cache).  See " .
    	          "<a href=\"http://www.php.net/mcrypt/\">this page</a> for more information " .
    	          "about downloading the MCrypt extensions to PHP, and instructions for " .
    	          "recompiling PHP with the --with-mcrypt flag.";
        print ("<td bgcolor=\"#FE899B\"><b>FAILED:</b> " . $errmsg . "</td>");
    } else {
    	if (preg_match("/mcrypt support.*?version.*?<td.*?>(.+?)</si", $php_info, $match)) {
    	    $mcrypt_version = $match[1];
    	} else {
    	    $mcrypt_version = "Unknown";
    	}
    	$have_blowfish = (preg_match("/mcrypt support.*?Supported ciphers.*?(blowfish)[^\-]/si", $php_info, $match));
    	$have_cbc = (preg_match("/mcrypt support.*?Supported modes.*?(cbc)/si", $php_info, $match));
    	if ($have_blowfish && $have_cbc) {
            print("<td bgcolor=\"#00DD00\"><b>OK:</b> " . $mcrypt_version . " with Blowfish and CBC</td>\n");
        } else {
            $errmsg = "Support for ";
            if (!$have_blowfish) {
                $errmsg .= "Blowfish ";
                if (!$have_cbc) {
                    $errmsg .= "and CBC ";
                }
            } else {
                $errmsg .= "CBC ";
            }
            $errmsg .= "must be compiled into libmcrypt.";
            print ("<td bgcolor=\"#FE899B\"><b>FAILED:</b> " . $errmsg . "</td>");
        }
    }
    print("</tr>\n");


    // BC
    print("<tr><td bgcolor=\"#CCCCCC\">BC math library</td>\n");
    if (!function_exists("bcadd")) {
    	$errmsg = "Not installed.  This PHP extension is required in order to decode certain " .
    	          "types of URLs.  See <a href=\"http://www.php.net/bc/\">this page</a> " .
    	          "for more information about recompiling PHP with the --enable-bcmath flag.";
        print ("<td bgcolor=\"#FE899B\" width=\"300\"><b>FAILED:</b> " . $errmsg . "</td>");
    } else {
        print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
    }
    print("</tr>\n");


    // gd
    print("<tr><td bgcolor=\"#CCCCCC\">gd graphics library</td>\n");
    if (!function_exists("gd_info")) {
    	$errmsg = "Not installed, but only required if you want to be able to generate charts " .
    	          "based on Maia's statistics.  See <a href=\"http://www.php.net/gd/\">this page</a> " .
    	          "for more information about recompiling PHP with the --with-gd flag.";
        print ("<td bgcolor=\"#FE899B\" width=\"300\"><b>FAILED:</b> " . $errmsg . "</td>");
    } else {
    	$info = gd_info();
        print("<td bgcolor=\"#00DD00\"><b>OK:</b> " . $info["GD Version"] . "</td>\n");
    }
    print("</tr>\n");


    // JpGraph
    print("<tr><td bgcolor=\"#CCCCCC\">JpGraph library</td>\n");
    if (!find_path($path_list, "jpgraph/jpgraph.php")) {
    	$errmsg = "Not installed, but only required if you want to be able to generate charts " .
    	          "based on Maia's statistics.  See <a href=\"http://www.aditus.nu/jpgraph/\">this page</a> " .
    	          "for more information about downloading the latest JpGraph library, which must " .
    	          "be installed in a directory called \"jpgraph\" somewhere in your PHP include_path " .
    	          "(" . $include_path . ").";
        print ("<td bgcolor=\"#FE899B\"><b>FAILED:</b> " . $errmsg . "</td>");
    } else {
        print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
    }
    print("</tr>\n");


    if (isset($db_version)) {

        // Check table: users
        print("<tr><td bgcolor=\"#CCCCCC\">Table: users</td>\n");
        if (table_exists($test_dbh, "users")) {
            $select = "SELECT * FROM users";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $users_cols[] = $col["name"];
            }
            $sth->free();
            $users_cols = array_flip($users_cols);
            $msg = "";

            if (!column_exists($users_cols, "priority")) {
            	add_column($test_dbh, "users", "priority", "int", 0, "7", false);
            	$msg .= "Added column: priority<br>";
            }

            if (!column_exists($users_cols, "policy_id")) {
            	add_column($test_dbh, "users", "policy_id", "uint", 0, "1", false);
            	$msg .= "Added column: policy_id<br>";
            }

            if (!column_exists($users_cols, "email")) {
            	add_column($test_dbh, "users", "email", "varchar", 255, "", false);
            	$msg .= "Added column: email<br>";
            }

            if (!column_exists($users_cols, "maia_user_id")) {
            	add_column($test_dbh, "users", "maia_user_id", "uint", 0, "", false);
            	$msg .= "Added column: maia_user_id<br>";
            }

            if (!column_exists($users_cols, "maia_domain_id")) {
            	add_column($test_dbh, "users", "maia_domain_id", "uint", 0, "", false);
            	$msg .= "Added column: maia_domain_id<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "users");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check index: users table
        print("<tr><td bgcolor=\"#CCCCCC\">Index: users table</td>\n");
        $msg = "";
        if (!index_exists($test_dbh, "users", "users_idx_email")) {
            create_index($test_dbh, "users", "email", true);
            $msg .= "Created index: users_idx_email";
        }
        if (empty($msg)) {
            print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
        } else {
            print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: mailaddr
        print("<tr><td bgcolor=\"#CCCCCC\">Table: mailaddr</td>\n");
        if (table_exists($test_dbh, "mailaddr")) {
            $select = "SELECT * FROM mailaddr";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $mailaddr_cols[] = $col["name"];
            }
            $sth->free();
            $mailaddr_cols = array_flip($mailaddr_cols);
            $msg = "";

            if (!column_exists($mailaddr_cols, "priority")) {
            	add_column($test_dbh, "mailaddr", "priority", "int", 0, "7", false);
            	$msg .= "Added column: priority<br>";
            }

            if (!column_exists($mailaddr_cols, "email")) {
            	add_column($test_dbh, "mailaddr", "email", "varchar", 255, "", false);
            	$msg .= "Added column: email<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "mailaddr");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check index: mailaddr table
        print("<tr><td bgcolor=\"#CCCCCC\">Index: mailaddr table</td>\n");
        $msg = "";
        if (!index_exists($test_dbh, "mailaddr", "mailaddr_idx_email")) {
            create_index($test_dbh, "mailaddr", "email", true);
            $msg .= "Created index: mailaddr_idx_email";
        }
        if (empty($msg)) {
            print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
        } else {
            print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: wblist
        print("<tr><td bgcolor=\"#CCCCCC\">Table: wblist</td>\n");
        if (table_exists($test_dbh, "wblist")) {
            $select = "SELECT * FROM wblist";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $wblist_cols[] = $col["name"];
            }
            $sth->free();
            $wblist_cols = array_flip($wblist_cols);
            $msg = "";

            if (!column_exists($wblist_cols, "rid")) {
            	add_column($test_dbh, "wblist", "rid", "uint", 0, "", false);
            	$msg .= "Added column: rid<br>";
            }

            if (!column_exists($wblist_cols, "sid")) {
            	add_column($test_dbh, "wblist", "sid", "uint", 0, "", false);
            	$msg .= "Added column: sid<br>";
            }

            if (!column_exists($wblist_cols, "wb")) {
            	add_column($test_dbh, "wblist", "wb", "char", 1, "", false);
            	$msg .= "Added column: wb<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "wblist");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: policy
        print("<tr><td bgcolor=\"#CCCCCC\">Table: policy</td>\n");
        if (table_exists($test_dbh, "policy")) {
            $select = "SELECT * FROM policy";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $policy_cols[] = $col["name"];
            }
            $sth->free();
            $policy_cols = array_flip($policy_cols);
            $msg = "";

            if (!column_exists($policy_cols, "policy_name")) {
            	add_column($test_dbh, "policy", "policy_name", "varchar", 255, "", true);
            	$msg .= "Added column: policy_name<br>";
            }

            if (!column_exists($policy_cols, "virus_lover")) {
            	add_column($test_dbh, "policy", "virus_lover", "char", 1, "Y", true);
            	$msg .= "Added column: virus_lover<br>";
            }

            if (!column_exists($policy_cols, "spam_lover")) {
            	add_column($test_dbh, "policy", "spam_lover", "char", 1, "Y", true);
            	$msg .= "Added column: spam_lover<br>";
            }

            if (!column_exists($policy_cols, "banned_files_lover")) {
            	add_column($test_dbh, "policy", "banned_files_lover", "char", 1, "Y", true);
            	$msg .= "Added column: banned_files_lover<br>";
            }

            if (!column_exists($policy_cols, "bad_header_lover")) {
            	add_column($test_dbh, "policy", "bad_header_lover", "char", 1, "Y", true);
            	$msg .= "Added column: bad_header_lover<br>";
            }

            if (!column_exists($policy_cols, "bypass_virus_checks")) {
            	add_column($test_dbh, "policy", "bypass_virus_checks", "char", 1, "Y", true);
            	$msg .= "Added column: bypass_virus_checks<br>";
            }

            if (!column_exists($policy_cols, "bypass_spam_checks")) {
            	add_column($test_dbh, "policy", "bypass_spam_checks", "char", 1, "Y", true);
            	$msg .= "Added column: bypass_spam_checks<br>";
            }

            if (!column_exists($policy_cols, "bypass_banned_checks")) {
            	add_column($test_dbh, "policy", "bypass_banned_checks", "char", 1, "Y", true);
            	$msg .= "Added column: bypass_banned_checks<br>";
            }

            if (!column_exists($policy_cols, "bypass_header_checks")) {
            	add_column($test_dbh, "policy", "bypass_header_checks", "char", 1, "Y", true);
            	$msg .= "Added column: bypass_header_checks<br>";
            }

            if (!column_exists($policy_cols, "discard_viruses")) {
            	add_column($test_dbh, "policy", "discard_viruses", "char", 1, "N", true);
            	$msg .= "Added column: discard_viruses<br>";
            }

            if (!column_exists($policy_cols, "discard_spam")) {
            	add_column($test_dbh, "policy", "discard_spam", "char", 1, "N", true);
            	$msg .= "Added column: discard_spam<br>";
            }

            if (!column_exists($policy_cols, "discard_banned_files")) {
            	add_column($test_dbh, "policy", "discard_banned_files", "char", 1, "N", true);
            	$msg .= "Added column: discard_banned_files<br>";
            }

            if (!column_exists($policy_cols, "discard_bad_headers")) {
            	add_column($test_dbh, "policy", "discard_bad_headers", "char", 1, "N", true);
            	$msg .= "Added column: discard_bad_headers<br>";
            }

            if (!column_exists($policy_cols, "spam_modifies_subj")) {
            	add_column($test_dbh, "policy", "spam_modifies_subj", "char", 1, "N", true);
            	$msg .= "Added column: spam_modifies_subj<br>";
            }

            if (!column_exists($policy_cols, "spam_quarantine_to")) {
            	add_column($test_dbh, "policy", "spam_quarantine_to", "varchar", 64, "", true);
            	$msg .= "Added column: spam_quarantine_to<br>";
            }

            if (!column_exists($policy_cols, "spam_tag_level")) {
            	add_column($test_dbh, "policy", "spam_tag_level", "float", "7,3", "999", true);
            	$msg .= "Added column: spam_tag_level<br>";
            }

            if (!column_exists($policy_cols, "spam_tag2_level")) {
            	add_column($test_dbh, "policy", "spam_tag2_level", "float", "7,3", "999", true);
            	$msg .= "Added column: spam_tag2_level<br>";
            }

            if (!column_exists($policy_cols, "spam_kill_level")) {
            	add_column($test_dbh, "policy", "spam_kill_level", "float", "7,3", "999", true);
            	$msg .= "Added column: spam_kill_level<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "policy");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_config
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_config</td>\n");
        if (table_exists($test_dbh, "maia_config")) {
            $select = "SELECT * FROM maia_config";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_config_cols[] = $col["name"];
            }
            $sth->free();
            $maia_config_cols = array_flip($maia_config_cols);
            $msg = "";

            if (column_exists($maia_config_cols, "amavisd_conf")) {
                drop_column($test_dbh, "maia_config", "amavisd_conf");
                $msg .= "Dropped column: amavisd_conf<br>";
            }

            if (column_exists($maia_config_cols, "newuser2_template_file")) {
                drop_column($test_dbh, "maia_config", "newuser2_template_file");
                $msg .= "Dropped column: newuser2_template_file<br>";
            }

            if (column_exists($maia_config_cols, "installation_directory")) {
                drop_column($test_dbh, "maia_config", "installation_directory");
                $msg .= "Dropped column: installation_directory<br>";
            }

            if (!column_exists($maia_config_cols, "enable_user_autocreation")) {
            	add_column($test_dbh, "maia_config", "enable_user_autocreation", "char", 1, "N", false);
            	$msg .= "Added column: enable_user_autocreation<br>";
            }

            if (!column_exists($maia_config_cols, "enable_false_negative_management")) {
            	add_column($test_dbh, "maia_config", "enable_false_negative_management", "char", 1, "Y", false);
            	$msg .= "Added column: enable_false_negative_management<br>";
            }

            if (!column_exists($maia_config_cols, "enable_stats_tracking")) {
            	add_column($test_dbh, "maia_config", "enable_stats_tracking", "char", 1, "Y", false);
            	$msg .= "Added column: enable_stats_tracking<br>";
            }

            if (!column_exists($maia_config_cols, "enable_virus_scanning")) {
            	add_column($test_dbh, "maia_config", "enable_virus_scanning", "char", 1, "Y", false);
            	$msg .= "Added column: enable_virus_scanning<br>";
            }

            if (!column_exists($maia_config_cols, "enable_spam_filtering")) {
            	add_column($test_dbh, "maia_config", "enable_spam_filtering", "char", 1, "Y", false);
            	$msg .= "Added column: enable_spam_filtering<br>";
            }

            if (!column_exists($maia_config_cols, "enable_banned_files_checking")) {
            	add_column($test_dbh, "maia_config", "enable_banned_files_checking", "char", 1, "Y", false);
            	$msg .= "Added column: enable_banned_files_checking<br>";
            }

            if (!column_exists($maia_config_cols, "enable_bad_header_checking")) {
            	add_column($test_dbh, "maia_config", "enable_bad_header_checking", "char", 1, "Y", false);
            	$msg .= "Added column: enable_bad_header_checking<br>";
            }

            if (!column_exists($maia_config_cols, "enable_charts")) {
            	add_column($test_dbh, "maia_config", "enable_charts", "char", 1, "N", false);
            	$msg .= "Added column: enable_charts<br>";
            }

            if (!column_exists($maia_config_cols, "enable_spamtraps")) {
            	add_column($test_dbh, "maia_config", "enable_spamtraps", "char", 1, "N", false);
            	$msg .= "Added column: enable_spamtraps<br>";
            }

            if (!column_exists($maia_config_cols, "enable_stats_reporting")) {
            	add_column($test_dbh, "maia_config", "enable_stats_reporting", "char", 1, "N", false);
            	$msg .= "Added column: enable_stats_reporting<br>";
            }

            if (!column_exists($maia_config_cols, "enable_address_linking")) {
            	add_column($test_dbh, "maia_config", "enable_address_linking", "char", 1, "Y", false);
            	$msg .= "Added column: enable_address_linking<br>";
            }

            if (!column_exists($maia_config_cols, "enable_privacy_invasion")) {
            	add_column($test_dbh, "maia_config", "enable_privacy_invasion", "char", 1, "N", false);
            	$msg .= "Added column: enable_privacy_invasion<br>";
            }

            if (!column_exists($maia_config_cols, "enable_username_changes")) {
            	add_column($test_dbh, "maia_config", "enable_username_changes", "char", 1, "Y", false);
            	$msg .= "Added column: enable_username_changes<br>";
            }

            if (!column_exists($maia_config_cols, "internal_auth")) {
            	add_column($test_dbh, "maia_config", "internal_auth", "char", 1, "N", false);
            	$msg .= "Added column: internal_auth<br>";
            }

            if (!column_exists($maia_config_cols, "system_default_user_is_local")) {
            	add_column($test_dbh, "maia_config", "system_default_user_is_local", "char", 1, "Y", false);
            	$msg .= "Added column: system_default_user_is_local<br>";
            }

            if (!column_exists($maia_config_cols, "user_virus_scanning")) {
            	add_column($test_dbh, "maia_config", "user_virus_scanning", "char", 1, "Y", false);
            	$msg .= "Added column: user_virus_scanning<br>";
            }

            if (!column_exists($maia_config_cols, "user_spam_filtering")) {
            	add_column($test_dbh, "maia_config", "user_spam_filtering", "char", 1, "Y", false);
            	$msg .= "Added column: user_spam_filtering<br>";
            }

            if (!column_exists($maia_config_cols, "user_banned_files_checking")) {
            	add_column($test_dbh, "maia_config", "user_banned_files_checking", "char", 1, "Y", false);
            	$msg .= "Added column: user_banned_files_checking<br>";
            }

            if (!column_exists($maia_config_cols, "user_bad_header_checking")) {
            	add_column($test_dbh, "maia_config", "user_bad_header_checking", "char", 1, "Y", false);
            	$msg .= "Added column: user_bad_header_checking<br>";
            }

            if (!column_exists($maia_config_cols, "admin_email")) {
            	add_column($test_dbh, "maia_config", "admin_email", "varchar", 255, "", true);
            	$msg .= "Added column: admin_email<br>";
            }

            if (!column_exists($maia_config_cols, "expiry_period")) {
            	add_column($test_dbh, "maia_config", "expiry_period", "uint", 0, "30", true);
            	$msg .= "Added column: expiry_period<br>";
            }

            if (!column_exists($maia_config_cols, "ham_cache_expiry_period")) {
            	add_column($test_dbh, "maia_config", "ham_cache_expiry_period", "uint", 0, "5", true);
            	$msg .= "Added column: ham_cache_expiry_period<br>";
            }

            if (!column_exists($maia_config_cols, "reminder_threshold_count")) {
            	add_column($test_dbh, "maia_config", "reminder_threshold_count", "uint", 0, "100", true);
            	$msg .= "Added column: reminder_threshold_count<br>";
            }

            if (!column_exists($maia_config_cols, "reminder_threshold_size")) {
            	add_column($test_dbh, "maia_config", "reminder_threshold_size", "uint", 0, "500000", true);
            	$msg .= "Added column: reminder_threshold_size<br>";
            }

            if (!column_exists($maia_config_cols, "reminder_template_file")) {
            	add_column($test_dbh, "maia_config", "reminder_template_file", "varchar", 255, "reminder.tpl", true);
            	$msg .= "Added column: reminder_template_file<br>";
            }

            if (!column_exists($maia_config_cols, "reminder_login_url")) {
            	add_column($test_dbh, "maia_config", "reminder_login_url", "varchar", 255, "", true);
            	$msg .= "Added column: reminder_login_url<br>";
            }

            if (!column_exists($maia_config_cols, "newuser_template_file")) {
            	add_column($test_dbh, "maia_config", "newuser_template_file", "varchar", 255, "newuser.tpl", true);
            	$msg .= "Added column: newuser_template_file<br>";
            }

            if (!column_exists($maia_config_cols, "smtp_server")) {
            	add_column($test_dbh, "maia_config", "smtp_server", "varchar", 255, "localhost", true);
            	$msg .= "Added column: smtp_server<br>";
            }

            if (!column_exists($maia_config_cols, "smtp_port")) {
            	add_column($test_dbh, "maia_config", "smtp_port", "uint", 0, "10025", true);
            	$msg .= "Added column: smtp_port<br>";
            }

            if (!column_exists($maia_config_cols, "currency_label")) {
            	add_column($test_dbh, "maia_config", "currency_label", "varchar", 15, "$", true);
            	$msg .= "Added column: currency_label<br>";
            }

            if (!column_exists($maia_config_cols, "bandwidth_cost")) {
            	add_column($test_dbh, "maia_config", "bandwidth_cost", "float", "14,2", "0.00", false);
            	$msg .= "Added column: bandwidth_cost<br>";
            }

            if (!column_exists($maia_config_cols, "chart_ham_colour")) {
            	add_column($test_dbh, "maia_config", "chart_ham_colour", "varchar", 32, "#DDDDB7", true);
            	$msg .= "Added column: chart_ham_colour<br>";
            }

            if (!column_exists($maia_config_cols, "chart_spam_colour")) {
            	add_column($test_dbh, "maia_config", "chart_spam_colour", "varchar", 32, "#FFAAAA", true);
            	$msg .= "Added column: chart_spam_colour<br>";
            }

            if (!column_exists($maia_config_cols, "chart_ham_colour")) {
            	add_column($test_dbh, "maia_config", "chart_ham_colour", "varchar", 32, "#DDDDB7", true);
            	$msg .= "Added column: chart_ham_colour<br>";
            }

            if (!column_exists($maia_config_cols, "chart_virus_colour")) {
            	add_column($test_dbh, "maia_config", "chart_virus_colour", "varchar", 32, "#CCFFCC", true);
            	$msg .= "Added column: chart_virus_colour<br>";
            }

            if (!column_exists($maia_config_cols, "chart_fp_colour")) {
            	add_column($test_dbh, "maia_config", "chart_fp_colour", "varchar", 32, "#C4CA73", true);
            	$msg .= "Added column: chart_fp_colour<br>";
            }

            if (!column_exists($maia_config_cols, "chart_fn_colour")) {
            	add_column($test_dbh, "maia_config", "chart_fn_colour", "varchar", 32, "#FF7575", true);
            	$msg .= "Added column: chart_fn_colour<br>";
            }

            if (!column_exists($maia_config_cols, "chart_suspected_ham_colour")) {
            	add_column($test_dbh, "maia_config", "chart_suspected_ham_colour", "varchar", 32, "#FFFFB7", true);
            	$msg .= "Added column: chart_suspected_ham_colour<br>";
            }

            if (!column_exists($maia_config_cols, "chart_suspected_spam_colour")) {
            	add_column($test_dbh, "maia_config", "chart_suspected_spam_colour", "varchar", 32, "#FFCCCC", true);
            	$msg .= "Added column: chart_suspected_spam_colour<br>";
            }

            if (!column_exists($maia_config_cols, "chart_wl_colour")) {
            	add_column($test_dbh, "maia_config", "chart_wl_colour", "varchar", 32, "#EEEEEE", true);
            	$msg .= "Added column: chart_wl_colour<br>";
            }

            if (!column_exists($maia_config_cols, "chart_bl_colour")) {
            	add_column($test_dbh, "maia_config", "chart_bl_colour", "varchar", 32, "#888888", true);
            	$msg .= "Added column: chart_bl_colour<br>";
            }

            if (!column_exists($maia_config_cols, "chart_background_colour")) {
            	add_column($test_dbh, "maia_config", "chart_background_colour", "varchar", 32, "#B0ECFF", true);
            	$msg .= "Added column: chart_background_colour<br>";
            }

            if (!column_exists($maia_config_cols, "chart_font_colour")) {
            	add_column($test_dbh, "maia_config", "chart_font_colour", "varchar", 32, "#3D3D50", true);
            	$msg .= "Added column: chart_font_colour<br>";
            }

            if (!column_exists($maia_config_cols, "chart_autogeneration_interval")) {
            	add_column($test_dbh, "maia_config", "chart_autogeneration_interval", "uint", 0, "60", true);
            	$msg .= "Added column: chart_autogeneration_interval<br>";
            }

            if (!column_exists($maia_config_cols, "banner_title")) {
            	add_column($test_dbh, "maia_config", "banner_title", "varchar", 255, "Maia Mailguard", true);
            	$msg .= "Added column: banner_title<br>";
            }

            if (!column_exists($maia_config_cols, "use_icons")) {
            	add_column($test_dbh, "maia_config", "use_icons", "char", 1, "Y", false);
            	$msg .= "Added column: use_icons<br>";
            }

            if (!column_exists($maia_config_cols, "use_logo")) {
            	add_column($test_dbh, "maia_config", "use_logo", "char", 1, "Y", false);
            	$msg .= "Added column: use_logo<br>";
            }

            if (!column_exists($maia_config_cols, "logo_url")) {
            	add_column($test_dbh, "maia_config", "logo_url", "varchar", 255, "http://www.renaissoft.com/maia/", true);
            	$msg .= "Added column: logo_url<br>";
            }

            if (!column_exists($maia_config_cols, "logo_file")) {
            	add_column($test_dbh, "maia_config", "logo_file", "varchar", 255, "images/maia-logotoolbar.gif", true);
            	$msg .= "Added column: logo_file<br>";
            }

            if (!column_exists($maia_config_cols, "logo_alt_text")) {
            	add_column($test_dbh, "maia_config", "logo_alt_text", "varchar", 255, "Maia Mailguard Home Page", true);
            	$msg .= "Added column: logo_alt_text<br>";
            }

            if (!column_exists($maia_config_cols, "virus_info_url")) {
            	add_column($test_dbh, "maia_config", "virus_info_url", "varchar", 255, "http://www.google.com/search?q=%%VIRUSNAME%%+virus+information", true);
            	$msg .= "Added column: virus_info_url<br>";
            }

            if (!column_exists($maia_config_cols, "virus_lookup")) {
            	add_column($test_dbh, "maia_config", "virus_lookup", "varchar", 20, "google", true);
            	$msg .= "Added column: virus_lookup<br>";
            }

            if (!column_exists($maia_config_cols, "primary_report_server")) {
            	add_column($test_dbh, "maia_config", "primary_report_server", "varchar", 255, "maia.renaissoft.com", true);
            	$msg .= "Added column: primary_report_server<br>";
            }

            if (!column_exists($maia_config_cols, "primary_report_port")) {
            	add_column($test_dbh, "maia_config", "primary_report_port", "uint", 0, "443", true);
            	$msg .= "Added column: primary_report_port<br>";
            }

            if (!column_exists($maia_config_cols, "primary_report_server")) {
            	add_column($test_dbh, "maia_config", "primary_report_server", "varchar", 255, "maia.renaissoft.com", true);
            	$msg .= "Added column: primary_report_server<br>";
            }

            if (!column_exists($maia_config_cols, "secondary_report_server")) {
            	add_column($test_dbh, "maia_config", "secondary_report_server", "varchar", 255, "", true);
            	$msg .= "Added column: secondary_report_server<br>";
            }

            if (!column_exists($maia_config_cols, "secondary_report_port")) {
            	add_column($test_dbh, "maia_config", "secondary_report_port", "uint", 0, "443", true);
            	$msg .= "Added column: secondary_report_port<br>";
            }

            if (!column_exists($maia_config_cols, "reporter_sitename")) {
            	add_column($test_dbh, "maia_config", "reporter_sitename", "varchar", 255, "", true);
            	$msg .= "Added column: reporter_sitename<br>";
            }

            if (!column_exists($maia_config_cols, "reporter_username")) {
            	add_column($test_dbh, "maia_config", "reporter_username", "varchar", 50, "", true);
            	$msg .= "Added column: reporter_username<br>";
            }

            if (!column_exists($maia_config_cols, "reporter_password")) {
            	add_column($test_dbh, "maia_config", "reporter_password", "varchar", 50, "", true);
            	$msg .= "Added column: reporter_password<br>";
            }

            if (!column_exists($maia_config_cols, "size_limit")) {
            	add_column($test_dbh, "maia_config", "size_limit", "uint", 0, "1000000", true);
            	$msg .= "Added column: size_limit<br>";
            }

            if (!column_exists($maia_config_cols, "oversize_policy")) {
            	add_column($test_dbh, "maia_config", "oversize_policy", "char", 1, "B", false);
            	$msg .= "Added column: oversize_policy<br>";
            }

            if (!column_exists($maia_config_cols, "sa_score_set")) {
            	add_column($test_dbh, "maia_config", "sa_score_set", "uint", 0, "0", false);
            	$msg .= "Added column: sa_score_set<br>";
            }

            if (!column_exists($maia_config_cols, "key_file")) {
            	add_column($test_dbh, "maia_config", "key_file", "varchar", 255, "blowfish.key", true);
            	$msg .= "Added column: key_file<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_config");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_languages
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_languages</td>\n");
        if (table_exists($test_dbh, "maia_languages")) {
            $select = "SELECT * FROM maia_languages";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_languages_cols[] = $col["name"];
            }
            $sth->free();
            $maia_languages_cols = array_flip($maia_languages_cols);
            $msg = "";

            if (!column_exists($maia_languages_cols, "language_name")) {
            	add_column($test_dbh, "maia_languages", "language_name", "varchar", 100, "", false);
            	$msg .= "Added column: language_name<br>";
            }

            if (!column_exists($maia_languages_cols, "abbreviation")) {
            	add_column($test_dbh, "maia_languages", "abbreviation", "char", 2, "", false);
            	$msg .= "Added column: abbreviation<br>";
            }

            if (!column_exists($maia_languages_cols, "installed")) {
            	add_column($test_dbh, "maia_languages", "installed", "char", 1, "N", false);
            	$msg .= "Added column: installed<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_languages");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check index: languages table
        print("<tr><td bgcolor=\"#CCCCCC\">Index: maia_languages table</td>\n");
        $msg = "";
        if (!index_exists($test_dbh, "maia_languages", "maia_languages_idx_abbreviation")) {
            create_index($test_dbh, "maia_languages", "abbreviation", true);
            $msg .= "Created index: maia_languages_idx_abbreviation";
        }
        if (empty($msg)) {
            print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
        } else {
            print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_users
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_users</td>\n");
        if (table_exists($test_dbh, "maia_users")) {
            $select = "SELECT * FROM maia_users";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_users_cols[] = $col["name"];
            }
            $sth->free();
            $maia_users_cols = array_flip($maia_users_cols);
            $msg = "";

            if (!column_exists($maia_users_cols, "user_name")) {
            	add_column($test_dbh, "maia_users", "user_name", "varchar", 255, "", false);
            	$msg .= "Added column: user_name<br>";
            }

            if (!column_exists($maia_users_cols, "user_level")) {
            	add_column($test_dbh, "maia_users", "user_level", "char", 1, "U", false);
            	$msg .= "Added column: user_level<br>";
            }

            if (!column_exists($maia_users_cols, "reminders")) {
            	add_column($test_dbh, "maia_users", "reminders", "char", 1, "Y", false);
            	$msg .= "Added column: reminders<br>";
            }

            if (!column_exists($maia_users_cols, "charts")) {
            	add_column($test_dbh, "maia_users", "charts", "char", 1, "N", false);
            	$msg .= "Added column: charts<br>";
            }

            if (!column_exists($maia_users_cols, "primary_email_id")) {
            	add_column($test_dbh, "maia_users", "primary_email_id", "uint", 0, "0", false);
            	$msg .= "Added column: primary_email_id<br>";
            }

            if (!column_exists($maia_users_cols, "language")) {
            	add_column($test_dbh, "maia_users", "language", "varchar", 10, "en", false);
            	$msg .= "Added column: language<br>";
            }

            if (!column_exists($maia_users_cols, "charset")) {
            	add_column($test_dbh, "maia_users", "charset", "varchar", 20, "ISO-8859-1", false);
            	$msg .= "Added column: charset<br>";
            }

            if (!column_exists($maia_users_cols, "spamtrap")) {
            	add_column($test_dbh, "maia_users", "spamtrap", "char", 1, "N", false);
            	$msg .= "Added column: spamtrap<br>";
            }

            if (!column_exists($maia_users_cols, "password")) {
            	add_column($test_dbh, "maia_users", "password", "varchar", 32, "", true);
            	$msg .= "Added column: password<br>";
            }

            if (!column_exists($maia_users_cols, "auto_whitelist")) {
            	add_column($test_dbh, "maia_users", "auto_whitelist", "char", 1, "Y", false);
            	$msg .= "Added column: auto_whitelist<br>";
            }

            if (!column_exists($maia_users_cols, "items_per_page")) {
            	add_column($test_dbh, "maia_users", "items_per_page", "uint", 0, "50", false);
            	$msg .= "Added column: items_per_page<br>";
            }

            if (!column_exists($maia_users_cols, "spam_quarantine_sort")) {
            	add_column($test_dbh, "maia_users", "spam_quarantine_sort", "char", 2, "XA", false);
            	$msg .= "Added column: spam_quarantine_sort<br>";
            }

            if (!column_exists($maia_users_cols, "virus_quarantine_sort")) {
            	add_column($test_dbh, "maia_users", "virus_quarantine_sort", "char", 2, "DA", false);
            	$msg .= "Added column: virus_quarantine_sort<br>";
            }

            if (!column_exists($maia_users_cols, "header_quarantine_sort")) {
            	add_column($test_dbh, "maia_users", "header_quarantine_sort", "char", 2, "DA", false);
            	$msg .= "Added column: header_quarantine_sort<br>";
            }

            if (!column_exists($maia_users_cols, "attachment_quarantine_sort")) {
            	add_column($test_dbh, "maia_users", "attachment_quarantine_sort", "char", 2, "DA", false);
            	$msg .= "Added column: attachment_quarantine_sort<br>";
            }

            if (!column_exists($maia_users_cols, "ham_cache_sort")) {
            	add_column($test_dbh, "maia_users", "ham_cache_sort", "char", 2, "XD", false);
            	$msg .= "Added column: ham_cache_sort<br>";
            }

            if (!column_exists($maia_users_cols, "discard_ham")) {
            	add_column($test_dbh, "maia_users", "discard_ham", "char", 1, "N", false);
            	$msg .= "Added column: discard_ham<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_users");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check index: maia_users table
        print("<tr><td bgcolor=\"#CCCCCC\">Index: maia_users table</td>\n");
        $msg = "";
        if (!index_exists($test_dbh, "maia_users", "maia_users_idx_user_name")) {
            create_index($test_dbh, "maia_users", "user_name", true);
            $msg .= "Created index: maia_users_idx_user_name";
        }
        if (empty($msg)) {
            print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
        } else {
            print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_domains
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_domains</td>\n");
        if (table_exists($test_dbh, "maia_domains")) {
            $select = "SELECT * FROM maia_domains";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_domains_cols[] = $col["name"];
            }
            $sth->free();
            $maia_domains_cols = array_flip($maia_domains_cols);
            $msg = "";

            if (!column_exists($maia_domains_cols, "domain")) {
            	add_column($test_dbh, "maia_domains", "domain", "varchar", 255, "", false);
            	$msg .= "Added column: domain<br>";
            }

            if (!column_exists($maia_domains_cols, "reminders")) {
            	add_column($test_dbh, "maia_domains", "reminders", "char", 1, "N", false);
            	$msg .= "Added column: reminders<br>";
            }

            if (!column_exists($maia_domains_cols, "charts")) {
            	add_column($test_dbh, "maia_domains", "charts", "char", 1, "N", false);
            	$msg .= "Added column: charts<br>";
            }

            if (!column_exists($maia_domains_cols, "enable_user_autocreation")) {
            	add_column($test_dbh, "maia_domains", "enable_user_autocreation", "char", 1, "Y", false);
            	$msg .= "Added column: enable_user_autocreation<br>";
            }

            if (!column_exists($maia_domains_cols, "language")) {
            	add_column($test_dbh, "maia_domains", "language", "varchar", 10, "en", false);
            	$msg .= "Added column: language<br>";
            }

            if (!column_exists($maia_domains_cols, "charset")) {
            	add_column($test_dbh, "maia_domains", "charset", "varchar", 20, "ISO-8859-1", false);
            	$msg .= "Added column: charset<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_domains");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check index: maia_domains table
        print("<tr><td bgcolor=\"#CCCCCC\">Index: maia_domains table</td>\n");
        $msg = "";
        if (!index_exists($test_dbh, "maia_domains", "maia_domains_idx_domain")) {
            create_index($test_dbh, "maia_domains", "domain", true);
            $msg .= "Created index: maia_domains_idx_domain";
        }
        if (empty($msg)) {
            print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
        } else {
            print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_domain_admins
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_domain_admins</td>\n");
        if (table_exists($test_dbh, "maia_domain_admins")) {
            $select = "SELECT * FROM maia_domain_admins";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_domain_admins_cols[] = $col["name"];
            }
            $sth->free();
            $maia_domain_admins_cols = array_flip($maia_domain_admins_cols);
            $msg = "";

            if (!column_exists($maia_domain_admins_cols, "domain_id")) {
            	add_column($test_dbh, "maia_domain_admins", "domain_id", "uint", 0, "", false);
            	$msg .= "Added column: domain_id<br>";
            }

            if (!column_exists($maia_domain_admins_cols, "admin_id")) {
            	add_column($test_dbh, "maia_domain_admins", "admin_id", "uint", 0, "", false);
            	$msg .= "Added column: admin_id<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_domain_admins");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_stats
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_stats</td>\n");
        if (table_exists($test_dbh, "maia_stats")) {
            $select = "SELECT * FROM maia_stats";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_stats_cols[] = $col["name"];
            }
            $sth->free();
            $maia_stats_cols = array_flip($maia_stats_cols);
            $msg = "";

            if (!column_exists($maia_stats_cols, "oldest_suspected_ham_date")) {
            	add_column($test_dbh, "maia_stats", "oldest_suspected_ham_date", "timestamp", 0, "", true);
            	$msg .= "Added column: oldest_suspected_ham_date<br>";
            }

            if (!column_exists($maia_stats_cols, "newest_suspected_ham_date")) {
            	add_column($test_dbh, "maia_stats", "newest_suspected_ham_date", "timestamp", 0, "", true);
            	$msg .= "Added column: newest_suspected_ham_date<br>";
            }

            if (!column_exists($maia_stats_cols, "smallest_suspected_ham_size")) {
            	add_column($test_dbh, "maia_stats", "smallest_suspected_ham_size", "uint", 0, "0", false);
            	$msg .= "Added column: smallest_suspected_ham_size<br>";
            }

            if (!column_exists($maia_stats_cols, "largest_suspected_ham_size")) {
            	add_column($test_dbh, "maia_stats", "largest_suspected_ham_size", "uint", 0, "0", false);
            	$msg .= "Added column: largest_suspected_ham_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_suspected_ham_size")) {
            	add_column($test_dbh, "maia_stats", "total_suspected_ham_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_suspected_ham_size<br>";
            }

            if (!column_exists($maia_stats_cols, "lowest_suspected_ham_score")) {
            	add_column($test_dbh, "maia_stats", "lowest_suspected_ham_score", "float", "7,3", "0.0", false);
            	$msg .= "Added column: lowest_suspected_ham_score<br>";
            }

            if (!column_exists($maia_stats_cols, "highest_suspected_ham_score")) {
            	add_column($test_dbh, "maia_stats", "highest_suspected_ham_score", "float", "7,3", "0.0", false);
            	$msg .= "Added column: highest_suspected_ham_score<br>";
            }

            if (!column_exists($maia_stats_cols, "total_suspected_ham_score")) {
            	add_column($test_dbh, "maia_stats", "total_suspected_ham_score", "float", "14,3", "0.0", false);
            	$msg .= "Added column: total_suspected_ham_score<br>";
            }

            if (!column_exists($maia_stats_cols, "total_suspected_ham_items")) {
            	add_column($test_dbh, "maia_stats", "total_suspected_ham_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_suspected_ham_items<br>";
            }

            if (!column_exists($maia_stats_cols, "oldest_ham_date")) {
            	add_column($test_dbh, "maia_stats", "oldest_ham_date", "timestamp", 0, "", true);
            	$msg .= "Added column: oldest_ham_date<br>";
            }

            if (!column_exists($maia_stats_cols, "newest_ham_date")) {
            	add_column($test_dbh, "maia_stats", "newest_ham_date", "timestamp", 0, "", true);
            	$msg .= "Added column: newest_ham_date<br>";
            }

            if (!column_exists($maia_stats_cols, "smallest_ham_size")) {
            	add_column($test_dbh, "maia_stats", "smallest_ham_size", "uint", 0, "0", false);
            	$msg .= "Added column: smallest_ham_size<br>";
            }

            if (!column_exists($maia_stats_cols, "largest_ham_size")) {
            	add_column($test_dbh, "maia_stats", "largest_ham_size", "uint", 0, "0", false);
            	$msg .= "Added column: largest_ham_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_ham_size")) {
            	add_column($test_dbh, "maia_stats", "total_ham_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_ham_size<br>";
            }

            if (!column_exists($maia_stats_cols, "lowest_ham_score")) {
            	add_column($test_dbh, "maia_stats", "lowest_ham_score", "float", "7,3", "0.0", false);
            	$msg .= "Added column: lowest_ham_score<br>";
            }

            if (!column_exists($maia_stats_cols, "highest_ham_score")) {
            	add_column($test_dbh, "maia_stats", "highest_ham_score", "float", "7,3", "0.0", false);
            	$msg .= "Added column: highest_ham_score<br>";
            }

            if (!column_exists($maia_stats_cols, "total_ham_score")) {
            	add_column($test_dbh, "maia_stats", "total_ham_score", "float", "14,3", "0.0", false);
            	$msg .= "Added column: total_ham_score<br>";
            }

            if (!column_exists($maia_stats_cols, "total_ham_items")) {
            	add_column($test_dbh, "maia_stats", "total_ham_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_ham_items<br>";
            }

            if (!column_exists($maia_stats_cols, "oldest_wl_date")) {
            	add_column($test_dbh, "maia_stats", "oldest_wl_date", "timestamp", 0, "", true);
            	$msg .= "Added column: oldest_wl_date<br>";
            }

            if (!column_exists($maia_stats_cols, "newest_wl_date")) {
            	add_column($test_dbh, "maia_stats", "newest_wl_date", "timestamp", 0, "", true);
            	$msg .= "Added column: newest_wl_date<br>";
            }

            if (!column_exists($maia_stats_cols, "smallest_wl_size")) {
            	add_column($test_dbh, "maia_stats", "smallest_wl_size", "uint", 0, "0", false);
            	$msg .= "Added column: smallest_wl_size<br>";
            }

            if (!column_exists($maia_stats_cols, "largest_wl_size")) {
            	add_column($test_dbh, "maia_stats", "largest_wl_size", "uint", 0, "0", false);
            	$msg .= "Added column: largest_wl_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_wl_size")) {
            	add_column($test_dbh, "maia_stats", "total_wl_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_wl_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_wl_items")) {
            	add_column($test_dbh, "maia_stats", "total_wl_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_wl_items<br>";
            }

            if (!column_exists($maia_stats_cols, "oldest_bl_date")) {
            	add_column($test_dbh, "maia_stats", "oldest_bl_date", "timestamp", 0, "", true);
            	$msg .= "Added column: oldest_bl_date<br>";
            }

            if (!column_exists($maia_stats_cols, "newest_bl_date")) {
            	add_column($test_dbh, "maia_stats", "newest_bl_date", "timestamp", 0, "", true);
            	$msg .= "Added column: newest_bl_date<br>";
            }

            if (!column_exists($maia_stats_cols, "smallest_bl_size")) {
            	add_column($test_dbh, "maia_stats", "smallest_bl_size", "uint", 0, "0", false);
            	$msg .= "Added column: smallest_bl_size<br>";
            }

            if (!column_exists($maia_stats_cols, "largest_bl_size")) {
            	add_column($test_dbh, "maia_stats", "largest_bl_size", "uint", 0, "0", false);
            	$msg .= "Added column: largest_bl_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_bl_size")) {
            	add_column($test_dbh, "maia_stats", "total_bl_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_bl_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_bl_items")) {
            	add_column($test_dbh, "maia_stats", "total_bl_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_bl_items<br>";
            }

            if (!column_exists($maia_stats_cols, "oldest_suspected_spam_date")) {
            	add_column($test_dbh, "maia_stats", "oldest_suspected_spam_date", "timestamp", 0, "", true);
            	$msg .= "Added column: oldest_suspected_spam_date<br>";
            }

            if (!column_exists($maia_stats_cols, "newest_suspected_spam_date")) {
            	add_column($test_dbh, "maia_stats", "newest_suspected_spam_date", "timestamp", 0, "", true);
            	$msg .= "Added column: newest_suspected_spam_date<br>";
            }

            if (!column_exists($maia_stats_cols, "smallest_suspected_spam_size")) {
            	add_column($test_dbh, "maia_stats", "smallest_suspected_spam_size", "uint", 0, "0", false);
            	$msg .= "Added column: smallest_suspected_spam_size<br>";
            }

            if (!column_exists($maia_stats_cols, "largest_suspected_spam_size")) {
            	add_column($test_dbh, "maia_stats", "largest_suspected_spam_size", "uint", 0, "0", false);
            	$msg .= "Added column: largest_suspected_spam_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_suspected_spam_size")) {
            	add_column($test_dbh, "maia_stats", "total_suspected_spam_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_suspected_spam_size<br>";
            }

            if (!column_exists($maia_stats_cols, "lowest_suspected_spam_score")) {
            	add_column($test_dbh, "maia_stats", "lowest_suspected_spam_score", "float", "7,3", "0.0", false);
            	$msg .= "Added column: lowest_suspected_spam_score<br>";
            }

            if (!column_exists($maia_stats_cols, "highest_suspected_spam_score")) {
            	add_column($test_dbh, "maia_stats", "highest_suspected_spam_score", "float", "7,3", "0.0", false);
            	$msg .= "Added column: highest_suspected_spam_score<br>";
            }

            if (!column_exists($maia_stats_cols, "total_suspected_spam_score")) {
            	add_column($test_dbh, "maia_stats", "total_suspected_spam_score", "float", "14,3", "0.0", false);
            	$msg .= "Added column: total_suspected_spam_score<br>";
            }

            if (!column_exists($maia_stats_cols, "total_suspected_spam_items")) {
            	add_column($test_dbh, "maia_stats", "total_suspected_spam_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_suspected_spam_items<br>";
            }

            if (!column_exists($maia_stats_cols, "oldest_fp_date")) {
            	add_column($test_dbh, "maia_stats", "oldest_fp_date", "timestamp", 0, "", true);
            	$msg .= "Added column: oldest_fp_date<br>";
            }

            if (!column_exists($maia_stats_cols, "newest_fp_date")) {
            	add_column($test_dbh, "maia_stats", "newest_fp_date", "timestamp", 0, "", true);
            	$msg .= "Added column: newest_fp_date<br>";
            }

            if (!column_exists($maia_stats_cols, "smallest_fp_size")) {
            	add_column($test_dbh, "maia_stats", "smallest_fp_size", "uint", 0, "0", false);
            	$msg .= "Added column: smallest_fp_size<br>";
            }

            if (!column_exists($maia_stats_cols, "largest_fp_size")) {
            	add_column($test_dbh, "maia_stats", "largest_fp_size", "uint", 0, "0", false);
            	$msg .= "Added column: largest_fp_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_fp_size")) {
            	add_column($test_dbh, "maia_stats", "total_fp_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_fp_size<br>";
            }

            if (!column_exists($maia_stats_cols, "lowest_fp_score")) {
            	add_column($test_dbh, "maia_stats", "lowest_fp_score", "float", "7,3", "0.0", false);
            	$msg .= "Added column: lowest_fp_score<br>";
            }

            if (!column_exists($maia_stats_cols, "highest_fp_score")) {
            	add_column($test_dbh, "maia_stats", "highest_fp_score", "float", "7,3", "0.0", false);
            	$msg .= "Added column: highest_fp_score<br>";
            }

            if (!column_exists($maia_stats_cols, "total_fp_score")) {
            	add_column($test_dbh, "maia_stats", "total_fp_score", "float", "14,3", "0.0", false);
            	$msg .= "Added column: total_fp_score<br>";
            }

            if (!column_exists($maia_stats_cols, "total_fp_items")) {
            	add_column($test_dbh, "maia_stats", "total_fp_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_fp_items<br>";
            }

            if (!column_exists($maia_stats_cols, "oldest_fn_date")) {
            	add_column($test_dbh, "maia_stats", "oldest_fn_date", "timestamp", 0, "", true);
            	$msg .= "Added column: oldest_fn_date<br>";
            }

            if (!column_exists($maia_stats_cols, "newest_fn_date")) {
            	add_column($test_dbh, "maia_stats", "newest_fn_date", "timestamp", 0, "", true);
            	$msg .= "Added column: newest_fn_date<br>";
            }

            if (!column_exists($maia_stats_cols, "smallest_fn_size")) {
            	add_column($test_dbh, "maia_stats", "smallest_fn_size", "uint", 0, "0", false);
            	$msg .= "Added column: smallest_fn_size<br>";
            }

            if (!column_exists($maia_stats_cols, "largest_fn_size")) {
            	add_column($test_dbh, "maia_stats", "largest_fn_size", "uint", 0, "0", false);
            	$msg .= "Added column: largest_fn_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_fn_size")) {
            	add_column($test_dbh, "maia_stats", "total_fn_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_fn_size<br>";
            }

            if (!column_exists($maia_stats_cols, "lowest_fn_score")) {
            	add_column($test_dbh, "maia_stats", "lowest_fn_score", "float", "7,3", "0.0", false);
            	$msg .= "Added column: lowest_fn_score<br>";
            }

            if (!column_exists($maia_stats_cols, "highest_fn_score")) {
            	add_column($test_dbh, "maia_stats", "highest_fn_score", "float", "7,3", "0.0", false);
            	$msg .= "Added column: highest_fn_score<br>";
            }

            if (!column_exists($maia_stats_cols, "total_fn_score")) {
            	add_column($test_dbh, "maia_stats", "total_fn_score", "float", "14,3", "0.0", false);
            	$msg .= "Added column: total_fn_score<br>";
            }

            if (!column_exists($maia_stats_cols, "total_fn_items")) {
            	add_column($test_dbh, "maia_stats", "total_fn_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_fn_items<br>";
            }

            if (!column_exists($maia_stats_cols, "oldest_spam_date")) {
            	add_column($test_dbh, "maia_stats", "oldest_spam_date", "timestamp", 0, "", true);
            	$msg .= "Added column: oldest_spam_date<br>";
            }

            if (!column_exists($maia_stats_cols, "newest_spam_date")) {
            	add_column($test_dbh, "maia_stats", "newest_spam_date", "timestamp", 0, "", true);
            	$msg .= "Added column: newest_spam_date<br>";
            }

            if (!column_exists($maia_stats_cols, "smallest_spam_size")) {
            	add_column($test_dbh, "maia_stats", "smallest_spam_size", "uint", 0, "0", false);
            	$msg .= "Added column: smallest_spam_size<br>";
            }

            if (!column_exists($maia_stats_cols, "largest_spam_size")) {
            	add_column($test_dbh, "maia_stats", "largest_spam_size", "uint", 0, "0", false);
            	$msg .= "Added column: largest_spam_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_spam_size")) {
            	add_column($test_dbh, "maia_stats", "total_spam_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_spam_size<br>";
            }

            if (!column_exists($maia_stats_cols, "lowest_spam_score")) {
            	add_column($test_dbh, "maia_stats", "lowest_spam_score", "float", "7,3", "0.0", false);
            	$msg .= "Added column: lowest_spam_score<br>";
            }

            if (!column_exists($maia_stats_cols, "highest_spam_score")) {
            	add_column($test_dbh, "maia_stats", "highest_spam_score", "float", "7,3", "0.0", false);
            	$msg .= "Added column: highest_spam_score<br>";
            }

            if (!column_exists($maia_stats_cols, "total_spam_score")) {
            	add_column($test_dbh, "maia_stats", "total_spam_score", "float", "14,3", "0.0", false);
            	$msg .= "Added column: total_spam_score<br>";
            }

            if (!column_exists($maia_stats_cols, "total_spam_items")) {
            	add_column($test_dbh, "maia_stats", "total_spam_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_spam_items<br>";
            }

            if (!column_exists($maia_stats_cols, "oldest_virus_date")) {
            	add_column($test_dbh, "maia_stats", "oldest_virus_date", "timestamp", 0, "", true);
            	$msg .= "Added column: oldest_virus_date<br>";
            }

            if (!column_exists($maia_stats_cols, "newest_virus_date")) {
            	add_column($test_dbh, "maia_stats", "newest_virus_date", "timestamp", 0, "", true);
            	$msg .= "Added column: newest_virus_date<br>";
            }

            if (!column_exists($maia_stats_cols, "smallest_virus_size")) {
            	add_column($test_dbh, "maia_stats", "smallest_virus_size", "uint", 0, "0", false);
            	$msg .= "Added column: smallest_virus_size<br>";
            }

            if (!column_exists($maia_stats_cols, "largest_virus_size")) {
            	add_column($test_dbh, "maia_stats", "largest_virus_size", "uint", 0, "0", false);
            	$msg .= "Added column: largest_virus_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_virus_size")) {
            	add_column($test_dbh, "maia_stats", "total_virus_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_virus_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_virus_items")) {
            	add_column($test_dbh, "maia_stats", "total_virus_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_virus_items<br>";
            }

            if (!column_exists($maia_stats_cols, "oldest_bad_header_date")) {
            	add_column($test_dbh, "maia_stats", "oldest_bad_header_date", "timestamp", 0, "", true);
            	$msg .= "Added column: oldest_bad_header_date<br>";
            }

            if (!column_exists($maia_stats_cols, "newest_bad_header_date")) {
            	add_column($test_dbh, "maia_stats", "newest_bad_header_date", "timestamp", 0, "", true);
            	$msg .= "Added column: newest_bad_header_date<br>";
            }

            if (!column_exists($maia_stats_cols, "smallest_bad_header_size")) {
            	add_column($test_dbh, "maia_stats", "smallest_bad_header_size", "uint", 0, "0", false);
            	$msg .= "Added column: smallest_bad_header_size<br>";
            }

            if (!column_exists($maia_stats_cols, "largest_bad_header_size")) {
            	add_column($test_dbh, "maia_stats", "largest_bad_header_size", "uint", 0, "0", false);
            	$msg .= "Added column: largest_bad_header_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_bad_header_size")) {
            	add_column($test_dbh, "maia_stats", "total_bad_header_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_bad_header_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_bad_header_items")) {
            	add_column($test_dbh, "maia_stats", "total_bad_header_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_bad_header_items<br>";
            }

            if (!column_exists($maia_stats_cols, "oldest_banned_file_date")) {
            	add_column($test_dbh, "maia_stats", "oldest_banned_file_date", "timestamp", 0, "", true);
            	$msg .= "Added column: oldest_banned_file_date<br>";
            }

            if (!column_exists($maia_stats_cols, "newest_banned_file_date")) {
            	add_column($test_dbh, "maia_stats", "newest_banned_file_date", "timestamp", 0, "", true);
            	$msg .= "Added column: newest_banned_file_date<br>";
            }

            if (!column_exists($maia_stats_cols, "smallest_banned_file_size")) {
            	add_column($test_dbh, "maia_stats", "smallest_banned_file_size", "uint", 0, "0", false);
            	$msg .= "Added column: smallest_banned_file_size<br>";
            }

            if (!column_exists($maia_stats_cols, "largest_banned_file_size")) {
            	add_column($test_dbh, "maia_stats", "largest_banned_file_size", "uint", 0, "0", false);
            	$msg .= "Added column: largest_banned_file_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_banned_file_size")) {
            	add_column($test_dbh, "maia_stats", "total_banned_file_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_banned_file_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_banned_file_items")) {
            	add_column($test_dbh, "maia_stats", "total_banned_file_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_banned_file_items<br>";
            }

            if (!column_exists($maia_stats_cols, "oldest_oversized_date")) {
            	add_column($test_dbh, "maia_stats", "oldest_oversized_date", "timestamp", 0, "", true);
            	$msg .= "Added column: oldest_oversized_date<br>";
            }

            if (!column_exists($maia_stats_cols, "newest_oversized_date")) {
            	add_column($test_dbh, "maia_stats", "newest_oversized_date", "timestamp", 0, "", true);
            	$msg .= "Added column: newest_oversized_date<br>";
            }

            if (!column_exists($maia_stats_cols, "smallest_oversized_size")) {
            	add_column($test_dbh, "maia_stats", "smallest_oversized_size", "uint", 0, "0", false);
            	$msg .= "Added column: smallest_oversized_size<br>";
            }

            if (!column_exists($maia_stats_cols, "largest_oversized_size")) {
            	add_column($test_dbh, "maia_stats", "largest_oversized_size", "uint", 0, "0", false);
            	$msg .= "Added column: largest_oversized_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_oversized_size")) {
            	add_column($test_dbh, "maia_stats", "total_oversized_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_oversized_size<br>";
            }

            if (!column_exists($maia_stats_cols, "total_oversized_items")) {
            	add_column($test_dbh, "maia_stats", "total_oversized_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_oversized_items<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_stats");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_mail
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_mail</td>\n");
        if (table_exists($test_dbh, "maia_mail")) {
            $select = "SELECT * FROM maia_mail";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_mail_cols[] = $col["name"];
            }
            $sth->free();
            $maia_mail_cols = array_flip($maia_mail_cols);
            $msg = "";

            if (!column_exists($maia_mail_cols, "received_date")) {
            	add_column($test_dbh, "maia_mail", "received_date", "timestamp", 0, "", false);
            	$msg .= "Added column: received_date<br>";
            }

            if (!column_exists($maia_mail_cols, "size")) {
            	add_column($test_dbh, "maia_mail", "size", "uint", 0, "", false);
            	$msg .= "Added column: size<br>";
            }

            if (!column_exists($maia_mail_cols, "sender_email")) {
            	add_column($test_dbh, "maia_mail", "sender_email", "varchar", 255, "", false);
            	$msg .= "Added column: sender_email<br>";
            }

            if (!column_exists($maia_mail_cols, "envelope_to")) {
            	add_column($test_dbh, "maia_mail", "envelope_to", "text", 0, "", false);
            	$msg .= "Added column: envelope_to<br>";
            }

            if (!column_exists($maia_mail_cols, "subject")) {
            	add_column($test_dbh, "maia_mail", "subject", "varchar", 255, "", false);
            	$msg .= "Added column: subject<br>";
            }

            if (!column_exists($maia_mail_cols, "contents")) {
            	add_column($test_dbh, "maia_mail", "contents", "ltext", 0, "", false);
            	$msg .= "Added column: contents<br>";
            }

            if (!column_exists($maia_mail_cols, "score")) {
            	add_column($test_dbh, "maia_mail", "score", "float", "7,3", "", true);
            	$msg .= "Added column: score<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_mail");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check index: maia_mail table
        print("<tr><td bgcolor=\"#CCCCCC\">Index: maia_mail table</td>\n");
        $msg = "";
        if (!index_exists($test_dbh, "maia_mail", "maia_mail_idx_received_date")) {
            create_index($test_dbh, "maia_mail", "received_date", false);
            $msg .= "Created index: maia_mail_idx_received_date";
        }
        if (empty($msg)) {
            print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
        } else {
            print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_mail_recipients
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_mail_recipients</td>\n");
        if (table_exists($test_dbh, "maia_mail_recipients")) {
            $select = "SELECT * FROM maia_mail_recipients";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_mail_recipients_cols[] = $col["name"];
            }
            $sth->free();
            $maia_mail_recipients_cols = array_flip($maia_mail_recipients_cols);
            $msg = "";

            if (!column_exists($maia_mail_recipients_cols, "mail_id")) {
            	add_column($test_dbh, "maia_mail_recipients", "mail_id", "uint", 0, "", false);
            	$msg .= "Added column: mail_id<br>";
            }

            if (!column_exists($maia_mail_recipients_cols, "recipient_id")) {
            	add_column($test_dbh, "maia_mail_recipients", "recipient_id", "uint", 0, "", false);
            	$msg .= "Added column: recipient_id<br>";
            }

            if (!column_exists($maia_mail_recipients_cols, "type")) {
            	add_column($test_dbh, "maia_mail_recipients", "type", "char", 1, "", false);
            	$msg .= "Added column: type<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_mail_recipients");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check index: maia_mail_recipients table
        print("<tr><td bgcolor=\"#CCCCCC\">Index: maia_mail_recipients table</td>\n");
        $msg = "";
        if (!index_exists($test_dbh, "maia_mail_recipients", "maia_mail_recipients_idx_type")) {
            create_index($test_dbh, "maia_mail_recipients", "type", false);
            $msg .= "Created index: maia_mail_recipients_idx_type<br>";
        }
        if (!index_exists($test_dbh, "maia_mail_recipients", "maia_mail_recipients_idx_mail_id")) {
            create_index($test_dbh, "maia_mail_recipients", "mail_id", false);
            $msg .= "Created index: maia_mail_recipients_idx_mail_id<br>";
        }
        if (empty($msg)) {
            print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
        } else {
            print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_viruses
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_viruses</td>\n");
        if (table_exists($test_dbh, "maia_viruses")) {
            $select = "SELECT * FROM maia_viruses";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_viruses_cols[] = $col["name"];
            }
            $sth->free();
            $maia_viruses_cols = array_flip($maia_viruses_cols);
            $msg = "";

            if (!column_exists($maia_viruses_cols, "virus_name")) {
            	add_column($test_dbh, "maia_viruses", "virus_name", "varchar", 255, "", false);
            	$msg .= "Added column: virus_name<br>";
            }

            if (!column_exists($maia_viruses_cols, "count")) {
            	add_column($test_dbh, "maia_viruses", "count", "uint", 0, "0", false);
            	$msg .= "Added column: count<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_viruses");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check index: maia_viruses table
        print("<tr><td bgcolor=\"#CCCCCC\">Index: maia_viruses table</td>\n");
        $msg = "";
        if (!index_exists($test_dbh, "maia_viruses", "maia_viruses_idx_virus_name")) {
            create_index($test_dbh, "maia_viruses", "virus_name", true);
            $msg .= "Created index: maia_viruses_idx_virus_name<br>";
        }
        if (empty($msg)) {
            print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
        } else {
            print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_virus_aliases
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_virus_aliases</td>\n");
        if (table_exists($test_dbh, "maia_virus_aliases")) {
            $select = "SELECT * FROM maia_virus_aliases";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_virus_aliases_cols[] = $col["name"];
            }
            $sth->free();
            $maia_virus_aliases_cols = array_flip($maia_virus_aliases_cols);
            $msg = "";

            if (!column_exists($maia_virus_aliases_cols, "virus_id")) {
            	add_column($test_dbh, "maia_virus_aliases", "virus_id", "uint", 0, "", false);
            	$msg .= "Added column: virus_id<br>";
            }

            if (!column_exists($maia_virus_aliases_cols, "virus_alias")) {
            	add_column($test_dbh, "maia_virus_aliases", "virus_alias", "varchar", 255, "", false);
            	$msg .= "Added column: virus_alias<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_virus_aliases");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_viruses_detected
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_viruses_detected</td>\n");
        if (table_exists($test_dbh, "maia_viruses_detected")) {
            $select = "SELECT * FROM maia_viruses_detected";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_viruses_detected_cols[] = $col["name"];
            }
            $sth->free();
            $maia_viruses_detected_cols = array_flip($maia_viruses_detected_cols);
            $msg = "";

            if (!column_exists($maia_viruses_detected_cols, "mail_id")) {
            	add_column($test_dbh, "maia_viruses_detected", "mail_id", "uint", 0, "", false);
            	$msg .= "Added column: mail_id<br>";
            }

            if (!column_exists($maia_viruses_detected_cols, "virus_id")) {
            	add_column($test_dbh, "maia_viruses_detected", "virus_id", "uint", 0, "", false);
            	$msg .= "Added column: virus_id<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_viruses_detected");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_sa_rules
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_sa_rules</td>\n");
        if (table_exists($test_dbh, "maia_sa_rules")) {
            $select = "SELECT * FROM maia_sa_rules";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_sa_rules_cols[] = $col["name"];
            }
            $sth->free();
            $maia_sa_rules_cols = array_flip($maia_sa_rules_cols);
            $msg = "";

            if (!column_exists($maia_sa_rules_cols, "rule_name")) {
            	add_column($test_dbh, "maia_sa_rules", "rule_name", "varchar", 255, "", false);
            	$msg .= "Added column: rule_name<br>";
            }

            if (!column_exists($maia_sa_rules_cols, "rule_description")) {
            	add_column($test_dbh, "maia_sa_rules", "rule_description", "varchar", 255, "", false);
            	$msg .= "Added column: rule_description<br>";
            }

            if (!column_exists($maia_sa_rules_cols, "rule_score_0")) {
            	add_column($test_dbh, "maia_sa_rules", "rule_score_0", "float", "7,3", "1.0", false);
            	$msg .= "Added column: rule_score_0<br>";
            }

            if (!column_exists($maia_sa_rules_cols, "rule_score_1")) {
            	add_column($test_dbh, "maia_sa_rules", "rule_score_1", "float", "7,3", "1.0", false);
            	$msg .= "Added column: rule_score_1<br>";
            }

            if (!column_exists($maia_sa_rules_cols, "rule_score_2")) {
            	add_column($test_dbh, "maia_sa_rules", "rule_score_2", "float", "7,3", "1.0", false);
            	$msg .= "Added column: rule_score_2<br>";
            }

            if (!column_exists($maia_sa_rules_cols, "rule_score_3")) {
            	add_column($test_dbh, "maia_sa_rules", "rule_score_3", "float", "7,3", "1.0", false);
            	$msg .= "Added column: rule_score_3<br>";
            }

            if (!column_exists($maia_sa_rules_cols, "rule_count")) {
            	add_column($test_dbh, "maia_sa_rules", "rule_count", "uint", 0, "0", false);
            	$msg .= "Added column: rule_count<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_sa_rules");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check index: maia_sa_rules table
        print("<tr><td bgcolor=\"#CCCCCC\">Index: maia_sa_rules table</td>\n");
        $msg = "";
        if (!index_exists($test_dbh, "maia_sa_rules", "maia_sa_rules_idx_rule_name")) {
            create_index($test_dbh, "maia_sa_rules", "rule_name", true);
            $msg .= "Created index: maia_sa_rules_idx_rule_name<br>";
        }
        if (empty($msg)) {
            print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
        } else {
            print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_sa_rules_triggered
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_sa_rules_triggered</td>\n");
        if (table_exists($test_dbh, "maia_sa_rules_triggered")) {
            $select = "SELECT * FROM maia_sa_rules_triggered";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_sa_rules_triggered_cols[] = $col["name"];
            }
            $sth->free();
            $maia_sa_rules_triggered_cols = array_flip($maia_sa_rules_triggered_cols);
            $msg = "";

            if (!column_exists($maia_sa_rules_triggered_cols, "mail_id")) {
            	add_column($test_dbh, "maia_sa_rules_triggered", "mail_id", "uint", 0, "", false);
            	$msg .= "Added column: mail_id<br>";
            }

            if (!column_exists($maia_sa_rules_triggered_cols, "rule_id")) {
            	add_column($test_dbh, "maia_sa_rules_triggered", "rule_id", "uint", 0, "", false);
            	$msg .= "Added column: rule_id<br>";
            }

            if (!column_exists($maia_sa_rules_triggered_cols, "rule_score")) {
            	add_column($test_dbh, "maia_sa_rules_triggered", "rule_score", "float", "7,3", "0.0", false);
            	$msg .= "Added column: rule_score<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_sa_rules_triggered");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_banned_attachments_found
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_banned_attachments_found</td>\n");
        if (table_exists($test_dbh, "maia_banned_attachments_found")) {
            $select = "SELECT * FROM maia_banned_attachments_found";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_banned_attachments_found_cols[] = $col["name"];
            }
            $sth->free();
            $maia_banned_attachments_found_cols = array_flip($maia_banned_attachments_found_cols);
            $msg = "";

            if (!column_exists($maia_banned_attachments_found_cols, "mail_id")) {
            	add_column($test_dbh, "maia_banned_attachments_found", "mail_id", "uint", 0, "", false);
            	$msg .= "Added column: mail_id<br>";
            }

            if (!column_exists($maia_banned_attachments_found_cols, "file_name")) {
            	add_column($test_dbh, "maia_banned_attachments_found", "file_name", "varchar", 255, "", false);
            	$msg .= "Added column: file_name<br>";
            }

            if (!column_exists($maia_banned_attachments_found_cols, "file_type")) {
            	add_column($test_dbh, "maia_banned_attachments_found", "file_type", "varchar", 20, "Unknown", false);
            	$msg .= "Added column: file_type<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_banned_attachments_found");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");


        // Check table: maia_stats_history
        print("<tr><td bgcolor=\"#CCCCCC\">Table: maia_stats_history</td>\n");
        if (table_exists($test_dbh, "maia_stats_history")) {
            $select = "SELECT * FROM maia_stats_history";
            $sth = $test_dbh->query($select);
            $info = $sth->tableInfo();
            foreach ($info as $col) {
    	        $maia_stats_history_cols[] = $col["name"];
            }
            $sth->free();
            $maia_stats_history_cols = array_flip($maia_stats_history_cols);
            $msg = "";

            if (!column_exists($maia_stats_history_cols, "user_id")) {
            	add_column($test_dbh, "maia_stats_history", "user_id", "uint", 0, "", false);
            	$msg .= "Added column: user_id<br>";
            }

            if (!column_exists($maia_stats_history_cols, "type")) {
            	add_column($test_dbh, "maia_stats_history", "type", "char", 1, "H", false);
            	$msg .= "Added column: type<br>";
            }

            if (!column_exists($maia_stats_history_cols, "taken_at")) {
            	add_column($test_dbh, "maia_stats_history", "taken_at", "timestamp", 0, "", false);
            	$msg .= "Added column: taken_at<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_ham_items")) {
            	add_column($test_dbh, "maia_stats_history", "total_ham_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_ham_items<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_ham_size")) {
            	add_column($test_dbh, "maia_stats_history", "total_ham_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_ham_size<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_spam_items")) {
            	add_column($test_dbh, "maia_stats_history", "total_spam_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_spam_items<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_spam_size")) {
            	add_column($test_dbh, "maia_stats_history", "total_spam_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_spam_size<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_virus_items")) {
            	add_column($test_dbh, "maia_stats_history", "total_virus_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_virus_items<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_virus_size")) {
            	add_column($test_dbh, "maia_stats_history", "total_virus_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_virus_size<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_fp_items")) {
            	add_column($test_dbh, "maia_stats_history", "total_fp_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_fp_items<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_fp_size")) {
            	add_column($test_dbh, "maia_stats_history", "total_fp_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_fp_size<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_fn_items")) {
            	add_column($test_dbh, "maia_stats_history", "total_fn_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_fn_items<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_fn_size")) {
            	add_column($test_dbh, "maia_stats_history", "total_fn_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_fn_size<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_banned_file_items")) {
            	add_column($test_dbh, "maia_stats_history", "total_banned_file_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_banned_file_items<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_banned_file_size")) {
            	add_column($test_dbh, "maia_stats_history", "total_banned_file_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_banned_file_size<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_bad_header_items")) {
            	add_column($test_dbh, "maia_stats_history", "total_bad_header_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_bad_header_items<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_bad_header_size")) {
            	add_column($test_dbh, "maia_stats_history", "total_bad_header_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_bad_header_size<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_wl_items")) {
            	add_column($test_dbh, "maia_stats_history", "total_wl_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_wl_items<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_wl_size")) {
            	add_column($test_dbh, "maia_stats_history", "total_wl_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_wl_size<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_bl_items")) {
            	add_column($test_dbh, "maia_stats_history", "total_bl_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_bl_items<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_bl_size")) {
            	add_column($test_dbh, "maia_stats_history", "total_bl_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_bl_size<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_oversized_items")) {
            	add_column($test_dbh, "maia_stats_history", "total_oversized_items", "uint", 0, "0", false);
            	$msg .= "Added column: total_oversized_items<br>";
            }

            if (!column_exists($maia_stats_history_cols, "total_oversized_size")) {
            	add_column($test_dbh, "maia_stats_history", "total_oversized_size", "uint", 0, "0", false);
            	$msg .= "Added column: total_oversized_size<br>";
            }

            if (empty($msg)) {
                print("<td bgcolor=\"#00DD00\"><b>OK</b></td>\n");
            } else {
            	print("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
            }

        } else {
            create_table($test_dbh, "maia_stats_history");
            $msg = "Created table";
            print ("<td bgcolor=\"#DDDD00\">" . $msg . "</td>\n");
        }
        print("</tr>\n");

    }

    print("</table></div>\n");

?>