<?php
    /*
     * $Id: display.php,v 1.4 2004/07/01 10:58:17 rjl Exp $
     *
     * MAIA MAILGUARD LICENSE v.1.0
     *
     * Copyright 2004 by Robert LeBlanc <rjl@renaissoft.com>
     * All rights reserved.
     *
     * PREAMBLE
     *
     * This License is designed for users of Maia Mailguard
     * ("the Software") who wish to support the Maia Mailguard project by
     * leaving "Maia Mailguard" branding information in the HTML output
     * of the pages generated by the Software, and providing links back
     * to the Maia Mailguard home page.  Users who wish to remove this
     * branding information should contact the copyright owner to obtain
     * a Rebranding License.
     *
     * DEFINITION OF TERMS
     *
     * The "Software" refers to Maia Mailguard, including all of the
     * associated PHP, Perl, and SQL scripts, documentation files, graphic
     * icons and logo images.
     *
     * GRANT OF LICENSE
     *
     * Redistribution and use in source and binary forms, with or without
     * modification, are permitted provided that the following conditions
     * are met:
     *
     * 1. Redistributions of source code must retain the above copyright
     *    notice, this list of conditions and the following disclaimer.
     *
     * 2. Redistributions in binary form must reproduce the above copyright
     *    notice, this list of conditions and the following disclaimer in the
     *    documentation and/or other materials provided with the distribution.
     *
     * 3. The end-user documentation included with the redistribution, if
     *    any, must include the following acknowledgment:
     *
     *    "This product includes software developed by Robert LeBlanc
     *    <rjl@renaissoft.com>."
     *
     *    Alternately, this acknowledgment may appear in the software itself,
     *    if and wherever such third-party acknowledgments normally appear.
     *
     * 4. At least one of the following branding conventions must be used:
     *
     *    a. The Maia Mailguard logo appears in the page-top banner of
     *       all HTML output pages in an unmodified form, and links
     *       directly to the Maia Mailguard home page; or
     *
     *    b. The "Powered by Maia Mailguard" graphic appears in the HTML
     *       output of all gateway pages that lead to this software,
     *       linking directly to the Maia Mailguard home page; or
     *
     *    c. A separate Rebranding License is obtained from the copyright
     *       owner, exempting the Licensee from 4(a) and 4(b), subject to
     *       the additional conditions laid out in that license document.
     *
     * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS
     * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
     * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
     * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
     * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
     * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
     * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
     * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
     * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
     * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
     * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
     *
     */

   require_once ("config.php");
   require_once ("constants.php");
   require_once ("db.php");
   require_once ("virus.php");


   /*
    * display_html_head(): Prints common HTML head part beginning with <html>
    *                      and ending with <body>
    */
   function display_html_head($subtitle, $display_language, $html_charset)
   {
      $banner_title = get_config_value("banner_title");
      print("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"\n");
      print("                      \"http://www.w3.org/TR/html4/loose.dtd\">\n");
      print("<html>\n");
      print("<head>\n");
      print("<title>");
      print($banner_title . ": " . $subtitle);
      print("</title>\n");
      print("<meta name=\"author\" content=\"Robert LeBlanc\">\n");
      print("<meta name=\"copyright\" content=\"Copyright 2004 Robert LeBlanc, All Rights Reserved\">\n");
      print("<meta name=\"description\" content=\"Maia Mailguard, a spam and virus management system for mail servers\">\n");
      print("<meta name=\"keywords\" lang=\"en\" content=\"maia,amavis,virus,spam,spamassassin,mail\">\n");
      print("<meta http-equiv=\"cache-control\" content=\"no-cache\">\n");
      print("<meta http-equiv=\"pragma\" content=\"no-cache\">\n");
      print("<meta http-equiv=\"expires\" content=\"0\">\n");
      print("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=$html_charset\">\n");
      print("<meta name=\"googlebot\" content=\"noarchive\">\n");
      print("<meta name=\"robots\" content=\"none\">\n");
      print("<link rel=\"stylesheet\" TYPE=\"text/css\" HREF=\"locale/$display_language/style.css\">\n");
      print("<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"favicon.ico\">\n");
      print("<script language=\"JavaScript\" type=\"text/javascript\" src=\"tooltips.js\"></script>\n");
      print("</head>\n");
      print("<body>\n");
      print("<div id=\"tooltip\" style=\"position:absolute;visibility:hidden\"></div>\n");
   }


   function display_html_foot()
   {
      print("</body>\n");
      print("</html>\n");
   }


   /*
    * display_banner(): Displays a multi-line page-top banner consisting
    *                   of a formatted table.
    *
    *                   $banner_title (from the settings.php file) will
    *                   be displayed on the top line.
    *
    *                   $subtitle will be displayed on the second line.
    *
    *                   The third line will display information about
    *                   the current user (or impersonated user, if an
    *                   administrator is masquerading).
    *
    *                   The fourth line will display a menu bar of options
    *                   appropriate to the status of the user.
    *
    *                   $subtitle - text string
    *                   $uid - user's ID number
    *                   $euid - user's effective ID number
    *                   $sid - session ID string
    *                   $showmenu - boolean, determines whether the third
    *                               and fourth lines are displayed.
    */
   function display_banner($subtitle, $uid, $euid, $sid, $showmenu)
   {
      global $dbh;
      global $lang_menu_stats;
      global $lang_menu_settings;
      global $lang_menu_whiteblacklist;
      global $lang_menu_quarantine;
      global $lang_menu_report;
      global $lang_menu_admin;
      global $lang_menu_help;
      global $lang_menu_logout;
      global $lang_text_version;

      $select = "SELECT banner_title, use_logo, use_icons, logo_file, logo_url, logo_alt_text, " .
                       "enable_false_negative_management, enable_stats_tracking " .
                "FROM maia_config WHERE id = 0";
      $sth = $dbh->query($select);
      if ($row = $sth->fetchrow()) {
         $banner_title = $row["banner_title"];
         $use_logo = ($row["use_logo"] == 'Y');
         $use_icons = ($row["use_icons"] == 'Y');
         $logo_file = $row["logo_file"];
         $logo_url = $row["logo_url"];
         $logo_alt_text = $row["logo_alt_text"];
         $enable_false_negative_management = ($row["enable_false_negative_management"] == 'Y');
         $enable_stats_tracking = ($row["enable_stats_tracking"] == 'Y');
      }
      $sth->free();

      if (!$uid && !$euid) {
         $username = "";
         $is_a_visitor = true;
      } else {
         $username = who_am_i($uid, $euid);
         $is_a_visitor = false;
      }
      $showmail = ok_to_impersonate($euid, $uid);
      if ($showmenu) {
         $admin = is_an_administrator($uid);
         $cols = 5;
         if ($enable_stats_tracking) {
             $cols++;
         }
         if ($enable_false_negative_management) {
             $cols++;
         }
         if ($admin) {
             $cols++;
         }
         if (!$showmail) {
             $cols -= 2;
         }
      } else {
         $cols = 1;
         $admin = false;
      }


      print("<div align=\"center\">\n");
      print("<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\">\n");


      print("<tr>\n");

      if ($use_logo) {
         list($width, $height, $type, $attr) = getimagesize($logo_file);
         print("<td valign=\"top\" width=\"" . $width . "\">\n");
         if (!empty($logo_url)) {
            print("<a href=\"" . $logo_url . "\" " .
                  "onmouseover=\"show_tooltip(this, event, '" . $logo_alt_text . "')\" " .
                  "onmouseout=\"hide_tooltip()\">");
         }
         print("<img src=\"" . $logo_file . "\" border=\"0\" alt=\"" . $logo_alt_text . "\">\n");
         if (!empty($logo_url)) {
            print("</a>\n");
         }
         print("</td>\n");
      }
      print("<td valign=\"top\">\n");
      print("<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\">\n");


      print("<tr><td class=\"topbanner\" align=\"center\" height=\"70\" colspan=\"" . $cols . "\">" . $banner_title . "</td></tr>\n");
      print("<tr><td class=\"topbanner2\" align=\"center\" height=\"40\" colspan=\"" . $cols . "\">" . $subtitle . "</td></tr>\n");
      if ($showmenu && !$is_a_visitor) {
         print("<tr><td class=\"topbanner3\" align=\"center\" height=\"30\" colspan=\"" . $cols . "\">" . $username . "</td></tr>\n");
         print("<tr>\n");

         if ($enable_stats_tracking) {
             print("<td class=\"topbanner4\" align=\"center\">\n");
             if ($use_icons) {
                 print("<a href=\"stats.php" . $sid . "\" " .
                       "onmouseover=\"show_tooltip(this, event, '" . $lang_menu_stats . "')\" " .
                       "onmouseout=\"hide_tooltip()\"><img src=\"images/stats.png\" border=\"0\" " .
                       "alt=\"" . $lang_menu_stats . "\"></a>\n");
             } else {
             	 print("<a href=\"stats.php" . $sid . "\">[" . $lang_menu_stats . "]</a>\n");
             }
             print("</td>\n");
         }
         print("<td class=\"topbanner4\" align=\"center\">\n");
         if ($use_icons) {
             print("<a href=\"settings.php" . $sid . "\" " .
                   "onmouseover=\"show_tooltip(this, event, '" . $lang_menu_settings . "')\" " .
                   "onmouseout=\"hide_tooltip()\"><img src=\"images/settings.png\" border=\"0\" " .
                   "alt=\"" . $lang_menu_settings . "\"></a>\n");
         } else {
             print("<a href=\"settings.php" . $sid . "\">[" . $lang_menu_settings . "]</a>\n");
         }
         print("</td>\n");
         print("<td class=\"topbanner4\" align=\"center\">\n");
         if ($use_icons) {
             print("<a href=\"wblist.php" . $sid . "\" " .
                   "onmouseover=\"show_tooltip(this, event, '" . $lang_menu_whiteblacklist . "')\" " .
                   "onmouseout=\"hide_tooltip()\"><img src=\"images/white-black-list.png\" border=\"0\" " .
                   "alt=\"" . $lang_menu_whiteblacklist . "\"></a>\n");
         } else {
             print("<a href=\"wblist.php" . $sid . "\">[" . $lang_menu_whiteblacklist . "]</a>\n");
         }
         print("</td>\n");
         if ($showmail) {
             print("<td class=\"topbanner4\" align=\"center\">\n");
             if ($use_icons) {
                 print("<a href=\"quarantine.php" . $sid . "\" " .
                       "onmouseover=\"show_tooltip(this, event, '" . $lang_menu_quarantine . "')\" " .
                       "onmouseout=\"hide_tooltip()\"><img src=\"images/quarantine.png\" border=\"0\" " .
                       "alt=\"" . $lang_menu_quarantine . "\"></a>\n");
             } else {
                 print("<a href=\"quarantine.php" . $sid . "\">[" . $lang_menu_quarantine . "]</a>\n");
             }
             print("</td>\n");

             if ($enable_false_negative_management) {
                 print("<td class=\"topbanner4\" align=\"center\">\n");
                 if ($use_icons) {
                     print("<a href=\"reportspam.php" . $sid . "\" " .
                           "onmouseover=\"show_tooltip(this, event, '" . $lang_menu_report . "')\" " .
                           "onmouseout=\"hide_tooltip()\"><img src=\"images/report-spam.png\" border=\"0\" " .
                           "alt=\"" . $lang_menu_report . "\"></a>\n");
                 } else {
                     print("<a href=\"reportspam.php" . $sid . "\">[" . $lang_menu_report . "]</a>\n");
                 }
                 print("</td>\n");
             }
         }
         if ($admin) {
             print("<td class=\"topbanner4\" align=\"center\">\n");
             if ($use_icons) {
                 print("<a href=\"admindex.php" . $sid . "\" " .
                       "onmouseover=\"show_tooltip(this, event, '" . $lang_menu_admin . "')\" " .
                       "onmouseout=\"hide_tooltip()\"><img src=\"images/admin-int.png\" border=\"0\" " .
                       "alt=\"" . $lang_menu_admin . "\"></a>\n");
            } else {
                 print("<a href=\"admindex.php" . $sid . "\">[" . $lang_menu_admin . "]</a>\n");
             }
             print("</td>\n");
         }
         print("<td class=\"topbanner4\" align=\"center\">\n");
         if ($use_icons) {
             print("<a href=\"help.php" . $sid . "\" " .
                   "onmouseover=\"show_tooltip(this, event, '" . $lang_menu_help . "')\" " .
                   "onmouseout=\"hide_tooltip()\"><img src=\"images/help.png\" border=\"0\" " .
                   "alt=\"" . $lang_menu_help . "\"></a>\n");
         } else {
             print("<a href=\"help.php" . $sid . "\">[" . $lang_menu_help . "]</a>\n");
         }
         print("</td>\n");
         print("<td class=\"topbanner4\" align=\"center\">\n");
         if ($use_icons) {
             print("<a href=\"logout.php" . $sid . "\" " .
                   "onmouseover=\"show_tooltip(this, event, '" . $lang_menu_logout . "')\" " .
                   "onmouseout=\"hide_tooltip()\"><img src=\"images/logout.png\" border=\"0\" " .
                   "alt=\"" . $lang_menu_logout . "\"></a>\n");
         } else {
             print("<a href=\"logout.php" . $sid . "\">[" . $lang_menu_logout . "]</a>\n");
         }
         print("</td>\n");
         print("</tr>\n");
      } elseif ($use_logo) {
      	 print("<tr><td class=\"topbanner3\" align=\"center\" height=\"30\" colspan=\"" . $cols . "\">" .
      	       $lang_text_version . " " . MAIA_VERSION . "</td></tr>\n");
      	 print("<tr><td class=\"topbanner4\" align=\"center\" colspan=\"" . $cols . "\">&nbsp;</td></tr>\n");
      }

      if ($use_logo) {
         print("</table>\n");
         print("</td>\n");

      	 print("</tr>");
      }

      print("</table></div><br>\n");
   }


   /*
    * display_item_stats(): Produces a table of statistics about mail items
    *                       of the specified type received by the specified
    *                       user, or all users if $user_id == 0.  This
    *                       emulates the style of Maia 0.9.x.
    */
   function display_item_stats($user_id, $type)
   {
      global $dbh;
      global $lang_text_day;
      global $lang_text_all_users;
      global $lang_array_header;
      global $lang_array_count;
      global $lang_array_arrivals;
      global $lang_array_highest_score;
      global $lang_array_lowest_score;
      global $lang_array_average_score;
      global $lang_array_largest_size;
      global $lang_array_smallest_size;
      global $lang_array_average_size;
      global $lang_array_bandwidth;
      global $lang_array_cost;
      global $lang_array_percentage;

      $select = "SELECT currency_label, bandwidth_cost FROM maia_config WHERE id = 0";
      $sth = $dbh->query($select);
      if ($row = $sth->fetchrow()) {
          $currency_label = $row["currency_label"];
          $bandwidth_cost = $row["bandwidth_cost"];
      }
      $sth->free();

      $items = count_items($user_id, $type);
      $total_mail = count_total_mail($user_id);
      $item_days = count_item_days($user_id, $type);
      $item_size = total_item_size($user_id, $type);
      if ($item_days > 0) {
          $bandwidth_per_day = $item_size / $item_days;
      } else {
      	  $bandwidth_per_day = 0;
      }

      $suffix = "";
      if (!$user_id) {
         $suffix = " (". $lang_text_all_users . ")";
      }

      print("<div align=\"center\">\n");
      print("<table border=\"0\" cellspacing=\"2\" cellpadding=\"2\" width=\"600\">\n");
      print("<tr><td class=\"" . $type . "banner\" colspan=\"2\" align=\"center\">" . $lang_array_header[$type] . $suffix . "</td></tr>\n");

      print("<tr><td class=\"" . $type . "header\">" . $lang_array_count[$type] . ":</td>\n");
      print("<td class=\"" . $type . "body\" align=\"center\" width=\"100\">" .
            $items . "</td></tr>\n");

      print("<tr><td class=\"" . $type . "header\">". $lang_array_arrivals[$type] . "/" . $lang_text_day . ":</td>\n");
      print("<td class=\"" . $type . "body\" align=\"center\" width=\"100\">");
      if ($item_days > 0) {
          printf("%.1f", $items / $item_days);
      } else {
          print("-");
      }
      print("</td></tr>\n");

      if ($type == "ham" || $type == "suspected_spam" || $type == "spam" || $type == "fp" || $type == "fn") {
          print("<tr><td class=\"" . $type . "header\">" . $lang_array_highest_score[$type] . ":</td>\n");
          print("<td class=\"" . $type . "body\" align=\"center\" width=\"100\">");
          printf("%.3f", highest_item_score($user_id, $type));
          print("</td></tr>\n");

          print("<tr><td class=\"" . $type . "header\">" . $lang_array_lowest_score[$type] . ":</td>\n");
          print("<td class=\"" . $type . "body\" align=\"center\" width=\"100\">");
          printf("%.3f", lowest_item_score($user_id, $type));
          print("</td></tr>\n");

          print("<tr><td class=\"" . $type . "header\">" . $lang_array_average_score[$type] . ":</td>\n");
          print("<td class=\"" . $type . "body\" align=\"center\" width=\"100\">");
          if ($items > 0) {
              printf("%.3f", total_item_score($user_id, $type) / $items);
          } else {
      	      print("-");
          }
          print("</td></tr>\n");
      }

      print("<tr><td class=\"" . $type . "header\">" . $lang_array_largest_size[$type] . " (kB):</td>\n");
      print("<td class=\"" . $type . "body\" align=\"center\" width=\"100\">");
      printf("%.1f", largest_item_size($user_id, $type) / KILOBYTE);
      print("</td></tr>\n");

      print("<tr><td class=\"" . $type . "header\">" . $lang_array_smallest_size[$type] . " (kB):</td>\n");
      print("<td class=\"" . $type . "body\" align=\"center\" width=\"100\">");
      printf("%.1f", smallest_item_size($user_id, $type) / KILOBYTE);
      print("</td></tr>\n");

      print("<tr><td class=\"" . $type . "header\">" . $lang_array_average_size[$type] . " (kB):</td>\n");
      print("<td class=\"" . $type . "body\" align=\"center\" width=\"100\">");
      if ($items > 0) {
          printf("%.1f", $item_size / $items / KILOBYTE);
      } else {
          print("-");
      }
      print("</td></tr>\n");

      print("<tr><td class=\"" . $type . "header\">" . $lang_array_bandwidth[$type] . " (MB/" . $lang_text_day . "):</td>\n");
      print("<td class=\"" . $type . "body\" align=\"center\" width=\"100\">");
      if ($bandwidth_per_day > 0) {
          printf("%.2f", $bandwidth_per_day / MEGABYTE);
      } else {
      	  print("-");
      }
      print("</td></tr>\n");

      if ($bandwidth_cost > 0) {
         print("<tr><td class=\"" . $type . "header\">" . $lang_array_cost[$type] . " (" . $currency_label . "/" . $lang_text_day . "):</td>\n");
         print("<td class=\"" . $type . "body\" align=\"center\" width=\"100\">");
         printf("%.3f", $bandwidth_cost * $bandwidth_per_day / GIGABYTE);
         print("</td></tr>\n");
      }

      print("<tr><td class=\"" . $type . "header\">" . $lang_array_percentage[$type] . ":</td>\n");
      print("<td class=\"" . $type . "body\" align=\"center\" width=\"100\">");
      if ($total_mail > 0) {
         printf("%.1f%%", 100 * $items / $total_mail);
      } else {
         printf("%.1f%%", 0);
      }
      print("</td></tr>\n");

      print("</table></div><br>\n");

   }


   /*
    * display_item_row(): Produces a row of statistics about mail items
    *                     of the specified type received by the specified
    *                     user, or all users if $user_id == 0.
    */
   function display_item_row($user_id, $type)
   {
       global $dbh;
       global $lang_array_header;

       $bandwidth_cost = get_config_value("bandwidth_cost");
       $items = count_items($user_id, $type);
       if ($items > 0) {
           $total_mail = count_total_mail($user_id);
           if ($total_mail > 0) {
               $pct = sprintf("%.1f%%", 100 * $items / $total_mail);
           } else {
       	       $pct = "0.0%";
           }
           $item_size = total_item_size($user_id, $type);
           $item_days = count_item_days($user_id, $type);
           if ($item_days > 0) {
               if ($item_days >= 1) {
                   $rate = sprintf("%.1f", $items / $item_days);
                   $bandwidth_per_day = $item_size / $item_days;
               } else {
                   $rate = $items;
                   $bandwidth_per_day = $item_size;
               }
           } else {
       	       $rate = "-";
       	       $bandwidth_per_day = 0;
           }
           if ($type == "ham" || $type == "spam" || $type == "suspected_spam" || $type == "suspected_ham" || $type == "fp" || $type == "fn") {
               $minscore = sprintf("%.3f", lowest_item_score($user_id, $type));
               $maxscore = sprintf("%.3f", highest_item_score($user_id, $type));
               $avgscore = sprintf("%.3f", total_item_score($user_id, $type) / $items);
           } else {
	       $minscore = "-";
	       $maxscore = "-";
	       $avgscore = "-";
           }
           $minsize = sprintf("%.1f", smallest_item_size($user_id, $type) / KILOBYTE);
           $maxsize = sprintf("%.1f", largest_item_size($user_id, $type) / KILOBYTE);
           $avgsize = sprintf("%.1f", $item_size / $items / KILOBYTE);
           if ($bandwidth_per_day > 0) {
               $bandwidth = sprintf("%.2f", $bandwidth_per_day / MEGABYTE);
           } else {
      	       $bandwidth = "-";
           }
           $cost = sprintf("%.3f", $bandwidth_cost * $bandwidth_per_day / GIGABYTE);
       } else {
           $rate = "-";
           $minscore = "-";
           $maxscore = "-";
           $avgscore = "-";
           $minsize = "-";
           $maxsize = "-";
           $avgsize = "-";
           $bandwidth = "-";
           $cost = "-";
           $pct = "-";
       }

       print("<tr>\n");
       print("<td class=\"" . $type . "body2\" align=\"center\">" . $lang_array_header[$type] . "</td>\n");
       print("<td class=\"" . $type . "body2\" align=\"center\">" . $items . "</td>\n");
       print("<td class=\"" . $type . "body2\" align=\"center\">" . $rate . "</td>\n");
       print("<td class=\"" . $type . "body2\" align=\"center\">" . $pct . "</td>\n");
       print("<td class=\"" . $type . "body2\" align=\"center\">" . $minscore . "</td>\n");
       print("<td class=\"" . $type . "body2\" align=\"center\">" . $maxscore . "</td>\n");
       print("<td class=\"" . $type . "body2\" align=\"center\">" . $avgscore . "</td>\n");
       print("<td class=\"" . $type . "body2\" align=\"center\">" . $minsize . "</td>\n");
       print("<td class=\"" . $type . "body2\" align=\"center\">" . $maxsize . "</td>\n");
       print("<td class=\"" . $type . "body2\" align=\"center\">" . $avgsize . "</td>\n");
       print("<td class=\"" . $type . "body2\" align=\"center\">" . $bandwidth . "</td>\n");
       print("<td class=\"" . $type . "body2\" align=\"center\">" . $cost . "</td>\n");
       print("</tr>\n");
   }


   /*
    * display_stats_summary(): Produces a table of statistics for the
    *                          specified user, or all users if $user_id == 0.
    *                          If $is_a_visitor == true, the summary will not
    *                          include any links to user-specific stats pages.
    */
   function display_stats_summary($user_id, $is_a_visitor, $sid, $msid)
   {
       global $dbh;
       global $lang_text_day;
       global $lang_text_efficiency;
       global $lang_text_sensitivity;
       global $lang_text_specificity;
       global $lang_text_ppv;
       global $lang_text_npv;
       global $lang_text_lrp;
       global $lang_text_lrn;
       global $lang_text_false_positive;
       global $lang_text_false_negative;
       global $lang_header_score;
       global $lang_header_size;
       global $lang_header_bandwidth;
       global $lang_header_type;
       global $lang_header_items;
       global $lang_header_count;
       global $lang_header_min;
       global $lang_header_max;
       global $lang_header_average;
       global $lang_header_cost;
       global $lang_header_percent;
       global $lang_header_user;
       global $lang_header_systemwide;
       global $lang_link_systemwide;
       global $lang_link_personal;
       global $lang_link_viruses;
       global $lang_link_rules;

       $select = "SELECT currency_label, enable_false_negative_management, " .
                        "enable_virus_scanning, enable_spam_filtering, " .
                        "enable_bad_header_checking, enable_banned_files_checking " .
                 "FROM maia_config WHERE id = 0";
       $sth = $dbh->query($select);
       if ($row = $sth->fetchrow()) {
           $currency_label = $row["currency_label"];
           $enable_false_negative_management = ($row["enable_false_negative_management"] == 'Y');
           $enable_virus_scanning = ($row["enable_virus_scanning"] == 'Y');
           $enable_spam_filtering = ($row["enable_spam_filtering"] == 'Y');
           $enable_bad_header_checking = ($row["enable_bad_header_checking"] == 'Y');
           $enable_banned_files_checking = ($row["enable_banned_files_checking"] == 'Y');
       }
       $sth->free();

       if ($user_id > 0) {
           $header = sprintf($lang_header_user, get_user_name($user_id));
       } else {
       	   $header = $lang_header_systemwide;
       }

       print("<div align=\"center\">\n");
       print("<table border=\"0\" cellspacing=\"1\" cellpadding=\"2\" width=\"100%\">\n");

       print("<tr>\n");
       print("<td class=\"menuheader\" colspan=\"12\" align=\"center\">" . $header . "</td>\n");
       print("</tr>\n");
       print("<tr>\n");
       print("<td class=\"menuheader2\" colspan=\"4\" align=\"center\">" . $lang_header_items . "</td>\n");
       print("<td class=\"menuheader2\" colspan=\"3\" align=\"center\">" . $lang_header_score . "</td>\n");
       print("<td class=\"menuheader2\" colspan=\"3\" align=\"center\">" . $lang_header_size . " (kB)</td>\n");
       print("<td class=\"menuheader2\" colspan=\"2\" align=\"center\">" . $lang_header_bandwidth . "/" . $lang_text_day . "</td>\n");
       print("</tr>\n");
       print("<tr>\n");
       print("<td class=\"menuheader2\" align=\"center\">" . $lang_header_type . "</td>\n");
       print("<td class=\"menuheader2\" align=\"center\">" . $lang_header_count . "</td>\n");
       print("<td class=\"menuheader2\" align=\"center\">" . $lang_header_items . "/" . $lang_text_day . "</td>\n");
       print("<td class=\"menuheader2\" align=\"center\">" . $lang_header_percent . "</td>\n");
       print("<td class=\"menuheader2\" align=\"center\">" . $lang_header_min . "</td>\n");
       print("<td class=\"menuheader2\" align=\"center\">" . $lang_header_max . "</td>\n");
       print("<td class=\"menuheader2\" align=\"center\">" . $lang_header_average . "</td>\n");
       print("<td class=\"menuheader2\" align=\"center\">" . $lang_header_min . "</td>\n");
       print("<td class=\"menuheader2\" align=\"center\">" . $lang_header_max . "</td>\n");
       print("<td class=\"menuheader2\" align=\"center\">" . $lang_header_average . "</td>\n");
       print("<td class=\"menuheader2\" align=\"center\">MB</td>\n");
       print("<td class=\"menuheader2\" align=\"center\">" . $lang_header_cost . " (" . $currency_label . ")</td>\n");

       print("</tr>\n");

       if ($enable_spam_filtering && $enable_false_negative_management) {
           display_item_row($user_id, "suspected_ham");
       }
       display_item_row($user_id, "ham");
       if ($enable_spam_filtering) {
           display_item_row($user_id, "fp");
           display_item_row($user_id, "suspected_spam");
           display_item_row($user_id, "spam");
           if ($enable_false_negative_management) {
               display_item_row($user_id, "fn");
           }
       }
       display_item_row($user_id, "wl");
       display_item_row($user_id, "bl");
       if ($enable_virus_scanning) {
           display_item_row($user_id, "virus");
       }
       if ($enable_banned_files_checking) {
           display_item_row($user_id, "banned_file");
       }
       if ($enable_bad_header_checking) {
           display_item_row($user_id, "bad_header");
       }
       display_item_row($user_id, "oversized");

       if ($enable_spam_filtering) {
       	   list($ppv_pct, $npv_pct, $sensitivity_pct, $specificity_pct,
       	        $efficiency_pct, $fp_pct, $fn_pct) = get_filter_stats($user_id);
           print("<tr>\n");
           print("<td class=\"menuheader2\" colspan=\"12\" align=\"center\">\n");
           if ($enable_false_negative_management) {
               printf("%s: %.2f%%&nbsp;&nbsp;&nbsp;%s: %.2f%%&nbsp;&nbsp;&nbsp;%s: %.2f%%<br>",
                      $lang_text_efficiency, 100 * $efficiency_pct,
                      $lang_text_false_positive, 100 * $fp_pct,
                      $lang_text_false_negative, 100 * $fn_pct);
               printf("%s: %.2f%%&nbsp;&nbsp;&nbsp;%s: %.2f%%&nbsp;&nbsp;&nbsp;%s: %.2f%%&nbsp;&nbsp;&nbsp;%s: %.2f%%",
                      $lang_text_sensitivity, 100 * $sensitivity_pct,
                      $lang_text_ppv, 100 * $ppv_pct,
                      $lang_text_specificity, 100 * $specificity_pct,
                      $lang_text_npv, 100 * $npv_pct);
           } else {
               printf("%s: %.2f%%&nbsp;&nbsp;&nbsp;%s: %.2f%%&nbsp;&nbsp;&nbsp;%s: %.2f%%<br>",
                      $lang_text_efficiency, 100 * $efficiency_pct,
                      $lang_text_false_positive, 100 * $fp_pct,
                      $lang_text_false_negative, 100 * $fn_pct);
               printf("%s: %.2f%%&nbsp;&nbsp;&nbsp;%s: %.2f%%",
                      $lang_text_sensitivity, 100 * $sensitivity_pct,
                      $lang_text_ppv, 100 * $ppv_pct);
           }
           print("</td></tr>\n");
       }

       if (!$is_a_visitor) {

           if ($user_id > 0) {
               print("<tr>\n");
               print("<td class=\"menuheader2\" colspan=\"12\" align=\"center\">\n");
               print("<a href=\"stats.php" . $msid . "id=0\"><b>[" . $lang_link_systemwide . "]</b></a></td>\n");
               print("</tr>\n");
           } else {
               print("<tr>\n");
               print("<td class=\"menuheader2\" colspan=\"12\" align=\"center\">\n");
       	       print("<a href=\"stats.php" . $sid . "\"><b>[" . $lang_link_personal . "]</b></a></td>\n");
       	       print("</tr>\n");
       	       if ($enable_virus_scanning) {
                   print("<tr>\n");
                   print("<td class=\"menuheader2\" colspan=\"12\" align=\"center\">\n");
       	           print("<a href=\"virusstats.php" . $sid . "\"><b>[" . $lang_link_viruses . "]</b></a></td>\n");
       	           print("</tr>\n");
                   print("<tr>\n");
               }
               if ($enable_spam_filtering) {
                   print("<td class=\"menuheader2\" colspan=\"12\" align=\"center\">\n");
       	           print("<a href=\"rulestats.php" . $sid . "\"><b>[" . $lang_link_rules . "]</b></a></td>\n");
       	           print("</tr>\n");
       	       }
           }
       }

       print("</table></div><br>\n");
   }


   /*
    * linkify(): Replaces $label in $text with a hyperlink to $link
    */
   function linkify($text, $label, $link)
   {
       $preg_label = preg_replace("/\//", "\/", $label);
       return preg_replace("/" . $preg_label . "/s",
                           "<a href=\"" . $link . "\">" . $label . "</a>",
                           $text);
   }


   /*
    * add_links_to_text(): Looks for certain "hot" words in a text string
    *                      and replaces them with corresponding URLs.
    */
   function add_links_to_text($text)
   {
       // Explicit URLs
       if (preg_match("/\((http:\/\/.+?)\)/si", $text, $match)) {
       	   $text = linkify($text, $match[1], $match[1]);
       }

       // SORBS: Spam and Open Relay Blocking System
       $text = linkify($text, "SORBS", "http://www.sorbs.net/");

       // Razor2: Vipul's Razor
       $text = linkify($text, "Razor2", "http://razor.sourceforge.net/");

       // Pyzor
       $text = linkify($text, "Pyzor", "http://pyzor.sourceforge.net/");

       // DCC: Distributed Checksum Clearinghouse
       $text = linkify($text, "DCC", "http://rhyolite.com/anti-spam/dcc/");

       // NJABL: Not Just Another Bogus List
       $text = linkify($text, "NJABL", "http://www.njabl.org/");
       $text = linkify($text, "dnsbl.njabl.org", "http://www.njabl.org/");

       // SpamCop Blocking List
       $text = linkify($text, "bl.spamcop.net", "http://www.spamcop.net/bl.shtml");

       // OPM: Blitzed Open Proxy Monitor
       $text = linkify($text, "opm.blitzed.org", "http://www.blitzed.org/bopm/");
       $text = linkify($text, "OPM", "http://www.blitzed.org/bopm/");

       // DSBL: Distributed Server Boycott List
       $text = linkify($text, "list.dsbl.org", "http://www.dsbl.org/");

       // RFCI: RFC-Ignorant
       $text = linkify($text, "ipwhois.rfc-ignorant.org", "http://www.rfc-ignorant.org/policy-ipwhois.php");
       $text = linkify($text, "dsn.rfc-ignorant.org", "http://www.rfc-ignorant.org/policy-dsn.php");

       // SBL: Spamhaus Block List
       $text = linkify($text, "Spamhaus Block List", "http://www.spamhaus.org/sbl/");

       // MAPS: Mail Abuse Prevention System
       $text = linkify($text, "RBL", "http://www.mail-abuse.org/rbl/");
       $text = linkify($text, "DUL", "http://www.mail-abuse.org/dul/");
       $text = linkify($text, "RSS", "http://www.mail-abuse.org/rss/");
       $text = linkify($text, "NML", "http://www.mail-abuse.org/nml/");

       // BSP: Bonded Sender Program
       $text = linkify($text, "Bonded Sender Program", "http://www.bondedsender.com/");

       // Habeas Sender Warranted E-mail
       $text = linkify($text, "Habeas", "http://www.habeas.com/");

       // CBL: Composite Blocking List
       $text = linkify($text, "cbl.abuseat.org", "http://cbl.abuseat.org/");

       // Easynet DNSBL
       $text = linkify($text, "proxies.blackholes.easynet.nl", "http://abuse.easynet.nl/proxies.html");

       // AHBL: Abusive Hosts Blocking List
       $text = linkify($text, "AHBL", "http://www.ahbl.org");
       $text = linkify($text, "dnsbl.ahbl.org", "http://www.ahbl.org");

       return $text;
   }


   /*
    * display_spam_report(): Produces a table listing of the SpamAssassin
    *                        rules that were triggered by the specified
    *                        mail item, for use in the Mail Viewer.
    */
   function display_spam_report($mail_id)
   {
       global $dbh;
       global $lang_header_spam_score;
       global $lang_header_rule_triggered;
       global $lang_header_explanation;

       $select = "SELECT rule_name, rule_description, maia_sa_rules_triggered.rule_score " .
                 "FROM maia_sa_rules, maia_sa_rules_triggered " .
                 "WHERE maia_sa_rules.id = maia_sa_rules_triggered.rule_id " .
                 "AND maia_sa_rules_triggered.mail_id = ? " .
                 "ORDER BY maia_sa_rules_triggered.rule_score DESC";
       $sth = $dbh->query($select, array($mail_id));
       if ($sth->numrows() > 0) {
           print("<div align=\"center\">\n");
           print("<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\">\n");
           print("<tr>\n");
           print("<td class=\"spamreportheader\" align=\"center\">" . $lang_header_spam_score . "</td>\n");
           print("<td class=\"spamreportheader\" align=\"center\">" . $lang_header_rule_triggered . "</td>\n");
           print("<td class=\"spamreportheader\" align=\"center\">" . $lang_header_explanation . "</td>\n");
           print("</tr>\n");
           while ($row = $sth->fetchrow()) {
       	       print("<tr>\n");
       	       print("<td class=\"spamreport\" align=\"center\">\n");
       	       printf("%.3f", $row["rule_score"]);
       	       print("</td>\n");
       	       print("<td class=\"spamreport\" align=\"left\">" . $row["rule_name"] . "</td>\n");
       	       $description = add_links_to_text($row["rule_description"]);
       	       print("<td class=\"spamreport\" align=\"left\">" . $description . "</td>\n");
       	       print("</tr>\n");
           }
           $sth->free();
           print("</table></div><br>\n");
       }
   }


   /*
    * display_virus_summary(): Displays a table of statistics about the viruses
    *                          received by all users site-wide.
    */
   function display_virus_summary()
   {
       global $dbh;
       global $lang_header_virus_name;
       global $lang_header_count;
       global $lang_message_no_viruses;
       global $lang_header_percent;
       global $lang_header_viruses1;
       global $lang_header_viruses2;
       global $lang_text_viruses;

       print("<div align=\"center\">\n");
       print("<table border=\"0\" cellspacing=\"2\" cellpadding=\"2\">\n");
       $select = "SELECT SUM(count) AS total, COUNT(count) AS vcount FROM maia_viruses WHERE count > 0";
       $sth = $dbh->query($select);
       if (($row = $sth->fetchrow()) && (($total = $row["total"]) > 0)) {
       	   $vcount = $row["vcount"];
           $select = "SELECT virus_name, count FROM maia_viruses WHERE count > 0 ORDER BY count DESC, virus_name ASC";
           $sth2 = $dbh->query($select);
           print("<tr>\n");
           print("<td class=\"virusbanner\" align=\"center\" colspan=\"3\">");
           print($lang_header_viruses1 . "<br>\n");
           print("<font size=\"3\">" . $lang_header_viruses2 . "</font></td>\n");
           print("</tr>\n");
           print("<tr>\n");
           print("<td class=\"virusheader\" align=\"center\">" . $lang_header_virus_name . "</td>\n");
           print("<td class=\"virusheader\" align=\"center\">" . $lang_header_count . "</td>\n");
           print("<td class=\"virusheader\" align=\"center\">" . $lang_header_percent . "</td>\n");
           print("</tr>\n");
           while ($row2 = $sth2->fetchrow()) {
               $vurl = get_virus_info_url($row2["virus_name"]);
               if ($vurl == "") {
                   $vname = $row2["virus_name"];
               } else {
                   $vname = "<a href=\"" . $vurl . "\">" . $row2["virus_name"] . "</a>";
               }
               print("<tr>\n");
               print("<td class=\"virusbody\" align=\"center\">" . $vname . "</td>\n");
               print("<td class=\"virusbody\" align=\"center\">" . $row2["count"] . "</td>\n");
               print("<td class=\"virusbody\" align=\"center\">");
               printf("%.1f%%</td>\n", 100 * $row2["count"] / $total);
               print("</tr>\n");
           }
           $sth2->free();
           print("<tr>\n");
           print("<td class=\"virusheader\" align=\"center\">");
           print($vcount . " " . $lang_text_viruses . "</td>\n");
           print("<td class=\"virusheader\" align=\"center\">" . $total . "</td>\n");
           print("<td class=\"virusheader\" align=\"center\">100%</td>\n");
           print("</tr>\n");
       } else {
           print("<tr>\n");
           print("<td class=\"messagebox\" valign=\"top\" align=\"center\">" . $lang_message_no_viruses . "</td>\n");
           print("</tr>\n");
       }
       $sth->free();
       print("</table></div><br>\n");
   }


   /*
    * display_sa_rules_summary(): Displays a table of statistics about
    *                             the SpamAssassin rules triggered by all
    *                             mail received by all users site-wide.
    */
   function display_sa_rules_summary()
   {
       global $dbh;
       global $lang_header_rule_name;
       global $lang_header_rule_count;
       global $lang_header_rule_description;
       global $lang_header_rule_score;
       global $lang_message_no_rules;
       global $lang_header_percent;
       global $lang_header_rules1;
       global $lang_header_rules2;
       global $lang_text_total;
       global $lang_text_rules;

       $ruleset = get_config_value("sa_score_set");
       print("<div align=\"center\">\n");
       print("<table border=\"0\" cellspacing=\"2\" cellpadding=\"2\">\n");
       $select = "SELECT SUM(rule_count) AS total, COUNT(rule_count) AS rcount FROM maia_sa_rules WHERE rule_count > 0";
       $sth = $dbh->query($select);
       if (($row = $sth->fetchrow()) && (($total = $row["total"]) > 0)) {
       	   $rcount = $row["rcount"];
           $select = "SELECT rule_name, rule_description, rule_score_" . $ruleset . ", rule_count " .
                     "FROM maia_sa_rules WHERE rule_count > 0 ORDER BY rule_count DESC, rule_name ASC";
           $sth2 = $dbh->query($select);
           print("<tr>\n");
           print("<td class=\"spambanner\" align=\"center\" colspan=\"5\">");
           print($lang_header_rules1 . "<br>\n");
           print("<font size=\"3\">" . $lang_header_rules2 . "</font></td>\n");
           print("</tr>\n");
           print("<tr>\n");
           print("<td class=\"spamheader\" align=\"center\">" . $lang_header_rule_name . "</td>\n");
           print("<td class=\"spamheader\" align=\"center\">" . $lang_header_rule_description . "</td>\n");
           print("<td class=\"spamheader\" align=\"center\">" . $lang_header_rule_score . "</td>\n");
           print("<td class=\"spamheader\" align=\"center\">" . $lang_header_rule_count . "</td>\n");
           print("<td class=\"spamheader\" align=\"center\">" . $lang_header_percent . "</td>\n");
           print("</tr>\n");
           while ($row2 = $sth2->fetchrow()) {
               print("<tr>\n");
               print("<td class=\"spambody\" align=\"center\">" . $row2["rule_name"] . "</td>\n");
               $description = add_links_to_text($row2["rule_description"]);
               print("<td class=\"spambody\" align=\"center\">" . $description . "</td>\n");
               print("<td class=\"spambody\" align=\"center\">");
               printf("%.3f</td>\n", $row2["rule_score_" . $ruleset]);
               print("<td class=\"spambody\" align=\"center\">" . $row2["rule_count"] . "</td>\n");
               print("<td class=\"spambody\" align=\"center\">");
               printf("%.1f%%</td>\n", 100 * $row2["rule_count"] / $total);
               print("</tr>\n");
           }
           $sth2->free();
           print("<tr>\n");
           print("<td class=\"spamheader\" align=\"center\">");
           print($rcount . " " . $lang_text_rules . "</td>\n");
           print("<td class=\"spamheader\" align=\"center\">&nbsp;</td>\n");
           print("<td class=\"spamheader\" align=\"center\">&nbsp;</td>\n");
           print("<td class=\"spamheader\" align=\"center\">" . $total . "</td>\n");
           print("<td class=\"spamheader\" align=\"center\">100%</td>\n");
           print("</tr>\n");
       } else {
           print("<tr>\n");
           print("<td class=\"messagebox\" valign=\"top\" align=\"center\">" . $lang_message_no_rules . "</td>\n");
           print("</tr>\n");
       }
       $sth->free();
       print("</table></div><br>\n");
   }
?>