<?php
    /*
     * $Id: internal-init.php,v 1.4 2004/03/07 23:56:22 rjl Exp $
     *
     * MAIA MAILGUARD LICENSE v.1.0
     *
     * Copyright 2004 by Robert LeBlanc <rjl@renaissoft.com>
     * All rights reserved.
     *
     * PREAMBLE
     *
     * This License is designed for users of Maia Mailguard
     * ("the Software") who wish to support the Maia Mailguard project by
     * leaving "Maia Mailguard" branding information in the HTML output
     * of the pages generated by the Software, and providing links back
     * to the Maia Mailguard home page.  Users who wish to remove this
     * branding information should contact the copyright owner to obtain
     * a Rebranding License.
     *
     * DEFINITION OF TERMS
     *
     * The "Software" refers to Maia Mailguard, including all of the
     * associated PHP, Perl, and SQL scripts, documentation files, graphic
     * icons and logo images.
     *
     * GRANT OF LICENSE
     *
     * Redistribution and use in source and binary forms, with or without
     * modification, are permitted provided that the following conditions
     * are met:
     *
     * 1. Redistributions of source code must retain the above copyright
     *    notice, this list of conditions and the following disclaimer.
     *
     * 2. Redistributions in binary form must reproduce the above copyright
     *    notice, this list of conditions and the following disclaimer in the
     *    documentation and/or other materials provided with the distribution.
     *
     * 3. The end-user documentation included with the redistribution, if
     *    any, must include the following acknowledgment:
     *
     *    "This product includes software developed by Robert LeBlanc
     *    <rjl@renaissoft.com>."
     *
     *    Alternately, this acknowledgment may appear in the software itself,
     *    if and wherever such third-party acknowledgments normally appear.
     *
     * 4. At least one of the following branding conventions must be used:
     *
     *    a. The Maia Mailguard logo appears in the page-top banner of
     *       all HTML output pages in an unmodified form, and links
     *       directly to the Maia Mailguard home page; or
     *
     *    b. The "Powered by Maia Mailguard" graphic appears in the HTML
     *       output of all gateway pages that lead to this software,
     *       linking directly to the Maia Mailguard home page; or
     *
     *    c. A separate Rebranding License is obtained from the copyright
     *       owner, exempting the Licensee from 4(a) and 4(b), subject to
     *       the additional conditions laid out in that license document.
     *
     * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS
     * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
     * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
     * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
     * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
     * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
     * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
     * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
     * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
     * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
     * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
     *
     */

    require_once ("config.php");
    require_once ("db.php");
    require_once ("display.php");
    require_once ("smtp.php");

    // Determine the user's language preference
    if (isset($HTTP_GET_VARS["lang"])) {
       $display_language = trim($HTTP_GET_VARS["lang"]);
    } else {
       $display_language = $default_display_language;
    }
    require_once ("./locale/$display_language/db.php");
    require_once ("./locale/$display_language/display.php");
    require_once ("./locale/$display_language/smtp.php");
    require_once ("./locale/$display_language/internal-init.php");

    // Determine the user's character set preference
    if (isset($HTTP_GET_VARS["charset"])) {
        $html_charset = trim($HTTP_GET_VARS["charset"]);
    } else {
        $html_charset = $default_charset;
    }

    // Only show this page if we've selected internal
    // authentication and there's no current superadmin
    if (($auth_method != "internal") || (get_superadmin_id() > 0)) {
        header("Location: index.php");
        exit();
    }

    if (isset($HTTP_POST_VARS["submit"])) {

        if (isset($HTTP_POST_VARS["your_email"])) {
            $your_email = trim($HTTP_POST_VARS["your_email"]);
        } else {
            $your_email = "";
        }
        if (isset($HTTP_POST_VARS["admin_email"])) {
            $admin_email = trim($HTTP_POST_VARS["admin_email"]);
        } else {
            $admin_email = "";
        }
        if (isset($HTTP_POST_VARS["reminder_login_url"])) {
            $reminder_login_url = trim($HTTP_POST_VARS["reminder_login_url"]);
        } else {
            $reminder_login_url = "";
        }
        if (isset($HTTP_POST_VARS["newuser_template_file"])) {
            $newuser_template_file = trim($HTTP_POST_VARS["newuser_template_file"]);
        } else {
            $newuser_template_file = "";
        }

        $update = "UPDATE maia_config SET enable_user_autocreation = 'N', " .
                                         "internal_auth = 'Y', " .
                                         "admin_email = ?, " .
                                         "reminder_login_url = ?, " .
                                        "newuser_template_file = ? " .
                  "WHERE id = 0";
        $dbh->query($update, array($admin_email,
                                   $reminder_login_url,
                                   $newuser_template_file));


        $new_email = get_rewritten_email_address($your_email, $address_rewriting_type);
        $username = $new_email;

        $new_user_id = add_user($username, $new_email);

        // Generate a random password and assign it to the new user
        list($password, $digest) = generate_random_password();
        $update = "UPDATE maia_users SET password = ? WHERE id = ?";
        $dbh->query($update, array($digest, $new_user_id));

        $fh = fopen($newuser_template_file, "r");
        $body = fread($fh, filesize($newuser_template_file));
        fclose($fh);
        $headers = "To: " . $new_email . "\n";
        $headers .= "Subject: " . $lang_text_email_subject . "\n";
        $headers .= "\n";
        $body = preg_replace("/%%ADMINEMAIL%%/", $admin_email, $body);
        $body = preg_replace("/%%LOGIN%%/", $username, $body);
        $body = preg_replace("/%%PASSWORD%%/", $password, $body);
        $body = preg_replace("/%%LOGINURL%%/", $reminder_login_url, $body);
        smtp_send($admin_email, $new_email, $headers . $body);

        display_html_head($lang_banner_subtitle, $display_language, $html_charset);
        display_banner($lang_banner_subtitle, $uid, $euid, $sid, true);
?>

<div align="center">
<table border="0" cellspacing="2" cellpadding="2">
<tr>
<td valign="top" class="messagebox" align="left">
<?php print($lang_text_auto_creation); ?><br>
<?php print($lang_text_auto_passwords); ?><br>
<br>
<?php print($lang_text_instructions); ?>
</td>
</tr>
</table>
</div><br>

<div align="center">
<a href="login.php?lang=<?php print($display_language); ?>&charset=<?php print($html_charset); ?>">[ <?php print($lang_link_login); ?> ]</a>
</div>


<?php display_html_foot(); ?>
<?php

    } else {

        $select = "SELECT admin_email, " .
                         "reminder_login_url, " .
                         "newuser_template_file " .
                  "FROM maia_config WHERE id = 0";
        $sth = $dbh->query($select);
        if ($row = $sth->fetchrow()) {
            $admin_email = $row["admin_email"];
            $reminder_login_url = $row["reminder_login_url"];
            $newuser_template_file = $row["newuser_template_file"];
        }
       $sth->free();

        display_html_head($lang_banner_subtitle, $display_language, $html_charset);
        display_banner($lang_banner_subtitle, $uid, $euid, $sid, true);
?>

<form method="post" action="internal-init.php">

<div align="center">
<table border="0" cellspacing="2" cellpadding="2">

<tr>
<td class="menuheader" align="center" colspan="2">
<?php print($lang_header_initial_settings); ?>
</td>
</tr>

<tr>
<td class="menubody" align="left"><?php print($lang_text_newuser_template_file); ?>:</td>
<td class="menubody" align="left">
<input type="text" name="newuser_template_file" value="<?php print($newuser_template_file); ?>" size="60">
</td>
</tr>

<tr>
<td class="menubody" align="left"><?php print($lang_text_reminder_login_url); ?>:</td>
<td class="menubody" align="left">
<input type="text" name="reminder_login_url" value="<?php print($reminder_login_url); ?>" size="60">
</td>
</tr>

<tr>
<td class="menubody" align="left"><?php print($lang_text_admin_email); ?>:</td>
<td class="menubody" align="left">
<input type="text" name="admin_email" value="<?php print($admin_email); ?>" size="60">
</td>
</tr>

<tr>
<td class="menubody" align="left"><?php print($lang_text_your_email); ?>:</td>
<td class="menubody" align="left">
<input type="text" name="your_email" value="" size="60">
</td>
</tr>

<tr>
<td class="menubody" align="center" colspan="2">
<input type="submit" name="submit" value=" <?php print($lang_button_initialize); ?> "></td>
</tr>

</table>
</div>

</form>

<?php display_html_foot(); ?>
<?php } ?>