<?php
    /*
     * $Id: quarantine.php,v 1.2 2004/02/02 22:18:02 rjl Exp $
     *
     * MAIA MAILGUARD LICENSE v.1.0
     *
     * Copyright 2004 by Robert LeBlanc <rjl@renaissoft.com>
     * All rights reserved.
     *
     * PREAMBLE
     *
     * This License is designed for users of Maia Mailguard
     * ("the Software") who wish to support the Maia Mailguard project by
     * leaving "Maia Mailguard" branding information in the HTML output
     * of the pages generated by the Software, and providing links back
     * to the Maia Mailguard home page.  Users who wish to remove this
     * branding information should contact the copyright owner to obtain
     * a Rebranding License.
     *
     * DEFINITION OF TERMS
     *
     * The "Software" refers to Maia Mailguard, including all of the
     * associated PHP, Perl, and SQL scripts, documentation files, graphic
     * icons and logo images.
     *
     * GRANT OF LICENSE
     *
     * Redistribution and use in source and binary forms, with or without
     * modification, are permitted provided that the following conditions
     * are met:
     *
     * 1. Redistributions of source code must retain the above copyright
     *    notice, this list of conditions and the following disclaimer.
     *
     * 2. Redistributions in binary form must reproduce the above copyright
     *    notice, this list of conditions and the following disclaimer in the
     *    documentation and/or other materials provided with the distribution.
     *
     * 3. The end-user documentation included with the redistribution, if
     *    any, must include the following acknowledgment:
     *
     *    "This product includes software developed by Robert LeBlanc
     *    <rjl@renaissoft.com>."
     *
     *    Alternately, this acknowledgment may appear in the software itself,
     *    if and wherever such third-party acknowledgments normally appear.
     *
     * 4. At least one of the following branding conventions must be used:
     *
     *    a. The Maia Mailguard logo appears in the page-top banner of
     *       all HTML output pages in an unmodified form, and links
     *       directly to the Maia Mailguard home page; or
     *
     *    b. The "Powered by Maia Mailguard" graphic appears in the HTML
     *       output of all gateway pages that lead to this software,
     *       linking directly to the Maia Mailguard home page; or
     *
     *    c. A separate Rebranding License is obtained from the copyright
     *       owner, exempting the Licensee from 4(a) and 4(b), subject to
     *       the additional conditions laid out in that license document.
     *
     * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS
     * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
     * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
     * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
     * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
     * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
     * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
     * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
     * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
     * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
     * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
     *
     */

    require_once ("config.php");
    require_once ("authcheck.php");
    require_once ("db.php");
    require_once ("display.php");

    $display_language = get_display_language($euid);
    $html_charset = get_html_charset($euid);
    require_once ("./locale/$display_language/display.php");
    require_once ("./locale/$display_language/db.php");
    require_once ("./locale/$display_language/quarantine.php");

    // Admins (including the superadmin) should not be looking at
    // other people's mail!
    if (!ok_to_impersonate($euid, $uid)) {
        header("Location: stats.php" . $sid);
        exit;
    }

    // Count the suspected spam
    $select = "SELECT COUNT(maia_mail.id) AS spamcount, " .
                     "MAX(maia_mail.id) AS maxspamid " .
              "FROM maia_mail, maia_mail_recipients " .
              "WHERE maia_mail.id = maia_mail_recipients.mail_id " .
              "AND maia_mail_recipients.type = 'S' " .
              "AND maia_mail_recipients.recipient_id = ?";
    $sth = $dbh->query($select, array($euid));
    if ($row = $sth->fetchrow()) {
        $spamcount = $row["spamcount"];
        $maxspamid = $row["maxspamid"];
    } else {
        $spamcount = 0;
        $maxspamid = 0;
    }
    $sth->free();

    // Count the viruses/malware
    $select = "SELECT COUNT(maia_mail.id) AS viruscount, " .
                     "MAX(maia_mail.id) AS maxvirusid " .
              "FROM maia_mail, maia_mail_recipients " .
              "WHERE maia_mail.id = maia_mail_recipients.mail_id " .
              "AND maia_mail_recipients.type = 'V' " .
              "AND maia_mail_recipients.recipient_id = ?";
    $sth = $dbh->query($select, array($euid));
    if ($row = $sth->fetchrow()) {
        $viruscount = $row["viruscount"];
        $maxvirusid = $row["maxvirusid"];
    } else {
        $viruscount = 0;
        $maxvirusid = 0;
    }
    $sth->free();

    // Count the banned file attachments
    $select = "SELECT COUNT(maia_mail.id) AS bannedcount, " .
                     "MAX(maia_mail.id) AS maxbannedid " .
              "FROM maia_mail, maia_mail_recipients " .
              "WHERE maia_mail.id = maia_mail_recipients.mail_id " .
              "AND maia_mail_recipients.type = 'F' " .
              "AND maia_mail_recipients.recipient_id = ?";
    $sth = $dbh->query($select, array($euid));
    if ($row = $sth->fetchrow()) {
        $bannedcount = $row["bannedcount"];
        $maxbannedid = $row["maxbannedid"];
    } else {
        $bannedcount = 0;
        $maxbannedid = 0;
    }
    $sth->free();

    // Count the bad header items
    $select = "SELECT COUNT(maia_mail.id) AS headercount, " .
                     "MAX(maia_mail.id) AS maxheaderid " .
              "FROM maia_mail, maia_mail_recipients " .
              "WHERE maia_mail.id = maia_mail_recipients.mail_id " .
              "AND maia_mail_recipients.type = 'B' " .
              "AND maia_mail_recipients.recipient_id = ?";
    $sth = $dbh->query($select, array($euid));
    if ($row = $sth->fetchrow()) {
        $headercount = $row["headercount"];
        $maxheaderid = $row["maxheaderid"];
    } else {
        $headercount = 0;
        $maxheaderid = 0;
    }
    $sth->free();

    $maxitemid = max($maxspamid, $maxvirusid, $maxbannedid, $maxheaderid);

    $nothing_to_show = ($spamcount + $viruscount + $bannedcount + $headercount == 0);
    display_html_head($lang_banner_subtitle, $display_language, $html_charset);
    display_banner($lang_banner_subtitle, $uid, $euid, $sid, true);
?>

<form method="post" action="xquarantine.php<?php print($sid); ?>" name="quarantine">
<input type="hidden" name="maxitemid" value="<?php print($maxitemid); ?>">

<?php if (!$nothing_to_show) { ?>

<div align="center">
<table border="0" cellspacing="2" cellpadding="2" width="400">

<tr>
<td class="menubanner" align="center"><?php print($lang_header_quarantine_contents); ?></td>
</tr>

<?php if ($spamcount > 0) { ?>
<tr>
<td class="suspected_spamheader" align="center">
<?php
    if ($spamcount == 1) {
        printf("<a href=\"spam-quarantine.php" . $sid . "\">1 %s</a>", $lang_text_suspected_spam_item);
    } else {
        printf("<a href=\"spam-quarantine.php" . $sid . "\">%d %s</a>", $spamcount, $lang_text_suspected_spam_items);
    }
?>
</td>
</tr>
<?php } ?>

<?php if ($viruscount > 0) { ?>
<tr>
<td class="virusheader" align="center">
<?php
    if ($viruscount == 1) {
        printf("<a href=\"virus-quarantine.php" . $sid . "\">1 %s</a>", $lang_text_virus_item);
    } else {
        printf("<a href=\"virus-quarantine.php" . $sid . "\">%d %s</a>", $viruscount, $lang_text_virus_items);
    }
?>
</td>
</tr>
<?php } ?>

<?php if ($bannedcount > 0) { ?>
<tr>
<td class="banned_fileheader" align="center">
<?php
    if ($bannedcount == 1) {
        printf("<a href=\"attachment-quarantine.php" . $sid . "\">1 %s</a>", $lang_text_banned_file_item);
    } else {
        printf("<a href=\"attachment-quarantine.php" . $sid . "\">%d %s</a>", $bannedcount, $lang_text_banned_file_items);
    }
?>
</td>
</tr>
<?php } ?>

<?php if ($headercount > 0) { ?>
<tr>
<td class="bad_headerheader" align="center">
<?php
    if ($headercount == 1) {
        printf("<a href=\"header-quarantine.php" . $sid . "\">1 %s</a>", $lang_text_bad_header_item);
    } else {
        printf("<a href=\"header-quarantine.php" . $sid . "\">%d %s</a>", $headercount, $lang_text_bad_header_items);
    }
?>
</td>
</tr>
<?php } ?>

<tr><td class="menulight" align="center">&nbsp;</td></tr>

<tr>
<td class="menuheader" align="center">
<input type="submit" name="submit" value=" <?php print($lang_button_delete_all_items); ?> ">
</td>
</tr>

</table>
</div>


<?php
   } else {
?>
<div align="center">
<table border="0" cellspacing="2" cellpadding="2">
<tr>
<td class="messagebox" align="center">
<?php print($lang_text_empty); ?>
</td>
</tr>
</table>
</div><br>
<?php
   }
?>

</form>

<?php display_html_foot(); ?>