<?php
    /*
     * $Id: smtp.php,v 1.1.1.1 2004/01/21 15:54:42 rjl Exp $
     *
     * MAIA MAILGUARD LICENSE v.1.0
     *
     * Copyright 2004 by Robert LeBlanc <rjl@renaissoft.com>
     * All rights reserved.
     *
     * PREAMBLE
     *
     * This License is designed for users of Maia Mailguard
     * ("the Software") who wish to support the Maia Mailguard project by
     * leaving "Maia Mailguard" branding information in the HTML output
     * of the pages generated by the Software, and providing links back
     * to the Maia Mailguard home page.  Users who wish to remove this
     * branding information should contact the copyright owner to obtain
     * a Rebranding License.
     *
     * DEFINITION OF TERMS
     *
     * The "Software" refers to Maia Mailguard, including all of the
     * associated PHP, Perl, and SQL scripts, documentation files, graphic
     * icons and logo images.
     *
     * GRANT OF LICENSE
     *
     * Redistribution and use in source and binary forms, with or without
     * modification, are permitted provided that the following conditions
     * are met:
     *
     * 1. Redistributions of source code must retain the above copyright
     *    notice, this list of conditions and the following disclaimer.
     *
     * 2. Redistributions in binary form must reproduce the above copyright
     *    notice, this list of conditions and the following disclaimer in the
     *    documentation and/or other materials provided with the distribution.
     *
     * 3. The end-user documentation included with the redistribution, if
     *    any, must include the following acknowledgment:
     *
     *    "This product includes software developed by Robert LeBlanc
     *    <rjl@renaissoft.com>."
     *
     *    Alternately, this acknowledgment may appear in the software itself,
     *    if and wherever such third-party acknowledgments normally appear.
     *
     * 4. At least one of the following branding conventions must be used:
     *
     *    a. The Maia Mailguard logo appears in the page-top banner of
     *       all HTML output pages in an unmodified form, and links
     *       directly to the Maia Mailguard home page; or
     *
     *    b. The "Powered by Maia Mailguard" graphic appears in the HTML
     *       output of all gateway pages that lead to this software,
     *       linking directly to the Maia Mailguard home page; or
     *
     *    c. A separate Rebranding License is obtained from the copyright
     *       owner, exempting the Licensee from 4(a) and 4(b), subject to
     *       the additional conditions laid out in that license document.
     *
     * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS
     * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
     * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
     * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
     * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
     * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
     * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
     * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
     * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
     * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
     * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
     *
     */

   require_once ("config.php");
   require_once ("db.php");


   /*
    * unbuffered_fwrite(): Ensure that for PHP versions < 4.3.0 the
    *                      output buffer is flushed after every
    *                      fwrite() call.
    */
   function unbuffered_fwrite($handle, $output)
   {
      $rvalue = fwrite($handle, $output);
      if (phpversion() < "4.3.0") {
         fflush($handle);
      }
      return ($rvalue);
   }


   /*
    * smtp_send(): Send an e-mail (with existing headers) to an SMTP
    *              server for delivery.  Returns the server's most
    *              recent response string, prefixed by the SMTP
    *              response code.  Normal exit codes are 2xx or 3xx,
    *              while 5xx signals an error.
    */
   function smtp_send($sender, $recips, $body)
   {
      global $dbh;
      global $lang_error_connect;
      global $lang_error_helo;
      global $lang_error_mail;
      global $lang_error_rcpt;
      global $lang_error_data;
      global $lang_error_body;
      global $lang_error_quit;

      $select = "SELECT smtp_server, smtp_port FROM maia_config WHERE id = 0";
      $sth = $dbh->query($select);
      if ($row = $sth->fetchrow()) {
          $smtp_server = $row["smtp_server"];
          $smtp_port = $row["smtp_port"];
      }
      $sth->free();

      // Try to connect to the SMTP server.  Typically this will be the
      // MTA-TX in a dual-MTA arrangement.
      if (($connect = fsockopen($smtp_server, $smtp_port, $errno, $errstr, 30)) === false) {
         return ("550 " . $lang_error_connect);
      }

      // Make sure the socket is in blocking mode, and that all fwrite()
      // commands are unbuffered (i.e. immediately flushed).
      if (phpversion() < "4.3.0") {
         socket_set_blocking($connect, true);
      } else {
         stream_set_blocking($connect, true);
         stream_set_write_buffer($connect, 0);
      }

      // Read the connection greeting from the server
      if (($rcv = fgets($connect, 1024)) === false || !strncmp($rcv, "5", 1)) {
         return ($rcv);
      }

      // Send the HELO string
      if (unbuffered_fwrite($connect, "HELO {$_SERVER['SERVER_NAME']}\r\n") === false) {
         return ("550 " . $lang_error_helo);
      }

      // Receive the server's acknowledgement message
      if (($rcv = fgets($connect, 1024)) === false || !strncmp($rcv, "5", 1)) {
         return ($rcv);
      }

      // Send the envelope sender
      if (unbuffered_fwrite($connect, "MAIL FROM:<" . $sender . ">\r\n") === false) {
         return ("550 " . $lang_error_mail);
      }

      // Receive the server's acknowledgement message
      if (($rcv = fgets($connect, 1024)) === false || !strncmp($rcv, "5", 1)) {
         return ($rcv);
      }

      // Send the envelope recipients
      $recip_array = explode(" ", $recips);
      foreach ($recip_array as $recipient) {
         if (unbuffered_fwrite($connect, "RCPT TO:<" . $recipient . ">\r\n") === false) {
            return ("550 " . $lang_error_rcpt);
         }
      }

      // Receive the server's acknowledgement message
      if (($rcv = fgets($connect, 1024)) === false || !strncmp($rcv, "5", 1)) {
         return ($rcv);
      }

      // Tell the server we're about to send the actual e-mail
      if (unbuffered_fwrite($connect, "DATA\r\n") === false) {
         return ("550 " . $lang_error_data);
      }

      // Receive the server's acknowledgement message
      if (($rcv = fgets($connect, 1024)) === false || !strncmp($rcv, "5", 1)) {
         return ($rcv);
      }

      // Send the message body (including existing headers)
      if (unbuffered_fwrite($connect, $body . "\r\n") === false) {
         return ("550 " . $lang_error_body);
      }

      // Send '.' on a line by itself to signal the end of the message
      if (unbuffered_fwrite($connect, ".\r\n") === false) {
         return ("550 " . $lang_error_body);
      }

      // Receive the server's acknowledgement message
      if (($rcv = fgets($connect, 1024)) === false || !strncmp($rcv, "5", 1)) {
      	 return ($rcv);
      }

      // Tell the server we're done
      if (unbuffered_fwrite($connect, "QUIT\r\n") === false) {
         return ("550 " . $lang_error_quit);
      }

      // Receive the server's acknowledgement message
      if (($rcv = fgets($connect, 1024)) === false || !strncmp($rcv, "5", 1)) {
         return ($rcv);
      }

      // Disconnect from the server
      fclose($connect);

      return ($rcv);
   }
?>