#!/usr/bin/perl -w

# $Id: configtest.pl,v 1.2 2004/04/07 05:45:36 rjl Exp $

########################################################################
# MAIA MAILGUARD LICENSE v.1.0
#
# Copyright 2004 by Robert LeBlanc <rjl@renaissoft.com>
# All rights reserved.
#
# PREAMBLE
#
# This License is designed for users of Maia Mailguard
# ("the Software") who wish to support the Maia Mailguard project by
# leaving "Maia Mailguard" branding information in the HTML output
# of the pages generated by the Software, and providing links back
# to the Maia Mailguard home page.  Users who wish to remove this
# branding information should contact the copyright owner to obtain
# a Rebranding License.
#
# DEFINITION OF TERMS
#
# The "Software" refers to Maia Mailguard, including all of the
# associated PHP, Perl, and SQL scripts, documentation files, graphic
# icons and logo images.
#
# GRANT OF LICENSE
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. The end-user documentation included with the redistribution, if
#    any, must include the following acknowledgment:
#
#    "This product includes software developed by Robert LeBlanc
#    <rjl@renaissoft.com>."
#
#    Alternately, this acknowledgment may appear in the software itself,
#    if and wherever such third-party acknowledgments normally appear.
#
# 4. At least one of the following branding conventions must be used:
#
#    a. The Maia Mailguard logo appears in the page-top banner of
#       all HTML output pages in an unmodified form, and links
#       directly to the Maia Mailguard home page; or
#
#    b. The "Powered by Maia Mailguard" graphic appears in the HTML
#       output of all gateway pages that lead to this software,
#       linking directly to the Maia Mailguard home page; or
#
#    c. A separate Rebranding License is obtained from the copyright
#       owner, exempting the Licensee from 4(a) and 4(b), subject to
#       the additional conditions laid out in that license document.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
########################################################################

    use strict;
    use DBI;

    # CONFIGURE THIS: Location of your database.cfg file
    my $cfg = "/var/amavisd/maia/scripts/database.cfg";

########################################################################
# End of user-configurable portion.  There should be no need to modify #
# anything below this point.                                           #
########################################################################

    # Retrieve the string value associated with a key in the database.cfg file.
    sub get_string_key($$)
    {
       my ($file, $key) = @_;

       if ($file =~ /\n[ \t]*$key[ \t]*=[ \t]*\"(.*)\"/) {
          return ($1);
       } else {
          die ("Maia: [get_string_key] Unable to find $key value in $file\n");
       }
    }


    # Returns the version number of the named module, or undef if the module
    # is not installed.
    sub get_module_version($) {
        my($module) = @_;

        my $version = `$^X -m$module -e 'print \$${module}::VERSION' 2>/dev/null`;
        $version =~ s/^\s*(.*?)\s*$/$1/;
        return ($version || undef);
    }


    # Prints a formatted text message.
    sub print_message($$$) {
        my($module, $version, $message) = @_;

        printf("%-20s : %8s : %s\n", $module, $version, $message);
    }


    my $module;
    my $message;


    print "\n\namavisd-new:\n\n";

    # file(1)
    my $min_file_version = "4.06";
    my $file_version = `file -v 2>/dev/null`;
    $file_version = $1 if $file_version =~ /\-([0-9\.]+)/si;
    if ($file_version < $min_file_version) {
        $message = "UPGRADE RECOMMENDED (ftp://ftp.astron.com/pub/file/)";
    } else {
        $message = "OK";
    }
    print_message("file(1)", $file_version, $message);

    # Archive::Tar
    $module = "Archive::Tar";
    my $tar_version = get_module_version($module);
    my $have_tar = (defined $tar_version);
    if ($have_tar) {
        print_message($module, $tar_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Archive::Zip
    $module = "Archive::Zip";
    my $min_zip_version = "1.09";
    my $zip_version = get_module_version($module);
    my $have_zip = (defined $zip_version);
    if ($have_zip) {
    	if ($zip_version < $min_zip_version) {
    	    $message = "UPGRADE RECOMMENDED";
    	} else {
    	    $message = "OK";
    	}
        print_message($module, $zip_version, $message);
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Compress::Zlib
    $module = "Compress::Zlib";
    my $zlib_version = get_module_version($module);
    my $have_zlib = (defined $zlib_version);
    if ($have_zlib) {
        print_message($module, $zlib_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Convert::TNEF
    $module = "Convert::TNEF";
    my $tnef_version = get_module_version($module);
    my $have_tnef = (defined $tnef_version);
    if ($have_tnef) {
        print_message($module, $tnef_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Convert::UUlib
    $module = "Convert::UUlib";
    my $uulib_version = get_module_version($module);
    my $have_uulib = (defined $uulib_version);
    if ($have_uulib) {
        print_message($module, $uulib_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # MIME::Base64
    $module = "MIME::Base64";
   my $base64_version = get_module_version($module);
    my $have_base64 = (defined $base64_version);
    if ($have_base64) {
        print_message($module, $base64_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # MIME::Parser
    $module = "MIME::Parser";
    my $min_mime_version = "5.411";
    my $mime_version = get_module_version($module);
    my $have_mime = (defined $mime_version);
    if ($have_mime) {
    	if ($mime_version < $min_mime_version) {
    	    $message = "UPGRADE RECOMMENDED";
    	} else {
    	    $message = "OK";
    	}
        print_message($module, $mime_version, $message);
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Mail::Internet
    $module = "Mail::Internet";
    my $min_mail_version = "1.58";
    my $mail_version = get_module_version($module);
    my $have_mail = (defined $mail_version);
    if ($have_mail) {
    	if ($mail_version < $min_mail_version) {
    	    $message = "UPGRADE RECOMMENDED";
    	} else {
    	    $message = "OK";
    	}
        print_message($module, $mail_version, $message);
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Net::Server
    $module = "Net::Server";
    my $server_version = get_module_version($module);
    my $have_server = (defined $server_version);
    if ($have_server) {
        print_message($module, $server_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Net::SMTP
    $module = "Net::SMTP";
    my $smtp_version = get_module_version($module);
    my $have_smtp = (defined $smtp_version);
    if ($have_smtp) {
        print_message($module, $smtp_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Digest::MD5
    $module = "Digest::MD5";
    my $md5_version = get_module_version($module);
    my $have_md5 = (defined $md5_version);
    if ($have_md5) {
        print_message($module, $md5_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # IO::Stringy
    $module = "IO::Stringy";
    my $stringy_version = get_module_version($module);
    my $have_stringy = (defined $stringy_version);
    if ($have_stringy) {
        print_message($module, $stringy_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Time::HiRes
    $module = "Time::HiRes";
    my $hires_version = get_module_version($module);
    my $have_hires = (defined $hires_version);
    if ($have_hires) {
        print_message($module, $hires_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Unix::Syslog
    $module = "Unix::Syslog";
    my $syslog_version = get_module_version($module);
    my $have_syslog = (defined $syslog_version);
    if ($have_syslog) {
        print_message($module, $syslog_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # DBI
    $module = "DBI";
    my $min_dbi_version = "1.40";
    my $dbi_version = get_module_version($module);
    my $have_dbi = (defined $dbi_version);
    if ($have_dbi) {
    	if ($dbi_version < $min_dbi_version) {
    	    $message = "UPGRADE REQUIRED";
    	} else {
    	    $message = "OK";
    	}
        print_message($module, $dbi_version, $message);
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # DBD::mysql
    $module = "DBD::mysql";
    my $min_dbd_mysql_version = "2.9003";
    my $dbd_mysql_version = get_module_version($module);
    my $have_dbd_mysql = (defined $dbd_mysql_version);
    if ($have_dbd_mysql) {
    	if ($dbd_mysql_version < $min_dbd_mysql_version) {
    	    $message = "UPGRADE REQUIRED (if you want to use MySQL)";
    	} else {
    	    $message = "OK";
    	}
        print_message($module, $dbd_mysql_version, $message);
    } else {
    	print_message($module, "N/A", "NOT INSTALLED (required if you want to use MySQL)");
    }

    # DBD::Pg
    $module = "DBD::Pg";
    my $min_dbd_pg_version = "1.31";
    my $dbd_pg_version = get_module_version($module);
    my $have_dbd_pg = (defined $dbd_pg_version);
    if ($have_dbd_pg) {
        if ($dbd_pg_version < $min_dbd_pg_version) {
            $message = "UPGRADE RECOMMENDED (if you want to use PostgreSQL)";
        } else {
            $message = "OK";
        }
        print_message($module, $dbd_pg_version, $message);
    } else {
    	print_message($module, "N/A", "NOT INSTALLED (required if you want to use PostgreSQL)");
    }

    print "\n\nSpamAssassin:\n\n";

    # Mail::SpamAssassin
    $module = "Mail::SpamAssassin";
    my $min_sa_version = "2.55";
    my $sa_version = get_module_version($module);
    my $have_sa = (defined $sa_version);
    if ($have_sa) {
        if ($sa_version < $min_sa_version) {
            $message = "UPGRADE RECOMMENDED";
        } else {
            $message = "OK";
        }
        print_message($module, $sa_version, $message);
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # File::Spec
    $module = "File::Spec";
    my $min_spec_version = "0.80";
    my $spec_version = get_module_version($module);
    my $have_spec = (defined $spec_version);
    if ($have_spec) {
    	if ($spec_version < $min_spec_version) {
    	    $message = "UPGRADE RECOMMENDED";
    	} else {
    	    $message = "OK";
    	}
        print_message($module, $spec_version, $message);
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Pod::Usage
    $module = "Pod::Usage";
    my $min_usage_version = "1.10";
    my $usage_version = get_module_version($module);
    my $have_usage = (defined $usage_version);
    if ($have_usage) {
    	if ($usage_version < $min_usage_version) {
    	    $message = "UPGRADE RECOMMENDED";
    	} else {
    	    $message = "OK";
    	}
        print_message($module, $usage_version, $message);
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # HTML::Parser
    $module = "HTML::Parser";
    my $min_html_parser_version = "3.29";
    my $html_parser_version = get_module_version($module);
    my $have_html_parser = (defined $html_parser_version);
    if ($have_html_parser) {
    	if ($html_parser_version < $min_html_parser_version) {
    	    $message = "UPGRADE RECOMMENDED";
    	} else {
    	    $message = "OK";
    	}
        print_message($module, $html_parser_version, $message);
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # DB_File
    $module = "DB_File";
    my $db_file_version = get_module_version($module);
    my $have_db_file = (defined $db_file_version);
    if ($have_db_file) {
        print_message($module, $db_file_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Net::DNS
    $module = "Net::DNS";
    my $dns_version = get_module_version($module);
    my $have_dns = (defined $dns_version);
    if ($have_dns) {
        print_message($module, $dns_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Digest::SHA1
    $module = "Digest::SHA1";
    my $sha1_version = get_module_version($module);
    my $have_sha1 = (defined $sha1_version);
    if ($have_sha1) {
        print_message($module, $sha1_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    print "\n\nMaia Mailguard:\n\n";

    # Crypt::Blowfish
    $module = "Crypt::Blowfish";
    my $blowfish_version = get_module_version($module);
    my $have_blowfish = (defined $blowfish_version);
    if ($have_blowfish) {
        print_message($module, $blowfish_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    # Crypt::CBC
    $module = "Crypt::CBC";
    my $cbc_version = get_module_version($module);
    my $have_cbc = (defined $cbc_version);
    if ($have_cbc) {
        print_message($module, $cbc_version, "OK");
    } else {
    	print_message($module, "N/A", "NOT INSTALLED");
    }

    print("\n");

    if ($have_dbi && ($have_dbd_mysql || $have_dbd_pg)) {

        # Read the database configuration file into memory once
        open CONFIGFILE, "<" . $cfg
            or die ("Maia: [configtest] Unable to open $cfg\n");
        my($config) = "";
        my $line;
        while ($line = <CONFIGFILE>) {
            $config .= $line;
        }
        close CONFIGFILE;

        # Connect to the database
        my $dsn = get_string_key($config, "dsn");
        my $username = get_string_key($config, "username");
        my $password = get_string_key($config, "password");
        my $dbh = DBI->connect($dsn, $username, $password)
                       or die("Can't connect to SQL database");

        print("Database DSN test    : PASSED\n");

    } else {
    	print("Database tests SKIPPED (missing DBI and/or DBD modules)\n");
    }

    print "\n";

    exit;