#!/usr/bin/perl -T

# $Id: load-sa-rules.pl,v 1.1.1.1 2004/01/21 15:54:43 rjl Exp $

########################################################################
# MAIA MAILGUARD LICENSE v.1.0
#
# Copyright 2004 by Robert LeBlanc <rjl@renaissoft.com>
# All rights reserved.
#
# PREAMBLE
#
# This License is designed for users of Maia Mailguard
# ("the Software") who wish to support the Maia Mailguard project by
# leaving "Maia Mailguard" branding information in the HTML output
# of the pages generated by the Software, and providing links back
# to the Maia Mailguard home page.  Users who wish to remove this
# branding information should contact the copyright owner to obtain
# a Rebranding License.
#
# DEFINITION OF TERMS
#
# The "Software" refers to Maia Mailguard, including all of the
# associated PHP, Perl, and SQL scripts, documentation files, graphic
# icons and logo images.
#
# GRANT OF LICENSE
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. The end-user documentation included with the redistribution, if
#    any, must include the following acknowledgment:
#
#    "This product includes software developed by Robert LeBlanc
#    <rjl@renaissoft.com>."
#
#    Alternately, this acknowledgment may appear in the software itself,
#    if and wherever such third-party acknowledgments normally appear.
#
# 4. At least one of the following branding conventions must be used:
#
#    a. The Maia Mailguard logo appears in the page-top banner of
#       all HTML output pages in an unmodified form, and links
#       directly to the Maia Mailguard home page; or
#
#    b. The "Powered by Maia Mailguard" graphic appears in the HTML
#       output of all gateway pages that lead to this software,
#       linking directly to the Maia Mailguard home page; or
#
#    c. A separate Rebranding License is obtained from the copyright
#       owner, exempting the Licensee from 4(a) and 4(b), subject to
#       the additional conditions laid out in that license document.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
########################################################################

    use strict;
    use DBI;
    use Mail::SpamAssassin;

    # CONFIGURE THIS: SpamAssassin directories to search for rules
    #                 files (*.cf and user_prefs)
    my $local_cf_dir = "/etc/mail/spamassassin";
    my $system_rules_dir = "/usr/local/share/spamassassin";
    my $user_rules_dir = "/var/amavisd/.spamassassin";

    # CONFIGURE THIS: Location of your database.cfg file
    my $cfg = "/var/amavisd/maia/scripts/database.cfg";

########################################################################
# End of user-configurable portion.  There should be no need to modify #
# anything below this point.                                           #
########################################################################

    # Retrieve the string value associated with a key in the database.cfg file.
    sub get_string_key($$)
    {
       my ($file, $key) = @_;

       if ($file =~ /\n[ \t]*$key[ \t]*=[ \t]*\"(.*)\"/) {
          return ($1);
       } else {
          die ("Maia: [get_string_key] Unable to find $key value in $file\n");
       }
    }


    # Scan all the *.cf and user_prefs files in a subdirectory,
    # looking for SpamAssassin rules.
    sub scan_rules_directory($$) {
    	my($dbh, $dir) = @_;
        my(@file_list) = glob($dir . "/*");
        my $file;
        my $rules_added = 0;
        my $rules_skipped = 0;
        my($select, $insert, $update, $sth, $sth2, $line, @row);

        print "Scanning " . $dir . " for SpamAssassin rules\n";

        # Scan each file in the directory for "description" strings,
        # which map rule names to text explanations.  Each of these
        # represents a SpamAssassin rule.  If this rule doesn't
        # already exist in the database, insert it, with a default
        # score of 1.0 (per SpamAssassin's documentation).
        foreach $file (@file_list)
        {
            if ($file =~ /^(.+\.cf|.*user_prefs)$/si) {
                print "$file\n";
                open RULEFILE, "<" . $file
                    or die ("Maia: [load-sa-rules] Couldn't read $file\n");
                $select = "SELECT id FROM maia_sa_rules WHERE rule_name LIKE ?";
                $sth = $dbh->prepare($select)
                    or die (sprintf("Maia: [load-sa-rules] Couldn't prepare query: %s", $dbh->errstr));
                my($default_score, $rule_name, $rule_description);
                while ($line = <RULEFILE>) {
                   if ($line =~ /^describe\s*([A-Za-z0-9_]+)[\s\t]*(.*)\s*\n$/si) {
                       $sth->execute($1)
                           or die (sprintf("Maia: [load-sa-rules] Couldn't execute query: %s", $dbh->errstr));
                       if (!(@row = $sth->fetchrow_array())) {
                       	   $rule_name = $1;
                       	   $rule_description = $2;
               	           print "Adding new rule: $rule_name ($rule_description)\n";
               	           if ($rule_name =~ /^T_.+$/) { # test rule
               	               $default_score = 0.01;
               	           } elsif ($rule_name =~ /^__.+$/) { # meta-rule
               	               $default_score = 0.00;
               	           } else {
               	               $default_score = 1.00;
               	           }
                           $insert = "INSERT INTO maia_sa_rules (rule_name, rule_description, rule_score_0, " .
                                        "rule_score_1, rule_score_2, rule_score_3) " .
                                        "VALUES (?, ?, ?, ?, ?, ?)";
                           $sth2 = $dbh->prepare($insert)
                               or die (sprintf("Maia: [load-sa-rules] Couldn't prepare query: %s", $dbh->errstr));
                           $sth2->execute($rule_name, $rule_description, $default_score,
                                          $default_score, $default_score, $default_score)
                               or die (sprintf("Maia: [load-sa-rules] Couldn't execute query: %s", $dbh->errstr));
                           $rules_added++;
                       } else {
                           print "Skipping existing rule: $1\n";
                           $rules_skipped++;
                       }
                   }
                }
                $sth->finish;
                close RULEFILE;
            }
        }

        # Scan each file in the directory for "score" strings,
        # which map rule names to numeric scores.  Any rule
        # without an explicit score retains its default score
        # of 1.0, per the SpamAssassin documentation.
        foreach $file (@file_list)
        {
            if ($file =~ /^(.+\.cf|.*user_prefs)$/si) {
    	        open SCOREFILE, "<" . $file
                    or die ("Maia: [load-sa-rules] Unable to open $file\n");
                $select = "SELECT id FROM maia_sa_rules WHERE rule_name LIKE ?";
                $sth = $dbh->prepare($select)
                    or die (sprintf("Maia: [load-sa-rules] Couldn't prepare query: %s", $dbh->errstr));
                $update = "UPDATE maia_sa_rules SET rule_score_0 = ?, " .
                                                   "rule_score_1 = ?, " .
                                                   "rule_score_2 = ?, " .
                                                   "rule_score_3 = ? " .
                          "WHERE id = ?";
                $sth2 = $dbh->prepare($update)
                    or die (sprintf("Maia: [load-sa-rules] Couldn't prepare query: %s", $dbh->errstr));
                my(@score, $rule_id);
                while ($line = <SCOREFILE>) {

            	    # Scores for all four rulesets explicitly provided, e.g.
            	    # score RULE_NAME 0 1 2 3
                    if ($line =~ /^score\s+([A-Za-z0-9_]+)[\s\t]+([0-9\-\.]+)[\s\t]+([0-9\-\.]+)[\s\t]+([0-9\-\.]+)[\s\t]+([0-9\-\.]+)[\s\t]*\n$/si) {
            	        $sth->execute($1)
                            or die (sprintf("Maia: [load-sa-rules] Couldn't execute query: %s", $dbh->errstr));
                        if (@row = $sth->fetchrow_array()) {
                            $score[0] = $2;
                            $score[1] = $3;
                            $score[2] = $4;
                            $score[3] = $5;
                            $rule_id = $1 if $row[0] =~ /^([1-9]+[0-9]*)$/si; # untaint
                            $sth2->execute($score[0], $score[1], $score[2], $score[3], $rule_id)
                                or die (sprintf("Maia: [load-sa-rules] Couldn't execute query: %s", $dbh->errstr));
                        }

                    # Scores for three rulesets explicitly provided, so set
                    # the fourth ruleset score to the value of the third, per
                    # the SpamAssassin documentation.
                    # score RULE_NAME 1 2 3
                    } elsif ($line =~ /^score\s+([A-Za-z0-9_]+)[\s\t]+([0-9\-\.]+)[\s\t]+([0-9\-\.]+)[\s\t]+([0-9\-\.]+)[\s\t]*\n$/si) {
            	        $sth->execute($1)
                            or die (sprintf("Maia: [load-sa-rules] Couldn't execute query: %s", $dbh->errstr));
                        if (@row = $sth->fetchrow_array()) {
                            $score[0] = $2;
                            $score[1] = $3;
                            $score[2] = $4;
                            $score[3] = $4;
                            $rule_id = $1 if $row[0] =~ /^([1-9]+[0-9]*)$/si; # untaint
                            $sth2->execute($score[0], $score[1], $score[2], $score[3], $rule_id)
                                or die (sprintf("Maia: [load-sa-rules] Couldn't execute query: %s", $dbh->errstr));
                        }

                    # Scores for two rulesets explicitly provided, so set
                    # the third and fourth scores to the value of the second,
                    # per the SpamAssassin documentation.
                    # score RULE_NAME 1 2
                    } elsif ($line =~ /^score\s+([A-Za-z0-9_]+)[\s\t]+([0-9\-\.]+)[\s\t]+([0-9\-\.]+)[\s\t]*\n$/si) {
            	        $sth->execute($1)
                            or die (sprintf("Maia: [load-sa-rules] Couldn't execute query: %s", $dbh->errstr));
                        if (@row = $sth->fetchrow_array()) {
                            $score[0] = $2;
                            $score[1] = $3;
                            $score[2] = $3;
                            $score[3] = $3;
                            $rule_id = $1 if $row[0] =~ /^([1-9]+[0-9]*)$/si; # untaint
                            $sth2->execute($score[0], $score[1], $score[2], $score[3], $rule_id)
                                or die (sprintf("Maia: [load-sa-rules] Couldn't execute query: %s", $dbh->errstr));
                        }

                    # Scores for only one ruleset explicitly provided, so set
                    # the second, third, and fourth scores to the value of the
                    # first, per the SpamAssassin documentation.
                    # score RULE_NAME 1
                    } elsif ($line =~ /^score\s+([A-Za-z0-9_]+)[\s\t]+([0-9\-\.]+)[\s\t]*\n$/si) {
             	        $sth->execute($1)
                            or die (sprintf("Maia: [load-sa-rules] Couldn't execute query: %s", $dbh->errstr));
                        if (@row = $sth->fetchrow_array()) {
                            $score[0] = $2;
                            $score[1] = $2;
                            $score[2] = $2;
                            $score[3] = $2;
                            $rule_id = $1 if $row[0] =~ /^([1-9]+[0-9]*)$/si; # untaint
                            $sth2->execute($score[0], $score[1], $score[2], $score[3], $rule_id)
                                or die (sprintf("Maia: [load-sa-rules] Couldn't execute query: %s", $dbh->errstr));
                        }
                    }
                }
                $sth->finish;
                close SCOREFILE;
            }
        }

        return $rules_added;
    }


    # Read the database configuration file into memory once
    open CONFIGFILE, "<" . $cfg
        or die ("Maia: [load-sa-rules] Unable to open $cfg\n");
    my($config) = "";
    my $line;
    while ($line = <CONFIGFILE>) {
        $config .= $line;
    }
    close CONFIGFILE;

    # Connect to the database
    my $dsn = get_string_key($config, "dsn");
    my $username = get_string_key($config, "username");
    my $password = get_string_key($config, "password");
    my $dbh = DBI->connect($dsn, $username, $password)
                       or die("Can't connect to SQL database");

    # Scan the rules directories in this specific order:
    #
    #    1. Default rules (e.g. /usr/local/share/spamassassin)
    #    2. System rules (e.g. /etc/mail/spamassassin)
    #    3. User rules (e.g. /var/amavisd/.spamassassin)
    #
    # The order is critical, since later rules override
    # the scores of earlier ones (e.g. a user rule could
    # assign a score of 0 to a rule to disable it, etc.).
    my $count = 0;
    $count += scan_rules_directory($dbh, $system_rules_dir);
    $count += scan_rules_directory($dbh, $local_cf_dir);
    $count += scan_rules_directory($dbh, $user_rules_dir);

    print "$count new rules added, all scores updated.\n";

    # Disconnect from the database
    $dbh->disconnect;

    # We're done.
    exit;